@@ -77,11 +77,26 @@ myMessageOutputFormat(QtMsgType type,
   QByteArray localMsg  = msg.toLocal8Bit();
   const char *file     = context.file ? context.file : "";
   const char *function = context.function ? context.function : "";
+
+  // The file char arrray contains the full path to the file name.
+  // We want to only have the basename of the file.
+
+  QString file_path(file);
+  QFileInfo file_info(file_path);
+  QString file_name                     = file_info.fileName();
+  const QByteArray file_name_char_array = file_name.toUtf8();
+  char file_name_c_array[200];
+  file_name_c_array[qMin(99, file_name_char_array.size())] = '\0';
+  std::copy(file_name_char_array.constBegin(),
+            file_name_char_array.constBegin() +
+              qMin(99, file_name_char_array.size()),
+            file_name_c_array);
+
   switch(type)
     {
       case QtDebugMsg:
         fprintf(stderr,
-                "Debug: %s@%u -- %s() %s\n",
+                "\t\t\tDebug: %s:%u\n\t\t%s\n%s\n",
                 file,
                 context.line,
                 function,
@@ -89,35 +104,35 @@ myMessageOutputFormat(QtMsgType type,
         break;
       case QtInfoMsg:
         fprintf(stderr,
-                "Info: %s (%s:%u, %s)\n",
-                localMsg.constData(),
+                "\t\t\tInfo: %s:%u\n\t\t%s\n%s\n",
                 file,
                 context.line,
-                function);
+                function,
+                localMsg.constData());
         break;
       case QtWarningMsg:
         fprintf(stderr,
-                "Warning: %s (%s:%u, %s)\n",
-                localMsg.constData(),
+                "\t\t\tWarning %s:%u\n\t\t%s\n%s\n",
                 file,
                 context.line,
-                function);
+                function,
+                localMsg.constData());
         break;
       case QtCriticalMsg:
         fprintf(stderr,
-                "Critical: %s (%s:%u, %s)\n",
-                localMsg.constData(),
+                "\t\t\tCritical %s:%u\n\t\t%s\n%s\n",
                 file,
                 context.line,
-                function);
+                function,
+                localMsg.constData());
         break;
       case QtFatalMsg:
         fprintf(stderr,
-                "Fatal: %s (%s:%u, %s)\n",
-                localMsg.constData(),
+                "\t\t\tFatal: %s:%u\n\t\t%s\n%s\n",
                 file,
                 context.line,
-                function);
+                function,
+                localMsg.constData());
         break;
     }
 }
@@ -151,7 +166,7 @@ Application::Application(int &argc, char **argv, const QString &moduleName)
   setWindowIcon(icon);
 
   // Set the debugging message formatting pattern.
-  // qSetMessagePattern(QString("Debug: %{file}@%{line}-%{function}():
+  // qSetMessagePattern(QString("Debug: %{file}:%{line}-%{function}():
   // %{message}"));
   qInstallMessageHandler(myMessageOutputFormat);
 
