@@ -1,7 +1,7 @@
 /* BEGIN software license
  *
  * msXpertSuite - mass spectrometry software suite
- * -----------------------------------------------
+:* -----------------------------------------------
  * Copyright (C) 2009--2020 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
@@ -36,6 +36,8 @@
 
 /////////////////////// Qt includes
 #include <QMessageBox>
+#include <QInputDialog>
+
 
 /////////////////////// QCustomPlot
 
@@ -86,7 +88,7 @@ BaseColorMapPlotCompositeWidget::~BaseColorMapPlotCompositeWidget()
 void
 BaseColorMapPlotCompositeWidget::createMainMenu()
 {
-  //qDebug() << "ENTER";
+  // qDebug() << "ENTER";
 
   // First create the base class menu.
   BasePlotCompositeWidget::createMainMenu();
@@ -95,6 +97,8 @@ BaseColorMapPlotCompositeWidget::createMainMenu()
   // integrations. We do not support the stuffing more than one color map into a
   // color map plot, contrary to what we do with graphs.
 
+  mp_mainMenu->addSeparator();
+
   // An now append new ColorMap-specific menu
 
   // The axes transposition.
@@ -107,6 +111,8 @@ BaseColorMapPlotCompositeWidget::createMainMenu()
 
   mp_mainMenu->addSeparator();
 
+  mp_mainMenu->addAction(transpose_axes_p);
+
   QMenu *z_axis_scales_menu_p = mp_mainMenu->addMenu("Z axis scales");
 
   // The z axis scale (log10 or linear).
@@ -120,18 +126,86 @@ BaseColorMapPlotCompositeWidget::createMainMenu()
 
   z_axis_scales_menu_p->addAction(z_axis_scale_to_log10_p);
 
-  // The z axis scale (log10 or linear).
-  QAction *z_axis_scale_to_original_p =
-    new QAction("Reset Z axis scale to original", this);
-  z_axis_scale_to_original_p->setStatusTip(tr("Reset Z axis scale to original"));
-  connect(z_axis_scale_to_original_p, &QAction::triggered, [this]() {
-    static_cast<pappso::BaseColorMapPlotWidget *>(mp_plotWidget)
-      ->zAxisScaleResetToOriginal();
+
+  // The low pass filter over the z axis.
+  QAction *double_spin_box_action_p =
+    new QAction("Low pass-filter Z axis data", this);
+  double_spin_box_action_p->setStatusTip(tr("Low pass-filter Z axis data"));
+  double_spin_box_action_p->setToolTip(
+    "Enter the percentage (%) of the highest intensity (i) in the map");
+
+  connect(double_spin_box_action_p, &QAction::triggered, [this]() {
+    bool ok = false;
+
+    double percentage = QInputDialog::getDouble(
+      this,
+      "Low pass filter the color map's intensities",
+      "Enter the threshold as the percentage (%) \n"
+      "of the highest intensity (i) in the map.\n"
+      "\n"
+      "The threshold (th) is computed as th = i * % / 100.\n"
+      "\n"
+      "The intensity values below th are kept unchanged.\n"
+      "\n"
+      "The intensity values above th are replaced with the th value.",
+      90,
+      0,
+      100,
+      1,
+      &ok);
+
+    if(ok)
+      static_cast<pappso::BaseColorMapPlotWidget *>(mp_plotWidget)
+        ->zAxisFilterLowPassPercentage(percentage);
   });
 
-  z_axis_scales_menu_p->addAction(z_axis_scale_to_original_p);
+  z_axis_scales_menu_p->addAction(double_spin_box_action_p);
 
 
+  // The high pass filter over the z axis.
+  double_spin_box_action_p = new QAction("High pass-filter Z axis data", this);
+  double_spin_box_action_p->setStatusTip(tr("High pass-filter Z axis data"));
+  double_spin_box_action_p->setToolTip(
+    "Enter the percentage (%) of the highest intensity (i) in the map");
+
+  connect(double_spin_box_action_p, &QAction::triggered, [this]() {
+    bool ok = false;
+
+    double percentage = QInputDialog::getDouble(
+      this,
+      "High pass filter the color map's intensities",
+      "Enter the threshold as the percentage (%) \n"
+      "of the highest intensity (i) in the map.\n"
+      "\n"
+      "The threshold (th) is computed as th = i * % / 100.\n"
+      "\n"
+      "The intensity values above th are kept unchanged.\n"
+      "\n"
+      "The intensity values below th are replaced with the th value.",
+      10,
+      0,
+      100,
+      1,
+      &ok);
+
+    if(ok)
+      static_cast<pappso::BaseColorMapPlotWidget *>(mp_plotWidget)
+        ->zAxisFilterHighPassPercentage(percentage);
+  });
+
+  z_axis_scales_menu_p->addAction(double_spin_box_action_p);
+
+  // The z axis scale (log10 or linear, that is, original).
+  QAction *z_axis_data_to_original_p =
+    new QAction("Reset Z axis data to original", this);
+  z_axis_data_to_original_p->setStatusTip(tr("Reset Z axis data to original"));
+  connect(z_axis_data_to_original_p, &QAction::triggered, [this]() {
+    static_cast<pappso::BaseColorMapPlotWidget *>(mp_plotWidget)
+      ->zAxisDataResetToOriginal();
+  });
+
+  z_axis_scales_menu_p->addAction(z_axis_data_to_original_p);
+
   disconnect(m_ui.mainMenuPushButton,
              &QPushButton::clicked,
              static_cast<BasePlotCompositeWidget *>(this),
@@ -144,25 +218,25 @@ BaseColorMapPlotCompositeWidget::createMainMenu()
           this,
           &BaseColorMapPlotCompositeWidget::mainMenuPushButtonClicked);
 
-  //qDebug() << "EXIT";
+  // qDebug() << "EXIT";
 }
 
 
 void
 BaseColorMapPlotCompositeWidget::mainMenuPushButtonClicked()
 {
-  //qDebug() << "ENTER";
+  // qDebug() << "ENTER";
 
   //// Create the contextual menu that will show when the button is clicked (see
   //// below).
 
   createMainMenu();
 
-  //qDebug() << "Done creating menu, now showing it.";
+  // qDebug() << "Done creating menu, now showing it.";
 
   m_ui.mainMenuPushButton->showMenu();
 
-  //qDebug() << "EXIT";
+  // qDebug() << "EXIT";
 }
 
 
