@@ -65,6 +65,45 @@ class BaseColorMapPlotWnd;
 class MsFragmentationSpecDlg;
 class MzIntegrationParamsDlg;
 
+// The QWidgetAction strategy has too many problems,
+// in particular on macOS.
+//class DoubleSpinBoxLabelAction : public QWidgetAction
+//{
+  //public:
+  //DoubleSpinBoxLabelAction(QWidget *parent_p,
+                           //const QString &label,
+                           //const QString &tooltip)
+    //: QWidgetAction(parent_p)
+  //{
+    //QWidget *widget_p     = new QWidget(parent_p);
+    //QHBoxLayout *layout_p = new QHBoxLayout();
+    //mp_label              = new QLabel(label);
+    //mp_label->setToolTip(tooltip);
+    //layout_p->addWidget(mp_label);
+    //mp_spinBox = new QDoubleSpinBox(parent_p);
+    //layout_p->addWidget(mp_spinBox);
+    //widget_p->setLayout(layout_p);
+
+    //setDefaultWidget(widget_p);
+  //}
+
+  //QDoubleSpinBox *
+  //getSpinBox()
+  //{
+    //return mp_spinBox;
+  //}
+
+  //QLabel *
+  //getLabel()
+  //{
+    //return mp_label;
+  //}
+
+  //private:
+  //QDoubleSpinBox *mp_spinBox = nullptr;
+  //QLabel *mp_label           = nullptr;
+//};
+
 
 // The BaseColorMapPlotCompositeWidget class is aimed at receiving QCPGraph_s,
 // not QCPColorMap_s. This class implements all the basic features related to
@@ -78,10 +117,9 @@ class BaseColorMapPlotCompositeWidget : public BasePlotCompositeWidget
   friend class BaseColorMapPlotWnd;
 
   public:
-  explicit BaseColorMapPlotCompositeWidget(
-    QWidget *parent,
-    const QString &x_axis_label,
-    const QString &y_axis_label);
+  explicit BaseColorMapPlotCompositeWidget(QWidget *parent,
+                                           const QString &x_axis_label,
+                                           const QString &y_axis_label);
 
   virtual ~BaseColorMapPlotCompositeWidget();
 
@@ -95,8 +133,8 @@ class BaseColorMapPlotCompositeWidget : public BasePlotCompositeWidget
               const ProcessingFlow &processing_flow,
               const QColor &color);
 
-	pappso::DataKind xAxisKind() const;
-	pappso::DataKind yAxisKind() const;
+  pappso::DataKind xAxisKind() const;
+  pappso::DataKind yAxisKind() const;
 
   public slots:
 
