@@ -477,21 +477,21 @@ BasePlotCompositeWidget::xAxisMeasurement(
 
 #if 0
 
-  // This was the previous version that only cared of X axis movements, but
-  // with the selection polygon we now want to show the full extent of the data:
-  // X axis and Y axis. 
-  QCPRange sorted_range(context.m_xRegionRangeStart, context.m_xRegionRangeEnd);
+        // This was the previous version that only cared of X axis movements, but
+        // with the selection polygon we now want to show the full extent of the data:
+        // X axis and Y axis. 
+        QCPRange sorted_range(context.m_xRegionRangeStart, context.m_xRegionRangeEnd);
 
-  QString message = QString("[%1--%2]")
-                      .arg(sorted_range.lower, 0, 'f', 3)
-                      .arg(sorted_range.upper, 0, 'f', 3);
+        QString message = QString("[%1--%2]")
+          .arg(sorted_range.lower, 0, 'f', 3)
+          .arg(sorted_range.upper, 0, 'f', 3);
 
-  if(with_delta)
-    message.append(QString(" - delta: %1").arg(context.m_xDelta, 0, 'f', 3));
+        if(with_delta)
+          message.append(QString(" - delta: %1").arg(context.m_xDelta, 0, 'f', 3));
 
-  // qDebug() << message;
+        // qDebug() << message;
 
-  m_ui.statusLineEdit->setText(message);
+        m_ui.statusLineEdit->setText(message);
 #endif
 
 
@@ -511,7 +511,7 @@ BasePlotCompositeWidget::xAxisMeasurement(
   // We want the Y delta only if the polygon is 2D.
   if(context.m_selectionPolygon.is2D() && with_delta)
     {
-      //qDebug() << "Selection polygon is 2D";
+      // qDebug() << "Selection polygon is 2D";
       context.m_selectionPolygon.rangeY(start, end);
       text += QString(" -- yDelta: %1").arg(end - start, 0, 'f', 3);
     }
@@ -894,7 +894,7 @@ BasePlotCompositeWidget::msFragmentationSpecChanged(
   // Set the default ms frag spec.
   processing_flow_p->setDefaultMsFragmentationSpec(ms_fragmentation_spec);
 
-  //qDebug() << "Now setting the MS level to the spin box.";
+  // qDebug() << "Now setting the MS level to the spin box.";
 
   // Change the MS level value in the spinbox.
   // But we need to disconnect the spin box' value changed signal, because
@@ -922,7 +922,7 @@ BasePlotCompositeWidget::msLevelValueChanged(int value)
   // The user has selected a new MS level value, we need to ensure that this
   // value is record in the MsFragmentationSpec.
 
-  //qDebug() << "Value: " << value;
+  // qDebug() << "Value: " << value;
 
   QCPAbstractPlottable *source_plottable_p =
     plottableToBeUsedAsIntegrationSource();
@@ -1024,6 +1024,22 @@ BasePlotCompositeWidget::getMzIntegrationParams(
 }
 
 
+pappso::FilterNameInterfaceCstSPtr
+BasePlotCompositeWidget::filter(const QString filter_name,
+                                const QCPAbstractPlottable *plottable_p) const
+{
+
+  // Get the processing flow for the plottable.
+  const ProcessingFlow &processing_flow =
+    getProcessingFlowForPlottable(plottable_p);
+
+  qDebug() << "The source plottable:" << plottable_p;
+  qDebug() << "The processing flow:" << &processing_flow;
+
+  return processing_flow.filter(filter_name);
+}
+
+
 void
 BasePlotCompositeWidget::showMzIntegrationParamsDlg()
 {
