@@ -51,6 +51,7 @@
 #include "MzIntegrationParamsDlg.hpp"
 #include "MsFragmentationSpecDlg.hpp"
 #include "Application.hpp"
+#include "PlotAreaOverlay.hpp"
 
 
 namespace msxps
@@ -173,7 +174,7 @@ BasePlotWnd::initialize()
   // Now that the main layout is set to the central widget, let's pack the
   // scroll area in it. Upon adding the area to the layout, that area is
   // reparented to the layout that is responsible for its destruction.
-  mp_scrollArea = new QScrollArea(mp_centralWidget);
+  mp_scrollArea = new QScrollArea(nullptr);
   mp_scrollArea->setObjectName(QStringLiteral("mp_scrollArea"));
   mp_scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
   mp_scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
@@ -183,7 +184,7 @@ BasePlotWnd::initialize()
   mp_scrollArea->setWidgetResizable(true);
   mp_scrollArea->setAlignment(Qt::AlignCenter);
 
-  mp_scrollAreaWidgetContents = new QWidget();
+  mp_scrollAreaWidgetContents = new QWidget(nullptr);
   mp_scrollAreaWidgetContents->setObjectName(
     QStringLiteral("mp_scrollAreaWidgetContents"));
   mp_scrollAreaWidgetContents->setGeometry(QRect(0, 0, 766, 525));
@@ -192,7 +193,7 @@ BasePlotWnd::initialize()
   mp_scrollAreaVBoxLayout->setObjectName(
     QStringLiteral("mp_scrollAreaVBoxLayout"));
 
-  mp_splitter = new QSplitter(Qt::Vertical, mp_centralWidget);
+  mp_splitter = new QSplitter(Qt::Vertical, mp_scrollAreaWidgetContents);
   mp_splitter->setHandleWidth(3);
 
   // We want the host of all the plot widget to be this QSplitter instance
@@ -202,7 +203,37 @@ BasePlotWnd::initialize()
 
   mp_scrollArea->setWidget(mp_scrollAreaWidgetContents);
 
-  mp_centralWidgetGridLayout->addWidget(mp_scrollArea);
+  // We want the scroll area to lie just below the vertical rule slider that
+  // we'll handle later. So set the row to 1 and the column to 0.
+  mp_centralWidgetGridLayout->addWidget(mp_scrollArea, 1, 0);
+
+  // At this point, make sure we allocate the transparent overlay widget in
+  // which we'll draw items that need to be drawn over all the widgets.
+
+  mpa_plotAreaOverlay = new PlotAreaOverlay(mp_centralWidget);
+  mpa_plotAreaOverlay->setObjectName("mpa_plotAreaOverlay");
+  mpa_plotAreaOverlay->raise();
+  mpa_plotAreaOverlay->show();
+
+  // At this point that the overlay plot has been created, we can create the
+  // slider that will govern the position of the vertical rule that span the
+  // whole height of the scroll area.
+  mp_vertRuleSlider = new QSlider(Qt::Horizontal, mp_centralWidget);
+  mp_vertRuleSlider->setSingleStep(1);
+  mp_vertRuleSlider->setPageStep(1);
+  mp_vertRuleSlider->setMinimum(1);
+  mp_vertRuleSlider->setMaximum(mp_centralWidget->size().width());
+
+  // Start hidden.
+  mp_vertRuleSlider->setVisible(m_isLockedYMarker);
+
+  // We add the ruler as the first widget from top (row 0, column 0)
+  mp_centralWidgetGridLayout->addWidget(mp_vertRuleSlider, 0, 0);
+
+  connect(mp_vertRuleSlider,
+          &QSlider::valueChanged,
+          this,
+          &BasePlotWnd::verticalRulerSliderValueChanged);
 
   readSettings();
 
@@ -210,6 +241,62 @@ BasePlotWnd::initialize()
 }
 
 
+void
+BasePlotWnd::verticalRulerSliderValueChanged(int value)
+{
+  // The user has modified the position of the slider, which means we need to
+  // update the position of the vertical rule that spans all the composite plot
+  // widgets that lie below.
+
+  // qDebug() << "The vertical ruler slide has value:" << value;
+
+  // We need the plot overlay widget to know the current value of the slider,
+  // that represents the position of the vertical ruler.
+  mpa_plotAreaOverlay->setLastHorSliderValue(value);
+
+  // Now that the new position of the rule is known, we can update the widget so
+  // that it repaints itself, thus drawing the vertical line at the right
+  // position.
+  mpa_plotAreaOverlay->update();
+}
+
+
+void
+BasePlotWnd::resizeEvent([[maybe_unused]] QResizeEvent *event)
+{
+  // qDebug();
+
+  // We way call this function without event.
+  if(event != nullptr)
+    {
+      QMainWindow::resizeEvent(event);
+    }
+
+  // We need to update the size of the vertical rule slider.
+  mp_vertRuleSlider->setMaximum(size().width());
+  // qDebug() << "The new slider max value:" << mp_vertRuleSlider->maximum();
+
+  // We need to update the size of the transparent overlay plot widget so that
+  // we can draw over the right surface: the surface of the scroll area.
+
+  // We do not want that the size of the overlay plot widget encompasses the
+  // scroll bars, so we craft a QSize object.
+
+  QSize reduced_size(mp_scrollArea->size().width() -
+                       mp_scrollArea->verticalScrollBar()->size().width(),
+                     mp_scrollArea->size().height() -
+                       mp_scrollArea->horizontalScrollBar()->size().height());
+
+  mpa_plotAreaOverlay->resize(reduced_size);
+
+  // Now move that plot overlay widget to he position of the scroll area.
+  mpa_plotAreaOverlay->move(mp_scrollArea->pos());
+
+  // At this point we need to update the vertical ruler position.
+  mpa_plotAreaOverlay->update();
+}
+
+
 void
 BasePlotWnd::setupToolBar()
 {
@@ -219,8 +306,8 @@ BasePlotWnd::setupToolBar()
   mp_toolBar = addToolBar("");
   mp_toolBar->setObjectName(QStringLiteral("mp_toolBar"));
 
-  // Make sure a right mouse button click does not elicit the context menu that
-  // when clicked made the toolbar disappear totally.
+  // Make sure a right mouse button click does not elicit the context menu
+  // that when clicked made the toolbar disappear totally.
   mp_toolBar->setContextMenuPolicy(Qt::PreventContextMenu);
 
   // Create the main menu that sits into the first button of the toolbar.
@@ -251,6 +338,20 @@ BasePlotWnd::setupToolBar()
     mp_lockYRangeAct, &QAction::toggled, this, &BasePlotWnd::lockYRangeToggle);
 
   mp_toolBar->addAction(mp_lockYRangeAct);
+
+  const QIcon yMarkerIcon = QIcon(":/images/svg/lock-y-marker.svg");
+  mp_lockYMarkerAct       = new QAction(
+    yMarkerIcon, tr("Lock multi-widget Y marker (Ctrl+L, W)"), this);
+  mp_lockYMarkerAct->setCheckable(true);
+  mp_lockYMarkerAct->setShortcut(QKeySequence("Ctrl+L, W"));
+  mp_lockYMarkerAct->setStatusTip(tr("Lock multi-widget Y marker"));
+
+  connect(mp_lockYMarkerAct,
+          &QAction::toggled,
+          this,
+          &BasePlotWnd::lockYMarkerToggle);
+
+  mp_toolBar->addAction(mp_lockYMarkerAct);
 }
 
 
@@ -531,8 +632,8 @@ void
 BasePlotWnd::mzIntegrationParamsChanged(
   pappso::MzIntegrationParams mz_integration_params)
 {
-  // We now know what are the mz integ params. We need to propagate them to all
-  // the pinned-down widgets.
+  // We now know what are the mz integ params. We need to propagate them to
+  // all the pinned-down widgets.
 
   std::vector<BasePlotCompositeWidget *> widget_list = pinnedDownWidgets();
 
@@ -569,8 +670,8 @@ BasePlotWnd::mzIntegrationParamsChanged(
           iter_mz_integration_params.setRemoveZeroValDataPoints(
             mz_integration_params.isRemoveZeroValDataPoints());
 
-          // And finally set the default mz integration params to be our updated
-          // mz integration params.
+          // And finally set the default mz integration params to be our
+          // updated mz integration params.
           processing_flow_p->setDefaultMzIntegrationParams(
             iter_mz_integration_params);
 
@@ -667,7 +768,6 @@ BasePlotWnd::show()
 void
 BasePlotWnd::savePinnedPlotsToGraphicsFile()
 {
-
   // Get a list of pinned down plot composite widgets.
 
   std::vector<BasePlotCompositeWidget *> pinned_down_widgets =
@@ -976,9 +1076,9 @@ BasePlotWnd::integrateToTicIntensity(
 
   // Make the connections
 
-  // When the MsRunReadTask instance has finished working, it will send a signal
-  // that we trap to finally destroy (after a time lag of some seconds, the
-  // monitor widget.
+  // When the MsRunReadTask instance has finished working, it will send a
+  // signal that we trap to finally destroy (after a time lag of some seconds,
+  // the monitor widget.
 
   connect(mass_data_integrator_task_p,
 
@@ -1002,9 +1102,9 @@ BasePlotWnd::integrateToTicIntensity(
 
   emit integrateToTicIntensitySignal(mass_data_integrator_p);
 
-  // Because the emitter is the same ProgramWindow object, we need to disconnect
-  // the signal so that at next opening of mass data file, it will not be called
-  // twice.
+  // Because the emitter is the same ProgramWindow object, we need to
+  // disconnect the signal so that at next opening of mass data file, it will
+  // not be called twice.
   disconnect(this,
 
              static_cast<void (BasePlotWnd::*)(
@@ -1096,6 +1196,22 @@ BasePlotWnd::lockYRangeToggle(bool checked)
 }
 
 
+void
+BasePlotWnd::lockYMarkerToggle(bool checked)
+{
+  m_isLockedYMarker = checked;
+  mpa_plotAreaOverlay->setVertRulerVisible(m_isLockedYMarker);
+  mp_vertRuleSlider->setVisible(m_isLockedYMarker);
+
+  if(m_isLockedYMarker)
+    {
+      // Make sure the overlay widget has the same size as that of the scroll
+      // area. This call will handle all the details.
+      resizeEvent(nullptr);
+    }
+}
+
+
 void
 BasePlotWnd::plotWidgetGotFocus(BasePlotCompositeWidget *composite_widget_p)
 {
@@ -1148,41 +1264,41 @@ BasePlotWnd::plotCompositeWidgetDestructionRequested(
 
 
 std::vector<BasePlotCompositeWidget *>
-BasePlotWnd::allWidgets()
+BasePlotWnd::allWidgets() const
 {
   // qDebug();
 
-  QList<BasePlotCompositeWidget *> widget_list =
+  QList<BasePlotCompositeWidget *> widget_q_list =
     findChildren<BasePlotCompositeWidget *>("plotCompositeWidget");
 
-  std::vector<BasePlotCompositeWidget *> pinned_down_widgets;
+  std::vector<BasePlotCompositeWidget *> widget_std_vector;
 
-  for(int iter = 0; iter < widget_list.size(); ++iter)
+  for(int iter = 0; iter < widget_q_list.size(); ++iter)
     {
-      BasePlotCompositeWidget *widget_p = widget_list.at(iter);
-      pinned_down_widgets.push_back(widget_p);
+      BasePlotCompositeWidget *widget_p = widget_q_list.at(iter);
+      widget_std_vector.push_back(widget_p);
     }
 
-  return pinned_down_widgets;
+  return widget_std_vector;
 }
 
 
 std::vector<BasePlotCompositeWidget *>
-BasePlotWnd::pinnedDownWidgets()
+BasePlotWnd::pinnedDownWidgets() const
 {
   // qDebug();
 
   // Iterate in the list of composite widgets currently in this window, and
   // check if they are pinned-down. If so add them to the vector.
 
-  QList<BasePlotCompositeWidget *> widget_list =
+  QList<BasePlotCompositeWidget *> widget_q_list =
     findChildren<BasePlotCompositeWidget *>("plotCompositeWidget");
 
   std::vector<BasePlotCompositeWidget *> pinned_down_widgets;
 
-  for(int iter = 0; iter < widget_list.size(); ++iter)
+  for(int iter = 0; iter < widget_q_list.size(); ++iter)
     {
-      BasePlotCompositeWidget *widget_p = widget_list.at(iter);
+      BasePlotCompositeWidget *widget_p = widget_q_list.at(iter);
 
       if(widget_p->isPinnedDown())
         pinned_down_widgets.push_back(widget_p);
@@ -1195,7 +1311,6 @@ BasePlotWnd::pinnedDownWidgets()
 BasePlotCompositeWidget *
 BasePlotWnd::plotCompositeWidgetForPlottable(QCPAbstractPlottable *plottable_p)
 {
-
   // We need to find the QCustomPlot first:
 
   QCustomPlot *q_custom_plot_p = plottable_p->parentPlot();
@@ -1292,6 +1407,33 @@ BasePlotWnd::showWindow()
 }
 
 
+QPoint
+BasePlotWnd::firstCompositeWidgetPosition() const
+{
+  std::vector<BasePlotCompositeWidget *> plot_widget_list = allWidgets();
+
+  if(plot_widget_list.size())
+    {
+      return plot_widget_list[0]->pos();
+    }
+  return pos();
+}
+
+
+QSize
+BasePlotWnd::firstCompositeWidgetSize() const
+{
+  std::vector<BasePlotCompositeWidget *> plot_widget_list = allWidgets();
+
+  if(plot_widget_list.size())
+    {
+      return plot_widget_list[0]->size();
+    }
+
+  return size();
+}
+
+
 std::size_t
 BasePlotWnd::fillInQualifiedMassSpectraVector(
   MsRunDataSetCstSPtr ms_run_data_set_csp,
@@ -1299,7 +1441,6 @@ BasePlotWnd::fillInQualifiedMassSpectraVector(
     &qualified_mass_spectra_sp,
   const ProcessingFlow &processing_flow)
 {
-
   // If the pointer to the vector is nullptr, we need to create the vector.
 
   if(qualified_mass_spectra_sp == nullptr)
@@ -1319,9 +1460,9 @@ BasePlotWnd::fillInQualifiedMassSpectraVector(
     }
 
   // Now, we know we can fill in the vector. But try to limit as much as
-  // possible the number of spectra to add. We do that by only pushing into the
-  // vector the spectra that have rt (and dt, if mobility data) values matching
-  // the innermost rt (and dt) ranges.
+  // possible the number of spectra to add. We do that by only pushing into
+  // the vector the spectra that have rt (and dt, if mobility data) values
+  // matching the innermost rt (and dt) ranges.
 
   std::pair<double, double> range_pair;
 
@@ -1330,8 +1471,7 @@ BasePlotWnd::fillInQualifiedMassSpectraVector(
   // mono- and two-dimensional, either from the ms run table view or any
   // trace/color map plot).
 
-  bool integration =
-    processing_flow.innermostRange("ANY_RT", range_pair);
+  bool integration = processing_flow.innermostRange("ANY_RT", range_pair);
 
   if(!integration)
     qFatal("Programming error.");
@@ -1353,8 +1493,7 @@ BasePlotWnd::fillInQualifiedMassSpectraVector(
   // We choose ANY_DT that matches in the widest possible manner all the
   // previous integrations that started with a dt-involving selection (both
   // mono- and two-dimensionale).
-  integration =
-    processing_flow.innermostRange("ANY_DT", range_pair);
+  integration = processing_flow.innermostRange("ANY_DT", range_pair);
 
   // Here, it is not an error not to have any DT integration because not all
   // the mass spectral acquisitions involved ion mobility.
