@@ -44,6 +44,8 @@
 #include <pappsomspp/processing/combiners/traceminuscombiner.h>
 #include <pappsomspp/processing/combiners/tracepluscombiner.h>
 
+#include <pappsomspp/processing/filters/savgolfilter.h>
+#include <pappsomspp/processing/filters/filterlowintensitysignalremoval.h>
 
 /////////////////////// Local includes
 #include "BaseTracePlotCompositeWidget.hpp"
@@ -52,6 +54,9 @@
 #include "MsFragmentationSpecDlg.hpp"
 #include "MzIntegrationParamsDlg.hpp"
 #include "ProgramWindow.hpp"
+#include "LabelledDoubleSpinBoxAction.hpp"
+#include "LabelledNumberLineEditAction.hpp"
+
 
 namespace msxps
 {
@@ -67,10 +72,10 @@ BaseTracePlotCompositeWidget::BaseTracePlotCompositeWidget(
 
   createMainMenu();
 
-  // The trace combined plot widget receives traces from integrations. It is possible
-  // to perform various actions with the traces inside a composite plot widget
-  // and the selection of the action to perform is done using check buttons or
-  // one combobox.
+  // The trace combined plot widget receives traces from integrations. It is
+  // possible to perform various actions with the traces inside a composite plot
+  // widget and the selection of the action to perform is done using check
+  // buttons or one combobox.
 
   // The replace trace(s) with new one action.
 
@@ -113,14 +118,14 @@ BaseTracePlotCompositeWidget::BaseTracePlotCompositeWidget(
             if(static_cast<TraceCombinationSign>(item_data) ==
                TraceCombinationSign::PLUS)
               {
-                qDebug() << "TraceCombinationSign::PLUS";
+                // qDebug() << "TraceCombinationSign::PLUS";
 
                 m_newTraceHandlingMethod = NewTraceHandlingMethod::PLUS_COMBINE;
               }
             else if(static_cast<TraceCombinationSign>(item_data) ==
                     TraceCombinationSign::MINUS)
               {
-                qDebug() << "TraceCombinationSign::MINUS";
+                // qDebug() << "TraceCombinationSign::MINUS";
 
                 m_newTraceHandlingMethod =
                   NewTraceHandlingMethod::MINUS_COMBINE;
@@ -184,17 +189,140 @@ BaseTracePlotCompositeWidget::createMainMenu()
 
   QMenu *filters_menu_p = mp_mainMenu->addMenu("&Filters");
 
-  QAction *savitzky_golay_filter_action_p = new QAction(
-    "Perform Savitzy-Golay smoothing", dynamic_cast<QObject *>(this));
-  savitzky_golay_filter_action_p->setStatusTip(
-    "Perform Savitzky-Golay smoothing");
-  savitzky_golay_filter_action_p->setShortcut(QKeySequence("Ctrl+S, G"));
+  // We want to provide the user with the ability to define a noise sample. They
+  // would create a new graph by integrating to that new graph with combination
+  // the various regions they feel are noise. They would get a noise sample
+  // trace. Then they'd call the menu below to store that trace for further use.
 
-  connect(savitzky_golay_filter_action_p, &QAction::triggered, [this]() {
-    applySavitzkyGolayFilter();
+  QMenu *noise_handling_menu_p = filters_menu_p->addMenu("Noise handling");
+
+  QAction *store_noise_sample_trace_action_p =
+    new QAction("Store trace as noise sample trace", dynamic_cast<QObject *>(this));
+  store_noise_sample_trace_action_p->setStatusTip(
+    "Set the (selected) trace as a noise sample trace");
+  connect(store_noise_sample_trace_action_p,
+          &QAction::triggered,
+          this,
+          &BaseTracePlotCompositeWidget::setNoiseSampleTrace);
+  noise_handling_menu_p->addAction(store_noise_sample_trace_action_p);
+
+  QAction *reset_noise_sample_trace_action_p =
+    new QAction("Reset the noise sample trace", dynamic_cast<QObject *>(this));
+  reset_noise_sample_trace_action_p->setStatusTip(
+    "Reset the noise sample trace");
+  connect(reset_noise_sample_trace_action_p,
+          &QAction::triggered,
+          this,
+          &BaseTracePlotCompositeWidget::resetNoiseSampleTrace);
+  noise_handling_menu_p->addAction(reset_noise_sample_trace_action_p);
+
+  filters_menu_p->addSeparator();
+
+  // Savitzky-Golay filtering: two submenus: one to parameterize and the other
+  // to actually run the filter.
+
+  QMenu *savgol_filter_menu_p = filters_menu_p->addMenu("&Savitzky-Golay");
+
+  QAction *savitzky_golay_config_filter_action_p = new QAction(
+    "Configure Savitzky-Golay smoothing filter", dynamic_cast<QObject *>(this));
+  savitzky_golay_config_filter_action_p->setStatusTip(
+    "Configure Savitzky-Golay smoothing filter");
+  savitzky_golay_config_filter_action_p->setShortcut(QKeySequence("Ctrl+S, C"));
+
+  connect(savitzky_golay_config_filter_action_p, &QAction::triggered, [this]() {
+    showSavitzkyGolayFilterParamsDlg();
   });
 
-  filters_menu_p->addAction(savitzky_golay_filter_action_p);
+  savgol_filter_menu_p->addAction(savitzky_golay_config_filter_action_p);
+
+  QAction *savitzky_golay_run_filter_action_p = new QAction(
+    "Run Savitzky-Golay smoothing filter", dynamic_cast<QObject *>(this));
+  savitzky_golay_run_filter_action_p->setStatusTip(
+    "Run Savitzky-Golay smoothing filter");
+  savitzky_golay_run_filter_action_p->setShortcut(QKeySequence("Ctrl+S, R"));
+
+  connect(savitzky_golay_run_filter_action_p, &QAction::triggered, [this]() {
+    runSavitzkyGolayFilter();
+  });
+
+  savgol_filter_menu_p->addAction(savitzky_golay_run_filter_action_p);
+
+  // Now add a new menu for the Low intensity signal removel filter.
+
+  QWidgetAction *widget_action_p =
+    new LabelledNumberLineEditAction("Low intensity threshold removal",
+                                     10000,
+                                     0,
+                                     10e100,
+                                     2,
+                                     true,
+                                     dynamic_cast<QObject *>(this));
+  widget_action_p->setStatusTip("Remove signal below the provided threshold");
+  filters_menu_p->addAction(widget_action_p);
+
+  connect(widget_action_p, &QAction::triggered, [this, widget_action_p]() {
+    bool ok = false;
+
+    double threshold =
+      static_cast<LabelledNumberLineEditAction *>(widget_action_p)
+        ->getValue()
+        .toDouble(&ok);
+
+    if(!ok)
+      qFatal("Failed to convert QVariant to double");
+
+    qDebug() << "triggered with threshold value:" << threshold;
+
+    // Get the source plottable that might be of interest to the user.
+    QCPAbstractPlottable *source_plottable_p =
+      plottableToBeUsedAsIntegrationSource();
+
+    // qDebug() << "The source plottable:" << source_plottable_p;
+
+    if(source_plottable_p == nullptr)
+      {
+        QMessageBox::information(
+          this,
+          "Please select *one* trace",
+          "In order to set the Savitzky-Golay filter parameters, the source "
+          "graph needs to be selected.",
+          QMessageBox::StandardButton::Ok,
+          QMessageBox::StandardButton::NoButton);
+
+        return;
+      }
+
+    std::size_t point_count;
+    double mean    = 0;
+    double std_dev = 0;
+
+    // If there is no noise sample trace, then provide 0-valued variables. The
+    // user will see nothing.
+    bool res = static_cast<BaseTracePlotWnd *>(mp_parentWnd)
+                 ->getNoiseSampleStats(point_count, mean, std_dev);
+
+    if(!res)
+      {
+        mean    = 0;
+        std_dev = 0;
+      }
+
+    // Create the new filter.
+    pappso::FilterLowIntensitySignalRemovalCstSPtr new_filter_csp =
+      std::make_shared<pappso::FilterLowIntensitySignalRemoval>(
+        pappso::FilterLowIntensitySignalRemoval(mean, std_dev, threshold));
+
+    // Get the processing flow for that specific graph.
+    ProcessingFlow *processing_flow_p =
+      getProcessingFlowPtrForPlottable(source_plottable_p);
+
+    // Add the filter to the processing flow for the proper plottable.
+    processing_flow_p->addFilter(new_filter_csp);
+
+    // Finally, since the action was triggered, run the filter.
+
+    runFilter(new_filter_csp->name());
+  });
 }
 
 
@@ -206,6 +334,7 @@ BaseTracePlotCompositeWidget::addTrace(const pappso::Trace &trace,
                                        const ProcessingFlow &processing_flow,
                                        const QColor &color)
 {
+  // qDebug();
 
   // Let's see the usage count of the MsRunDataSetCstSPtr.
   // qDebug() << "Usage count:" << ms_run_data_set_csp.use_count();
@@ -249,6 +378,8 @@ BaseTracePlotCompositeWidget::addTrace(const pappso::Trace &trace,
   else if(m_newTraceHandlingMethod == NewTraceHandlingMethod::MINUS_COMBINE ||
           m_newTraceHandlingMethod == NewTraceHandlingMethod::PLUS_COMBINE)
     {
+      // qDebug();
+
       // Combination is requested. There are some situations:
 
       // Now check what is the text:
@@ -307,22 +438,26 @@ BaseTracePlotCompositeWidget::addTrace(const pappso::Trace &trace,
               QVector<double> key_vector;
               QVector<double> value_vector;
 
-#pragma GCC warning "Filippo Rusconi: Please check if the bug was fixed in Qt"
+              // Apparently, as of 20210928, that bug is fixed.
+              //#pragma GCC warning "Filippo Rusconi: Please check if the bug
+              // was fixed in Qt"
 
-#if 0
-                  // Now replace the graph's data. Note that the data are
-                  // inherently sorted (true below).
-                  
-                  // The begin() -- end() ranges constructor did not work as of
-                  // Qt 5.14.2 this day: 20200721
+              // Now replace the graph's data. Note that the data are
+              // inherently sorted (true below).
 
-                  key_vector =
-                    QVector(map_trace.firstToVector().begin(),
-                            map_trace.firstToVector().end());
-                  value_vector =
-                    QVector(map_trace.secondToVector().begin(),
-                            map_trace.secondToVector().end());
-#endif
+              // The begin() -- end() ranges constructor did not work as of
+              // Qt 5.14.2 this day: 20200721 -- and still does not work
+              // 20211018.
+
+
+              // key_vector =
+              // QVector(map_trace.xValues().begin(),
+              // map_trace.xValues().end());
+              // value_vector =
+              // QVector(map_trace.yValues().begin(),
+              // map_trace.yValues().end());
+
+              // This is the workaround fix.
 
               for(auto &value : map_trace.xValues())
                 key_vector.push_back(value);
@@ -404,14 +539,135 @@ BaseTracePlotCompositeWidget::addTrace(const pappso::Trace &trace,
 
 
 void
-BaseTracePlotCompositeWidget::applySavitzkyGolayFilter()
+BaseTracePlotCompositeWidget::showSavitzkyGolayFilterParamsDlg()
 {
 
-  // First we get the source graph that might be of interest to the user.
+  // First we get the source plottable that might be of interest to the user.
 
   QCPAbstractPlottable *source_plottable_p =
     plottableToBeUsedAsIntegrationSource();
 
+  if(source_plottable_p == nullptr)
+    {
+      QMessageBox::information(
+        this,
+        "Please select *one* trace",
+        "In order to set the m/z integration parameters, the source "
+        "graph needs to be selected.",
+        QMessageBox::StandardButton::Ok,
+        QMessageBox::StandardButton::NoButton);
+
+      return;
+    }
+
+  // qDebug() << "The source plottable:" << source_plottable_p;
+
+  // Set aside SavGolParams that we'll need later.
+  pappso::SavGolParams sav_gol_params;
+
+  // Now see if we can get the filter from the ProcessingFlow. If it is not
+  // there, not a problem, since the idea is precisely that we are setting one
+  // up.
+
+  pappso::FilterNameInterfaceCstSPtr filter_csp =
+    filter("Savitzky-Golay", source_plottable_p);
+
+  if(filter_csp != nullptr)
+    {
+      const pappso::FilterSavitzkyGolay *filter_p =
+        static_cast<const pappso::FilterSavitzkyGolay *>(filter_csp.get());
+
+      if(filter_p != nullptr)
+        {
+          sav_gol_params = filter_p->getParameters();
+          // qDebug() << "Got parameters from found filter:"
+          //<< sav_gol_params.toString();
+        }
+    }
+
+  // Else, we keep the default values set in the constructor.
+
+  if(mp_savitzkyGolayFilterParamsDlg == nullptr)
+    {
+      // qDebug();
+
+      mp_savitzkyGolayFilterParamsDlg = new SavitzkyGolayFilterParamsDlg(
+        this, sav_gol_params, source_plottable_p->pen().color());
+
+      connect(mp_savitzkyGolayFilterParamsDlg,
+              &SavitzkyGolayFilterParamsDlg::
+                savitzkyGolayFilterParamsDlgShouldBeDestroyedSignal,
+              [this]() {
+                // qDebug();
+                delete QObject::sender();
+                mp_savitzkyGolayFilterParamsDlg = nullptr;
+              });
+
+      connect(
+        mp_savitzkyGolayFilterParamsDlg,
+        &SavitzkyGolayFilterParamsDlg::savitzkyGolayFilterParamsChangedSignal,
+        this,
+        &BaseTracePlotCompositeWidget::savitzkyGolayFilterParamsChanged);
+    }
+
+  mp_savitzkyGolayFilterParamsDlg->show();
+}
+
+
+void
+BaseTracePlotCompositeWidget::savitzkyGolayFilterParamsChanged(
+  pappso::SavGolParams sav_gol_params)
+{
+  // qDebug();
+
+  // First we get the source plottable that might be of interest to the user.
+
+  QCPAbstractPlottable *source_plottable_p =
+    plottableToBeUsedAsIntegrationSource();
+
+  // qDebug() << "The source plottable:" << source_plottable_p;
+
+  if(source_plottable_p == nullptr)
+    {
+      QMessageBox::information(
+        this,
+        "Please select *one* trace",
+        "In order to set the Savitzky-Golay filter parameters, the source "
+        "graph needs to be selected.",
+        QMessageBox::StandardButton::Ok,
+        QMessageBox::StandardButton::NoButton);
+
+      return;
+    }
+
+  // Create a new filter
+
+  pappso::FilterSavitzkyGolayCstSPtr filter_csp =
+    std::make_shared<const pappso::FilterSavitzkyGolay>(
+      pappso::FilterSavitzkyGolay(sav_gol_params));
+
+  // Next get the processing flow for that specific graph.
+
+  ProcessingFlow *processing_flow_p =
+    getProcessingFlowPtrForPlottable(source_plottable_p);
+
+  // And finally set the parameters.
+  processing_flow_p->addFilter(filter_csp);
+}
+
+
+void
+BaseTracePlotCompositeWidget::runFilter(const QString &filter_name)
+{
+  // qDebug();
+
+  // First get the plottable for which to apply the filter.
+
+  QCPAbstractPlottable *source_plottable_p =
+    plottableToBeUsedAsIntegrationSource();
+
+  // qDebug() << "The source plottable:" << source_plottable_p;
+
   if(source_plottable_p == nullptr)
     {
       QMessageBox::information(
@@ -425,36 +681,43 @@ BaseTracePlotCompositeWidget::applySavitzkyGolayFilter()
       return;
     }
 
+  // Now see if we can get a filter by that name and for that plottable.
+
+  pappso::FilterNameInterfaceCstSPtr filter_csp =
+    filter(filter_name, source_plottable_p);
+
+  if(filter_csp == nullptr)
+    {
+      QMessageBox::information(this,
+                               "Filter not found",
+                               "The filter was not found. Please, check that "
+                               "it was configured properly.",
+                               QMessageBox::StandardButton::Ok,
+                               QMessageBox::StandardButton::NoButton);
+
+      return;
+    }
+
+  // qDebug() << "Running filter by name:" << filter_name
+  //<< "with string representation : " << filter_csp->toString();
+
+  // We can go on with the filtering task.
+
   pappso::Trace data_trace =
     static_cast<pappso::BaseTracePlotWidget *>(mp_plotWidget)
       ->toTrace(static_cast<QCPGraph *>(source_plottable_p));
 
-  // Now we need to set the parameters of the SavGol filter.
-
-  // Get the mz integration params for the plottable.
-  pappso::MzIntegrationParams mz_integration_params =
-    getMzIntegrationParams(source_plottable_p);
-
-  pappso::SavGolParams sav_gol_params = mz_integration_params.getSavGolParams();
-
-  pappso::FilterSavitzkyGolay filter_sav_gol =
-    pappso::FilterSavitzkyGolay(sav_gol_params.nL,
-                                sav_gol_params.nR,
-                                sav_gol_params.m,
-                                sav_gol_params.lD,
-                                sav_gol_params.convolveWithNr);
-
-  pappso::Trace filtered_trace = filter_sav_gol.filter(data_trace);
+  pappso::Trace filtered_trace = filter_csp->filter(data_trace);
 
   // At this point, add the trace ! But only if it has points in it.
 
   if(!filtered_trace.size())
     {
-      // qDebug() << "The SavGolFilter produced an empty trace. Plotting
-      // nothing.";
+      qDebug() << "The trace produced by the filter is empty.";
 
       m_ui.statusLineEdit->setText(
-        "The SavGolFilter produced an empty trace. Plotting nothing.");
+        QString("The filter %1 produced an empty trace. Plotting nothing.")
+          .arg(filter_name));
 
       return;
     }
@@ -476,6 +739,13 @@ BaseTracePlotCompositeWidget::applySavitzkyGolayFilter()
 }
 
 
+void
+BaseTracePlotCompositeWidget::runSavitzkyGolayFilter()
+{
+  runFilter("Savitzky-Golay");
+}
+
+
 void
 BaseTracePlotCompositeWidget::plotWidgetKeyPressEvent(
   const pappso::BasePlotContext &context)
@@ -494,14 +764,15 @@ BaseTracePlotCompositeWidget::plotWidgetKeyPressEvent(
   if(context.m_pressedKeyCode == Qt::Key_Space)
     {
       // At the moment, in this class we have nothing very specific to do. But
-      // we could reparse the m_analysisStanza string and fill-in any remaining
+      // we could reparse the m_analysisStanza string and fill-in any
+      // remaining
       // %<char> that might have been remained unfilled by the base class
       // function call.
     }
 
-  // We do not call the stanza recording function because that will be performed
-  // by the derived classes when all the chances of having filled-in the fields
-  // have been met.
+  // We do not call the stanza recording function because that will be
+  // performed by the derived classes when all the chances of having filled-in
+  // the fields have been met.
 }
 
 
@@ -511,9 +782,9 @@ BaseTracePlotCompositeWidget::plotWidgetKeyReleaseEvent(
 {
   BasePlotCompositeWidget::plotWidgetKeyReleaseEvent(context);
 
-  // The F3 key triggers the displacement of the mouse cursor to the next graph
-  // point. If integrations are requested, then an integration for that single
-  // point is performed (see m_isSinglePointIntegrationMode).
+  // The F3 key triggers the displacement of the mouse cursor to the next
+  // graph point. If integrations are requested, then an integration for that
+  // single point is performed (see m_isSinglePointIntegrationMode).
 
   if(m_plotWidgetReleasedKeyCode == Qt::Key_F3)
     {
@@ -523,8 +794,8 @@ BaseTracePlotCompositeWidget::plotWidgetKeyReleaseEvent(
 
   if(m_plotWidgetPressedKeyCode == Qt::Key_Escape)
     {
-      // The user wants to nullify any ongoing integration using the right mouse
-      // button.
+      // The user wants to nullify any ongoing integration using the right
+      // mouse button.
 
       // The simplest way is to simulate a left click mouse button.
 
@@ -549,7 +820,7 @@ BaseTracePlotCompositeWidget::plotWidgetKeyReleaseEvent(
           QCoreApplication::sendEvent(QWidget::focusWidget(), mouse_event_p);
         }
 
-      qDebug() << "Escape key released.";
+      // qDebug() << "Escape key released.";
     }
 }
 
@@ -590,8 +861,9 @@ BaseTracePlotCompositeWidget::moveMouseCursorToNextGraphPoint(
 
   // This is the secret of the whole process. In order to be able to use the
   // findEnd() function from QCustomPlot, we need to virtually displace the
-  // current cursor point one pixel to the right. In this manner findEnd() will
-  // not return the same iterator position as for the previous function call.
+  // current cursor point one pixel to the right. In this manner findEnd()
+  // will not return the same iterator position as for the previous function
+  // call.
   //
   // From the docs:
   // findEnd() returns an iterator to the element after the data point with a
@@ -658,8 +930,8 @@ BaseTracePlotCompositeWidget::moveMouseCursorToNextGraphPoint(
 
       if(next_data_point_iterator == graph_data_container_sp->begin())
         {
-          // qDebug() << "next_data_point_iterator is begin(). Subtracting 10 to
-          // " "the begin() key.";
+          // qDebug() << "next_data_point_iterator is begin(). Subtracting 10
+          // to " "the begin() key.";
 
           m_integrationRange.lower = graph_data_container_sp->begin()->key - 10;
         }
@@ -708,13 +980,70 @@ BaseTracePlotCompositeWidget::moveMouseCursorToNextGraphPoint(
       // qDebug() << "Integration range:" << m_integrationRange.lower << "-"
       //<< m_integrationRange.upper;
 
-      // qDebug() << "The context from the plot widget, now that we have moved "
+      // qDebug() << "The context from the plot widget, now that we have moved
+      // "
       //"the cursor:"
       //<< mp_plotWidget->getContext().toString();
     }
 }
 
 
+void
+BaseTracePlotCompositeWidget::setNoiseSampleTrace()
+{
+
+  // First we get the source plottable that might be of interest to the user.
+
+  QCPGraph *source_graph_p =
+    static_cast<QCPGraph *>(plottableToBeUsedAsIntegrationSource());
+
+  if(source_graph_p == nullptr)
+    {
+      QMessageBox::information(
+        this,
+        "Please select *one* trace",
+        "In order to set the noise sample trace, the source "
+        "graph needs to be selected.",
+        QMessageBox::StandardButton::Ok,
+        QMessageBox::StandardButton::NoButton);
+
+      return;
+    }
+
+  pappso::Trace trace =
+    static_cast<pappso::BaseTracePlotWidget *>(mp_plotWidget)
+      ->toTrace(source_graph_p);
+
+  static_cast<BaseTracePlotWnd *>(mp_parentWnd)->setNoiseSampleTrace(trace);
+}
+
+
+void
+BaseTracePlotCompositeWidget::resetNoiseSampleTrace()
+{
+
+  // First we get the source plottable that might be of interest to the user.
+
+  QCPGraph *source_graph_p =
+    static_cast<QCPGraph *>(plottableToBeUsedAsIntegrationSource());
+
+  if(source_graph_p == nullptr)
+    {
+      QMessageBox::information(
+        this,
+        "Please select *one* trace",
+        "In order to reset the noise sample trace, the source "
+        "graph needs to be selected.",
+        QMessageBox::StandardButton::Ok,
+        QMessageBox::StandardButton::NoButton);
+
+      return;
+    }
+
+  static_cast<BaseTracePlotWnd *>(mp_parentWnd)->resetNoiseSampleTrace();
+}
+
+
 bool
 BaseTracePlotCompositeWidget::savePeakListToClipboard()
 {
