@@ -53,7 +53,7 @@
 #include "BasePlotCompositeWidget.hpp"
 #include "../nongui/MsRunDataSet.hpp"
 #include "../nongui/ProcessingFlow.hpp"
-#include <pappsomspp/processing/combiners/mzintegrationparams.h>
+#include "SavitzkyGolayFilterParamsDlg.hpp"
 
 
 namespace msxps
@@ -93,17 +93,27 @@ class BaseTracePlotCompositeWidget : public BasePlotCompositeWidget
   virtual void
   moveMouseCursorToNextGraphPoint(const pappso::BasePlotContext &context);
 
-  void applySavitzkyGolayFilter();
+  void showSavitzkyGolayFilterParamsDlg();
+
+  void savitzkyGolayFilterParamsChanged(pappso::SavGolParams sav_gol_params);
+
+  void runFilter(const QString &filter_name);
+  void runSavitzkyGolayFilter();
 
   bool savePeakListToClipboard();
 
+
   public slots:
 
-  void plotWidgetKeyPressEvent(const pappso::BasePlotContext &context) override;
+  virtual void
+  plotWidgetKeyPressEvent(const pappso::BasePlotContext &context) override;
 
   virtual void
   plotWidgetKeyReleaseEvent(const pappso::BasePlotContext &context) override;
 
+  virtual void setNoiseSampleTrace();
+  virtual void resetNoiseSampleTrace();
+
   signals:
 
   void plotWidgetMouseReleaseEventSignal(QMouseEvent *event);
@@ -113,7 +123,10 @@ class BaseTracePlotCompositeWidget : public BasePlotCompositeWidget
   // widget.
   bool m_isSinglePointIntegrationMode = false;
   QCPRange m_integrationRange;
+  SavitzkyGolayFilterParamsDlg *mp_savitzkyGolayFilterParamsDlg = nullptr;
+
   virtual void createMainMenu() override;
+  pappso::Trace lowIntensitySignalRemoval(pappso::Trace &trace, double value);
 };
 
 
