@@ -175,29 +175,57 @@ void
 DriftSpecMassSpecColorMapPlotCompositeWidget::integrationRequested(
   const pappso::BasePlotContext &context)
 {
-  // qDebug().noquote() << "context:" << context.toString();
+   qDebug().noquote() << "context:" << context.toString();
 
   // We are getting that signal from a plot widget that operates as a
-  // drift spec / mass spec color map plot widget.
+  // drift spec / mass spec color map plot widget. By essence, the selected
+  // region needs to be two-dimensional, so the selection polygon needs to be
+  // so.
 
-  // Remember that the xregion start and end are not sorted.
-  double x_range_start =
-    std::min<double>(context.xRegionRangeStart, context.xRegionRangeEnd);
-  double x_range_end =
-    std::max<double>(context.xRegionRangeStart, context.xRegionRangeEnd);
+  // Check if the selection polygon contain meaninful data. We are in a colormap
+  // plot widget, so the selection polygon cannot be in the form of a
+  // mono-dimensional polygon.
+
+  if(!context.m_selectionPolygon.is2D())
+    {
+      qDebug() << "The selection polygon cannot be mono-dimensional when "
+                  "selecting for integration from color map";
+      return;
+    }
+  else
+    {
+      if(context.m_selectionPolygon.isRectangle())
+        qDebug() << "The selection polygon is rectangle.";
+      else
+        qDebug() << "The selection polygon is oblique (skewed).";
+    }
+
+  // Sanity check.
+
+  // Immediately check if the selection range makes sense. For example, having
+  // both the start/end values in the negative values does not mean anything
+  // with respect to drift spec vs mass spectrum data!
+
+  double x_range_start;
+  double x_range_end;
+
+  context.m_selectionPolygon.rangeX(x_range_start, x_range_end);
+
+  qDebug() << "X axis range:" << x_range_start << "-" << x_range_end;
 
   if(x_range_start < 0 && x_range_end < 0)
     return;
 
-  double y_range_start =
-    std::min<double>(context.yRegionRangeStart, context.yRegionRangeEnd);
-  double y_range_end =
-    std::max<double>(context.yRegionRangeStart, context.yRegionRangeEnd);
+  double y_range_start;
+  double y_range_end;
+
+  context.m_selectionPolygon.rangeY(y_range_start, y_range_end);
+
+  qDebug() << "Y axis range:" << y_range_start << "-" << y_range_end;
 
   if(y_range_start < 0 && y_range_end < 0)
     return;
 
-
   // We need check which destination button is currently checked.
 
   // The widget does not handle the integrations. There are the
@@ -211,7 +239,6 @@ DriftSpecMassSpecColorMapPlotCompositeWidget::integrationRequested(
   // selected is the right one. If there are more than one traces selected, then
   // we cannot do anything. Display an error message.
 
-
   // The QCustomPlot-based widget might contain more than one graph, and we
   // need to know on what graph the user is willing to perform the
   // integration.
@@ -243,26 +270,10 @@ DriftSpecMassSpecColorMapPlotCompositeWidget::integrationRequested(
 
   // qDebug() << ms_run_data_set_csp->getMsRunDataSetStats().toString();
 
-  // Now we need to add a new step in that processing flow so that we document
-  // this new integration that we are performing.
+  // Create a processing step to document the specifics of this new integration.
 
   ProcessingStep *processing_step_p = new ProcessingStep();
 
-  // The step has a multi-map of pairs that each relate a processing type with a
-  // processing spec. So we need to document a new pair type/spec, let's start
-  // with the spec that documents the range of the data we are willing to
-  // integrate, that is, the selected region of the graph.
-
-  // The ProcessingSpec has the range start/end values, an optional ms
-  // fragmentation specification and a date time that is set automatically at
-  // construction time.
-
-  // Allocate a new spec that will hold the DT dimension range.
-  ProcessingSpec *dt_processing_spec_p = new ProcessingSpec();
-
-  // And another one for the MZ dimension range.
-  ProcessingSpec *mz_processing_spec_p = new ProcessingSpec();
-
   // If the user had set ms fragmentation specifications, all these are listed
   // in the various ProcessingSpec instances, so we do not loose any. However,
   // the user is entitled to define any new ms fragmentation specifications
@@ -280,7 +291,7 @@ DriftSpecMassSpecColorMapPlotCompositeWidget::integrationRequested(
       // flow " "is valid, using it:"
       //<< ms_fragmentation_spec.toString();
 
-      mz_processing_spec_p->setMsFragmentationSpec(ms_fragmentation_spec);
+      processing_step_p->setMsFragmentationSpec(ms_fragmentation_spec);
     }
   else
     {
@@ -290,99 +301,118 @@ DriftSpecMassSpecColorMapPlotCompositeWidget::integrationRequested(
 
   // Now define the graph ranges for the integration operation.
 
-  // Remember that the region start and end are not sorted. Also, the colormap
-  // can have its axes transposed, so we need to ensure that we use the proper
-  // axis for mz and the proper axis for the other dimension (rt or dt).
+  // Sanity check.
 
-  if(static_cast<pappso::BaseColorMapPlotWidget *>(mp_plotWidget)
-       ->xAxisDataKind() == pappso::DataKind::dt)
+  pappso::DataKind x_axis_data_kind =
+    static_cast<pappso::BaseColorMapPlotWidget *>(mp_plotWidget)
+      ->xAxisDataKind();
+  pappso::DataKind y_axis_data_kind =
+    static_cast<pappso::BaseColorMapPlotWidget *>(mp_plotWidget)
+      ->yAxisDataKind();
+
+  if(x_axis_data_kind == pappso::DataKind::dt)
+    qDebug() << "X axis: dt";
+  else if(x_axis_data_kind == pappso::DataKind::mz)
+    qDebug() << "X axis: mz";
+
+  if(x_axis_data_kind == pappso::DataKind::dt &&
+     y_axis_data_kind != pappso::DataKind::mz)
     {
-      dt_processing_spec_p->setRange(x_range_start, x_range_end);
-      mz_processing_spec_p->setRange(y_range_start, y_range_end);
-
-      qDebug() << "x axis data kind is dt"
-               << "dt range:" << x_range_start << "-" << x_range_end
-               << "mz range:" << y_range_start << "-" << y_range_end;
+      qFatal("Programming error.");
     }
-  else
+  else if(x_axis_data_kind == pappso::DataKind::mz &&
+          y_axis_data_kind != pappso::DataKind::dt)
     {
-      dt_processing_spec_p->setRange(y_range_start, y_range_end);
-      mz_processing_spec_p->setRange(x_range_start, x_range_end);
-
-      qDebug() << "x axis data kind is mz"
-               << "dt range:" << y_range_start << "-" << y_range_end
-               << "mz range:" << x_range_start << "-" << x_range_end;
+      qFatal("Programming error.");
     }
 
+  // The colormap can have its axes transposed, so we need to ensure that we
+  // document the axis/data-kind relation properly. Note that we want the
+  // processing step to be documented according to the alpha numerical ordering.
+  // So, if we are working in a DT_RT colormap, then, x axis needs to be DT and
+  // y axis needs to be RT.
+
+  // Make a local copy of the selection polygon because we might need to
+  // transpose it.
+
+  pappso::SelectionPolygon local_selection_polygon = context.m_selectionPolygon;
+
+  if(x_axis_data_kind == pappso::DataKind::mz)
+    {
+      // We have to transpose the selection rectangle so that x:MZ and y:RT.
+      local_selection_polygon = local_selection_polygon.transpose();
+    }
+  // else
+  // No need to tranpose, we deal with data in the right x:MZ y:RT.
+
+   qDebug() << "The selection polygon:" << local_selection_polygon.toString();
+
+  // Now we know we have the right selection polygon for x:MZ and y:RT.
+
+  processing_step_p->setDataKind(pappso::Axis::x, pappso::DataKind::dt);
+  processing_step_p->setDataKind(pappso::Axis::y, pappso::DataKind::mz);
+
+
+  // And now actually set the selection polygon to the step.
+
+  processing_step_p->setSelectionPolygon(local_selection_polygon);
+
+  // qDebug() << "Selection polygon as set to the step:"
+  //<< processing_step_p->getSelectionPolygon().toString();
+
+  // We must say what is the data source processing type.
+  processing_step_p->setSrcProcessingType(pappso::Axis::x, "DT");
+  processing_step_p->setSrcProcessingType(pappso::Axis::y, "MZ");
+
+  // Now we need to document not the source but the destination. What kind of
+  // integration are we about to perform?
+
   if(m_ui.integrateToRtPushButton->isChecked())
     {
-      // qDebug() << "Integration to XIC chromatogram.";
+       qDebug() << "Integrating to XIC chromatogram.";
 
       // We need to set two specs to the step.
 
-      processing_step_p->newSpec(ProcessingType("DT_MZ_DT_TO_RT"),
-                                 dt_processing_spec_p);
-      processing_step_p->newSpec(ProcessingType("DT_MZ_MZ_TO_RT"),
-                                 mz_processing_spec_p);
+      processing_step_p->setDestProcessingType("RT");
 
       local_processing_flow.push_back(processing_step_p);
 
       static_cast<BaseColorMapPlotWnd *>(mp_parentWnd)
         ->integrateToRt(plottable_p, local_processing_flow);
     }
-  else if(m_ui.integrateToRtDtPushButton->isChecked())
+  else if(m_ui.integrateToDtRtPushButton->isChecked())
     {
-      // qDebug() << "Integration to XIC chromatogram. / drift spectrum color
-      // map";
+       qDebug() << "Integrating to dt / rt color map.";
 
-      // We need to set two specs to the step.
-
-      processing_step_p->newSpec(ProcessingType("DT_MZ_DT_TO_RT_DT"),
-                                 dt_processing_spec_p);
-      processing_step_p->newSpec(ProcessingType("DT_MZ_MZ_TO_RT_DT"),
-                                 mz_processing_spec_p);
+      processing_step_p->setDestProcessingType("DT_RT");
 
       local_processing_flow.push_back(processing_step_p);
 
       static_cast<BaseColorMapPlotWnd *>(mp_parentWnd)
-        ->integrateToRtDt(plottable_p, local_processing_flow);
+        ->integrateToDtRt(plottable_p, local_processing_flow);
     }
+  // Handle all the integrations to MZ in one place
   else if(m_ui.integrateToMzPushButton->isChecked() ||
           m_ui.integrateToDtMzPushButton->isChecked() ||
-          m_ui.integrateToRtMzPushButton->isChecked())
+          m_ui.integrateToMzRtPushButton->isChecked())
     {
       if(m_ui.integrateToMzPushButton->isChecked())
         {
-          // qDebug() << "Integrating to mass spectrum.";
+           qDebug() << "Integrating to mass spectrum.";
 
-          // We need to set two specs to the step.
-
-          processing_step_p->newSpec(ProcessingType("DT_MZ_DT_TO_MZ"),
-                                     dt_processing_spec_p);
-          processing_step_p->newSpec(ProcessingType("DT_MZ_MZ_TO_MZ"),
-                                     mz_processing_spec_p);
+          processing_step_p->setDestProcessingType("MZ");
         }
-      else if(m_ui.integrateToRtMzPushButton->isChecked())
+      else if(m_ui.integrateToMzRtPushButton->isChecked())
         {
-          // qDebug() << "Integrating to rt / m/z color map.";
+           qDebug() << "Integrating to rt / m/z color map.";
 
-          // We need to set two specs to the step.
-
-          processing_step_p->newSpec(ProcessingType("DT_MZ_DT_TO_RT_MZ"),
-                                     dt_processing_spec_p);
-          processing_step_p->newSpec(ProcessingType("DT_MZ_MZ_TO_RT_MZ"),
-                                     mz_processing_spec_p);
+          processing_step_p->setDestProcessingType("MZ_RT");
         }
       else if(m_ui.integrateToDtMzPushButton->isChecked())
         {
-          // qDebug() << "Integrating to dt / m/z color map.";
+           qDebug() << "Integrating to dt / m/z color map.";
 
-          // We need to set two specs to the step.
-
-          processing_step_p->newSpec(ProcessingType("DT_MZ_DT_TO_DT_MZ"),
-                                     dt_processing_spec_p);
-          processing_step_p->newSpec(ProcessingType("DT_MZ_MZ_TO_DT_MZ"),
-                                     mz_processing_spec_p);
+          processing_step_p->setDestProcessingType("DT_MZ");
         }
 
       // At this point we need to define how the mass data integrator will
@@ -425,7 +455,7 @@ DriftSpecMassSpecColorMapPlotCompositeWidget::integrationRequested(
       // Note that we cannot integrate mass spectra with a high resolution
       // here. We need to reduce the resolution to integer resolution.
       if(m_ui.integrateToDtMzPushButton->isChecked() ||
-         m_ui.integrateToRtMzPushButton->isChecked())
+         m_ui.integrateToMzRtPushButton->isChecked())
         {
           mz_integration_params.setBinningType(pappso::BinningType::NONE);
           mz_integration_params.setDecimalPlaces(0);
@@ -456,22 +486,17 @@ DriftSpecMassSpecColorMapPlotCompositeWidget::integrationRequested(
           static_cast<BaseColorMapPlotWnd *>(mp_parentWnd)
             ->integrateToDtMz(plottable_p, local_processing_flow);
         }
-      else if(m_ui.integrateToRtMzPushButton->isChecked())
+      else if(m_ui.integrateToMzRtPushButton->isChecked())
         {
           static_cast<BaseColorMapPlotWnd *>(mp_parentWnd)
-            ->integrateToRtMz(plottable_p, local_processing_flow);
+            ->integrateToMzRt(plottable_p, local_processing_flow);
         }
     }
   else if(m_ui.integrateToDtPushButton->isChecked())
     {
-      // qDebug() << "Integrating to drift spectrum.";
+       qDebug() << "Integrating to drift spectrum.";
 
-      // We need to set two specs to the step.
-
-      processing_step_p->newSpec(ProcessingType("DT_MZ_DT_TO_DT"),
-                                 dt_processing_spec_p);
-      processing_step_p->newSpec(ProcessingType("DT_MZ_MZ_TO_DT"),
-                                 mz_processing_spec_p);
+      processing_step_p->setDestProcessingType("DT");
 
       local_processing_flow.push_back(processing_step_p);
 
@@ -480,14 +505,9 @@ DriftSpecMassSpecColorMapPlotCompositeWidget::integrationRequested(
     }
   else if(m_ui.integrateToIntPushButton->isChecked())
     {
-      // qDebug() << "Integrating to TIC intensity.";
+       qDebug() << "Integrating to TIC intensity.";
 
-      // We need to set two specs to the step.
-
-      processing_step_p->newSpec(ProcessingType("DT_MZ_DT_TO_INT"),
-                                 dt_processing_spec_p);
-      processing_step_p->newSpec(ProcessingType("DT_MZ_MZ_TO_INT"),
-                                 mz_processing_spec_p);
+      processing_step_p->setDestProcessingType("INT");
 
       local_processing_flow.push_back(processing_step_p);
 
