@@ -111,8 +111,6 @@ DriftSpecMassSpecColorMapWnd::integrateToDtMz(
     qualified_mass_spectra_sp,
   const ProcessingFlow &processing_flow)
 {
-  // qDebug();
-
   // qDebug().noquote() << "Integrating to dt | mz with processing flow:"
   //<< processing_flow.toString();
 
@@ -123,28 +121,39 @@ DriftSpecMassSpecColorMapWnd::integrateToDtMz(
   if(ms_run_data_set_csp == nullptr)
     qFatal("Cannot be that the pointer is nullptr.");
 
-  // We will try to limit the number of mass spectra to iterate through with the
-  // integration visitor.
+  // FIXME: BEGIN Sanity check that might be removed when the program is stabilized.
+  std::pair<double, double> range_pair;
 
-  double start_rt = std::numeric_limits<double>::infinity();
-  double end_rt   = std::numeric_limits<double>::infinity();
-
-  bool integration_rt = processing_flow.innermostRtRange(start_rt, end_rt);
+  bool integration_rt = processing_flow.innermostRange("ANY_RT", range_pair);
 
   if(!integration_rt)
     qFatal("Programming error.");
 
-  // First prepare a vector of QualifiedMassSpectrumCstSPtr
+  qDebug() << qSetRealNumberPrecision(10) << "Innermost RT range:"
+           << "for y axis: " << range_pair.first << "-" << range_pair.second;
+
+  bool integration_dt = processing_flow.innermostRange("ANY_DT", range_pair);
+
+  if(integration_dt)
+    qDebug() << qSetRealNumberPrecision(10) << "Innermost DT range:"
+             << "for y axis: " << range_pair.first << "-" << range_pair.second;
+  // FIXME: END Sanity check that might be removed when the program is stabilized.
+
+  // First prepare a vector of QualifiedMassSpectrumCstSPtr. If the
+  // qualified_mass_spectra_sp is not empty, the
+  // fillInQualifiedMassSpectraVector() function below does not change the
+  // vector and returns its size.
 
   std::size_t qualified_mass_spectra_count = fillInQualifiedMassSpectraVector(
     ms_run_data_set_csp, qualified_mass_spectra_sp, processing_flow);
 
+  qDebug()
+    << "The number of selected mass spectra on the basis of rough RT/DT values:"
+    << qualified_mass_spectra_count;
+
   if(!qualified_mass_spectra_count)
     return;
 
-  // qDebug() << "The number of remaining mass spectra:"
-  //<< qualified_mass_spectra_count;
-
   // Now start the actual integration work.
 
   // Allocate a mass data integrator to integrate the data.
@@ -154,7 +163,8 @@ DriftSpecMassSpecColorMapWnd::integrateToDtMz(
       new QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz(
         ms_run_data_set_csp, processing_flow, qualified_mass_spectra_sp);
 
-  mass_data_integrator_p->setMaxThreadUseCount(mp_programWindow->getMaxThreadUseCount());
+  mass_data_integrator_p->setMaxThreadUseCount(
+    mp_programWindow->getMaxThreadUseCount());
 
   // Ensure the mass data integrator messages are used.
 
@@ -169,21 +179,17 @@ DriftSpecMassSpecColorMapWnd::integrateToDtMz(
   MassDataIntegratorTask *mass_data_integrator_task_p =
     new MassDataIntegratorTask();
 
+  qRegisterMetaType<std::size_t>("std::size_t");
   qRegisterMetaType<pappso::DataKind>("pappso::DataKind");
 
   // This signal starts the computation in the MassDataIntegratorTask object.
   connect(this,
 
-          static_cast<void (DriftSpecMassSpecColorMapWnd::*)(
-            QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz *,
-            pappso::DataKind)>(
-            &DriftSpecMassSpecColorMapWnd::integrateToDtRtMzSignal),
+          &DriftSpecMassSpecColorMapWnd::integrateToDtRtMzSignal,
 
           mass_data_integrator_task_p,
 
-          static_cast<void (MassDataIntegratorTask::*)(
-            QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz *,
-            pappso::DataKind)>(&MassDataIntegratorTask::integrateToDtRtMz),
+          &MassDataIntegratorTask::integrateToDtRtMz,
 
           // Fundamental for signals that travel across QThread instances...
           Qt::QueuedConnection);
@@ -278,7 +284,7 @@ DriftSpecMassSpecColorMapWnd::integrateToDtMz(
   //"data integrator:"
   //<< mass_data_integrator_p;
 
-  emit integrateToDtRtMzSignal(mass_data_integrator_p, pappso::DataKind::dt);
+  emit integrateToDtRtMzSignal(mass_data_integrator_p);
 
   // We do not want to make signal/slot calls more than once. This is because
   // one user might well trigger more than one integration from this window to a
@@ -291,15 +297,11 @@ DriftSpecMassSpecColorMapWnd::integrateToDtMz(
 
     this,
 
-    static_cast<void (DriftSpecMassSpecColorMapWnd::*)(
-      QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz *,
-      pappso::DataKind)>(
-      &DriftSpecMassSpecColorMapWnd::integrateToDtRtMzSignal),
+    &DriftSpecMassSpecColorMapWnd::integrateToDtRtMzSignal,
 
     mass_data_integrator_task_p,
-    static_cast<void (MassDataIntegratorTask::*)(
-      QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz *,
-      pappso::DataKind)>(&MassDataIntegratorTask::integrateToDtRtMz));
+
+    &MassDataIntegratorTask::integrateToDtRtMz);
 }
 
 
@@ -359,7 +361,9 @@ DriftSpecMassSpecColorMapWnd::finishedIntegratingToDtMz(
     mass_data_integrator_to_rtdtmz_p->getColorMapMinKey(),
     mass_data_integrator_to_rtdtmz_p->getColorMapMaxKey(),
     mass_data_integrator_to_rtdtmz_p->getColorMapMinMz(),
-    mass_data_integrator_to_rtdtmz_p->getColorMapMaxMz());
+    mass_data_integrator_to_rtdtmz_p->getColorMapMaxMz(),
+    /*the min Z value will be determined later*/ 0,
+    /*the max Z value will be determined later*/ 0);
 
   // qDebug() << "color_map_plot_config:" << color_map_plot_config.toString();
 
