@@ -170,8 +170,8 @@ DriftSpecTracePlotCompositeWidget::moveMouseCursorToNextGraphPoint(
   // in the base class function.
 
   pappso::BasePlotContext local_context = context;
-  local_context.xRegionRangeStart       = m_integrationRange.lower;
-  local_context.xRegionRangeEnd         = m_integrationRange.upper;
+  local_context.m_xRegionRangeStart     = m_integrationRange.lower;
+  local_context.m_xRegionRangeEnd       = m_integrationRange.upper;
 
   if(m_isSinglePointIntegrationMode)
     integrationRequested(local_context);
@@ -187,16 +187,34 @@ DriftSpecTracePlotCompositeWidget::integrationRequested(
   // The reference implementation is the TicXicChromTracePlotCompositeWidget
   // one. Look there for all the explanative comments.
 
-  // Immediately check if the selection range makes sense. For example, having
-  // both the start/end values in the negative values does not mean anything
-  // with respect to mass spectrum data!
+  // Make a local copy of the context because we may need to modify it.
+  pappso::BasePlotContext local_context = context;
 
-  // Remember that the xregion start and end are not sorted.
-  double range_start =
-    std::min<double>(context.xRegionRangeStart, context.xRegionRangeEnd);
-  double range_end =
-    std::max<double>(context.xRegionRangeStart, context.xRegionRangeEnd);
+  // Check if the selection polygon contain meaningful data. We are in trace
+  // plot widget, so the selection polygon might be in the form of a
+  // mono-dimensional polygon, although this is not mandatory. But we rely on
+  // the selection polygon to be of 1 or 2 dimension for some optimizations. So,
+  // if the polygon is 2D, convert it to 1D.
 
+  if(local_context.m_selectionPolygon.is2D())
+    qDebug() << "Odd situation where the selection polygon is two-dimensional "
+                "and the trace plot selection is mono-dimensional. Converting "
+                "it to a 1D selection polygon.";
+
+  local_context.m_selectionPolygon.convertTo1D();
+
+  // Sanity check.
+
+  // The user may select at the left of the x=0, but that does not make sense if
+  // the whole selected region is in the x axis negative space.
+  double range_start;
+  double range_end;
+
+  local_context.m_selectionPolygon.rangeX(range_start, range_end);
+
+  qDebug() << "X axis range:" << range_start << "-" << range_end;
+
+  // Having negative drift times does not make any sense.
   if(range_start < 0 && range_end < 0)
     return;
 
@@ -216,7 +234,6 @@ DriftSpecTracePlotCompositeWidget::integrationRequested(
   // selected is the right one. If there are more than one traces selected, then
   // we cannot do anything. Display an error message.
 
-
   // The QCustomPlot-based widget might contain more than one graph, and we
   // need to know on what graph the user is willing to perform the
   // integration.
@@ -248,13 +265,10 @@ DriftSpecTracePlotCompositeWidget::integrationRequested(
 
   // qDebug() << ms_run_data_set_csp->getMsRunDataSetStats().toString();
 
-  // Now we need to add a new step in that processing flow so that we document
-  // this new integration that we are performing.
+  // Create a processing step to document the specifics of this new integration.
 
   ProcessingStep *processing_step_p = new ProcessingStep();
 
-  ProcessingSpec *processing_spec_p = new ProcessingSpec();
-
   MsFragmentationSpec ms_fragmentation_spec =
     local_processing_flow.getDefaultMsFragmentationSpec();
 
@@ -265,7 +279,7 @@ DriftSpecTracePlotCompositeWidget::integrationRequested(
       // flow " "is valid, using it:"
       //<< ms_fragmentation_spec.toString();
 
-      processing_spec_p->setMsFragmentationSpec(ms_fragmentation_spec);
+      processing_step_p->setMsFragmentationSpec(ms_fragmentation_spec);
     }
   else
     {
@@ -273,15 +287,21 @@ DriftSpecTracePlotCompositeWidget::integrationRequested(
       //"valid. Not using it.";
     }
 
-  // Now define the graph range for the integration operation.
+  processing_step_p->setSelectionPolygon(local_context.m_selectionPolygon);
 
-  processing_spec_p->setRange(range_start, range_end);
+  // We must say what is the data source processing type.
+  processing_step_p->setSrcProcessingType(pappso::Axis::x, "DT");
+
+  // It is essential that we document the relation between the axes and the
+  // corresponding data type.
+  processing_step_p->setDataKind(pappso::Axis::x, pappso::DataKind::dt);
+  processing_step_p->setDataKind(pappso::Axis::y, pappso::DataKind::unset);
 
   if(m_ui.integrateToRtPushButton->isChecked())
     {
       // qDebug() << "Integrating to XIC chromatogram.";
 
-      processing_step_p->newSpec(ProcessingType("DT_TO_RT"), processing_spec_p);
+      processing_step_p->setDestProcessingType("RT");
 
       local_processing_flow.push_back(processing_step_p);
 
@@ -290,25 +310,25 @@ DriftSpecTracePlotCompositeWidget::integrationRequested(
     }
   else if(m_ui.integrateToMzPushButton->isChecked() ||
           m_ui.integrateToDtMzPushButton->isChecked() ||
-          m_ui.integrateToRtMzPushButton->isChecked())
+          m_ui.integrateToMzRtPushButton->isChecked())
     {
       if(m_ui.integrateToMzPushButton->isChecked())
         {
-          qDebug() << "Integrating to mass spectrum.";
-          processing_step_p->newSpec(ProcessingType("DT_TO_MZ"),
-                                     processing_spec_p);
+          // qDebug() << "Integrating to mass spectrum.";
+
+          processing_step_p->setDestProcessingType("MZ");
         }
-      else if(m_ui.integrateToRtMzPushButton->isChecked())
+      else if(m_ui.integrateToMzRtPushButton->isChecked())
         {
           // qDebug() << "Integrating to rt|m/z color map.";
-          processing_step_p->newSpec(ProcessingType("DT_TO_RT_MZ"),
-                                     processing_spec_p);
+
+          processing_step_p->setDestProcessingType("MZ_RT");
         }
       else if(m_ui.integrateToDtMzPushButton->isChecked())
         {
           // qDebug() << "Integrating to dt|m/z color map.";
-          processing_step_p->newSpec(ProcessingType("DT_TO_DT_MZ"),
-                                     processing_spec_p);
+
+          processing_step_p->setDestProcessingType("DT_MZ");
         }
 
       // At this point we need to define how the mass data integrator will
@@ -329,8 +349,11 @@ DriftSpecTracePlotCompositeWidget::integrationRequested(
           // qDebug()
           //<< "The processing flow's default mz integ params are not valid.";
 
-          mz_integration_params =
-            *(local_processing_flow.mostRecentMzIntegrationParams());
+          const pappso::MzIntegrationParams *mz_integration_params_p =
+            local_processing_flow.mostRecentMzIntegrationParams();
+
+          if(mz_integration_params_p != nullptr)
+            mz_integration_params = *mz_integration_params_p;
         }
 
       // If it is not valid, finally resort to the parameters that can be
@@ -381,48 +404,40 @@ DriftSpecTracePlotCompositeWidget::integrationRequested(
           static_cast<DriftSpecTracePlotWnd *>(mp_parentWnd)
             ->integrateToDtMz(plottable_p, local_processing_flow);
         }
-      else if(m_ui.integrateToRtMzPushButton->isChecked())
+      else if(m_ui.integrateToMzRtPushButton->isChecked())
         {
           static_cast<DriftSpecTracePlotWnd *>(mp_parentWnd)
-            ->integrateToRtMz(plottable_p, local_processing_flow);
+            ->integrateToMzRt(plottable_p, local_processing_flow);
         }
     }
   else if(m_ui.integrateToDtPushButton->isChecked())
     {
       // qDebug() << "Integrating to drift spectrum.";
 
-      processing_step_p->newSpec(ProcessingType("DT_TO_DT"), processing_spec_p);
+      processing_step_p->setDestProcessingType("DT");
 
       local_processing_flow.push_back(processing_step_p);
 
       static_cast<DriftSpecTracePlotWnd *>(mp_parentWnd)
         ->integrateToDt(plottable_p, nullptr, local_processing_flow);
     }
-  else if(m_ui.integrateToRtDtPushButton->isChecked())
+  else if(m_ui.integrateToDtRtPushButton->isChecked())
     {
       // qDebug() << "Integrating to TIC|XIC chromatogram / drift spectrum
       // colormap.";
 
-      ProcessingSpec *spec_p = new ProcessingSpec();
-
-      processing_step_p->newSpec(ProcessingType("DT_TO_RT_DT"), spec_p);
-
-      // We now need to specify the DT range.
-
-      processing_step_p->newSpec(ProcessingType("DT_TO_ANY"),
-                                 processing_spec_p);
+      processing_step_p->setDestProcessingType("DT_RT");
 
       local_processing_flow.push_back(processing_step_p);
 
       static_cast<DriftSpecTracePlotWnd *>(mp_parentWnd)
-        ->integrateToRtDt(plottable_p, local_processing_flow);
+        ->integrateToDtRt(plottable_p, local_processing_flow);
     }
   else if(m_ui.integrateToIntPushButton->isChecked())
     {
       // qDebug() << "Integrating to TIC intensity.";
 
-      processing_step_p->newSpec(ProcessingType("DT_TO_INT"),
-                                 processing_spec_p);
+      processing_step_p->setDestProcessingType("INT");
 
       local_processing_flow.push_back(processing_step_p);
 
@@ -554,10 +569,10 @@ DriftSpecTracePlotCompositeWidget::plotWidgetKeyPressEvent(
   // qDebug() << "After filling-in by base class, the stanza is:"
   //<< m_analysisStanza;
 
-  m_plotWidgetPressedKeyCode = context.pressedKeyCode;
+  m_plotWidgetPressedKeyCode = context.m_pressedKeyCode;
 
   // Now handle here the mass specific data, like z, Mr, m/z.
-  if(context.pressedKeyCode == Qt::Key_Space)
+  if(context.m_pressedKeyCode == Qt::Key_Space)
     {
       QCPAbstractPlottable *plottable_p =
         plottableToBeUsedAsIntegrationSource();
@@ -594,7 +609,7 @@ DriftSpecTracePlotCompositeWidget::plotWidgetKeyPressEvent(
       mp_parentWnd->recordAnalysisStanza(stanza, plottable_p->pen().color());
     }
   // End of
-  // if(context.baseContext.pressedKeyCode == Qt::Key_Space)
+  // if(context.m_baseContext.pressedKeyCode == Qt::Key_Space)
 }
 
 
@@ -729,7 +744,7 @@ DriftSpecTracePlotCompositeWidget::craftAnalysisStanza(
           if(curChar == 'X')
             {
               stanza += QString("%1").arg(
-                context.lastCursorHoveredPoint.x(), 0, 'g', 6);
+                context.m_lastCursorHoveredPoint.x(), 0, 'g', 6);
               prevChar = ' ';
               continue;
             }
@@ -740,7 +755,7 @@ DriftSpecTracePlotCompositeWidget::craftAnalysisStanza(
               // of the point at X.
               stanza += QString("%1").arg(
                 static_cast<pappso::BaseTracePlotWidget *>(mp_plotWidget)
-                  ->getYatX(context.lastCursorHoveredPoint.x(),
+                  ->getYatX(context.m_lastCursorHoveredPoint.x(),
                             static_cast<QCPGraph *>(plottable_p)),
                 0,
                 'g',
@@ -751,13 +766,13 @@ DriftSpecTracePlotCompositeWidget::craftAnalysisStanza(
             }
           if(curChar == 'x')
             {
-              stanza += QString("%1").arg(context.xDelta, 0, 'g', 6);
+              stanza += QString("%1").arg(context.m_xDelta, 0, 'g', 6);
               prevChar = ' ';
               continue;
             }
           if(curChar == 'y')
             {
-              stanza += QString("%1").arg(context.yDelta, 0, 'g', 6);
+              stanza += QString("%1").arg(context.m_yDelta, 0, 'g', 6);
 
               prevChar = ' ';
               continue;
@@ -771,14 +786,15 @@ DriftSpecTracePlotCompositeWidget::craftAnalysisStanza(
             }
           if(curChar == 'b')
             {
-              stanza += QString("%1").arg(context.xRegionRangeStart, 0, 'f', 3);
+              stanza +=
+                QString("%1").arg(context.m_xRegionRangeStart, 0, 'f', 3);
 
               prevChar = ' ';
               continue;
             }
           if(curChar == 'e')
             {
-              stanza += QString("%1").arg(context.xRegionRangeEnd, 0, 'f', 3);
+              stanza += QString("%1").arg(context.m_xRegionRangeEnd, 0, 'f', 3);
 
               prevChar = ' ';
               continue;
