@@ -111,10 +111,8 @@ DriftSpecTracePlotWnd::integrateToDt(
     qualified_mass_spectra_sp,
   const ProcessingFlow &processing_flow)
 {
-  // qDebug();
-
-  // qDebug().noquote() << "Integrating to dt with processing flow:"
-  //<< processing_flow.toString();
+  qDebug().noquote() << "Integrating to dt with processing flow:"
+                     << processing_flow.toString();
 
   // Get the ms run data set that for the graph we are going to base the
   // integration on.
@@ -123,27 +121,44 @@ DriftSpecTracePlotWnd::integrateToDt(
   if(ms_run_data_set_csp == nullptr)
     qFatal("Cannot be that the pointer is nullptr.");
 
-  // We will try to limit the number of mass spectra to iterate through with the
-  // integration visitor.
+  // FIXME: BEGIN Sanity check that might be removed when the program is
+  // stabilized.
+  std::pair<double, double> range_pair;
 
-  double start_rt = std::numeric_limits<double>::infinity();
-  double end_rt   = std::numeric_limits<double>::infinity();
-
-  bool integration_rt = processing_flow.innermostRtRange(start_rt, end_rt);
+  bool integration_rt = processing_flow.innermostRange("ANY_RT", range_pair);
 
   if(!integration_rt)
     qFatal("Programming error.");
 
-  // First prepare a vector of QualifiedMassSpectrumCstSPtr
+  qDebug() << qSetRealNumberPrecision(10) << "Innermost RT range:"
+           << "for y axis: " << range_pair.first << "-" << range_pair.second;
+
+  bool integration_dt = processing_flow.innermostRange("ANY_DT", range_pair);
+
+  if(integration_dt)
+    qDebug() << qSetRealNumberPrecision(10) << "Innermost DT range:"
+             << "for y axis: " << range_pair.first << "-" << range_pair.second;
+  // FIXME: END Sanity check that might be removed when the program is
+  // stabilized.
+
+  // First prepare a vector of QualifiedMassSpectrumCstSPtr. If the
+  // qualified_mass_spectra_sp is not empty, the
+  // fillInQualifiedMassSpectraVector() function below does not change the
+  // vector and returns its size.
 
   std::size_t qualified_mass_spectra_count = fillInQualifiedMassSpectraVector(
     ms_run_data_set_csp, qualified_mass_spectra_sp, processing_flow);
 
+  qDebug()
+    << "The number of selected mass spectra on the basis of rough RT/DT values:"
+    << qualified_mass_spectra_count;
+
   if(!qualified_mass_spectra_count)
     return;
 
-  // qDebug() << "The number of remaining mass spectra:"
-  //<< qualified_mass_spectra_count;
+  // qDebug() << qSetRealNumberPrecision(10) << "Retention time of first
+  // spectrum:"
+  //<< qualified_mass_spectra_sp->front()->getRtInMinutes();
 
   // Now start the actual integration work.
 
@@ -153,7 +168,8 @@ DriftSpecTracePlotWnd::integrateToDt(
     new QualifiedMassSpectrumVectorMassDataIntegratorToDt(
       ms_run_data_set_csp, processing_flow, qualified_mass_spectra_sp);
 
-  mass_data_integrator_p->setMaxThreadUseCount(mp_programWindow->getMaxThreadUseCount());
+  mass_data_integrator_p->setMaxThreadUseCount(
+    mp_programWindow->getMaxThreadUseCount());
 
   // Ensure the mass data integrator messages are used.
 
@@ -374,7 +390,7 @@ void
 DriftSpecTracePlotWnd::integrateToRt(QCPAbstractPlottable *parent_plottable_p,
                                      const ProcessingFlow &processing_flow)
 {
-  // qDebug();
+  qDebug();
 
   if(parent_plottable_p == nullptr)
     qFatal("Cannot be that pointer is nullptr.");
@@ -415,7 +431,7 @@ DriftSpecTracePlotWnd::integrateToDtMz(QCPAbstractPlottable *parent_plottable_p,
 
 
 void
-DriftSpecTracePlotWnd::integrateToRtMz(QCPAbstractPlottable *parent_plottable_p,
+DriftSpecTracePlotWnd::integrateToMzRt(QCPAbstractPlottable *parent_plottable_p,
                                        const ProcessingFlow &processing_flow)
 {
   // qDebug();
@@ -423,13 +439,13 @@ DriftSpecTracePlotWnd::integrateToRtMz(QCPAbstractPlottable *parent_plottable_p,
   if(parent_plottable_p == nullptr)
     qFatal("Cannot be that pointer is nullptr.");
 
-  mp_programWindow->integrateToRtMz(
+  mp_programWindow->integrateToMzRt(
     parent_plottable_p, nullptr, processing_flow);
 }
 
 
 void
-DriftSpecTracePlotWnd::integrateToRtDt(QCPAbstractPlottable *parent_plottable_p,
+DriftSpecTracePlotWnd::integrateToDtRt(QCPAbstractPlottable *parent_plottable_p,
                                        const ProcessingFlow &processing_flow)
 {
   // qDebug();
@@ -437,7 +453,7 @@ DriftSpecTracePlotWnd::integrateToRtDt(QCPAbstractPlottable *parent_plottable_p,
   if(parent_plottable_p == nullptr)
     qFatal("Cannot be that pointer is nullptr.");
 
-  mp_programWindow->integrateToRtDt(
+  mp_programWindow->integrateToDtRt(
     parent_plottable_p, nullptr, processing_flow);
 }
 
