@@ -1124,10 +1124,10 @@ IsoSpecDlg::addIsotopeFrame()
 
   newIsotopeFrameGridLayout->addWidget(newIsotopeProbDoubleSpinBox, 0, 1, 1, 1);
 
-  QPushButton *plusIsotopePushButton = new QPushButton(newIsotopeFrame);
-  plusIsotopePushButton->setObjectName(QStringLiteral("plusIsotopePushButton"));
+  QPushButton *addIsotopePushButton = new QPushButton(newIsotopeFrame);
+  addIsotopePushButton->setObjectName(QStringLiteral("addIsotopePushButton"));
 
-  connect(plusIsotopePushButton,
+  connect(addIsotopePushButton,
           &QPushButton::clicked,
           this,
           &IsoSpecDlg::addIsotopeFrame);
@@ -1137,33 +1137,33 @@ IsoSpecDlg::addIsotopeFrame()
   sizePolicy.setVerticalStretch(0);
 
   sizePolicy.setHeightForWidth(
-    plusIsotopePushButton->sizePolicy().hasHeightForWidth());
-  plusIsotopePushButton->setSizePolicy(sizePolicy);
+    addIsotopePushButton->sizePolicy().hasHeightForWidth());
+  addIsotopePushButton->setSizePolicy(sizePolicy);
   QIcon icon1;
   icon1.addFile(
-    QStringLiteral(":/images/plus.svg"), QSize(), QIcon::Normal, QIcon::Off);
-  plusIsotopePushButton->setIcon(icon1);
+    QStringLiteral(":/images/svg/add-isotope.svg"), QSize(), QIcon::Normal, QIcon::Off);
+  addIsotopePushButton->setIcon(icon1);
 
-  newIsotopeFrameGridLayout->addWidget(plusIsotopePushButton, 0, 2, 1, 1);
+  newIsotopeFrameGridLayout->addWidget(addIsotopePushButton, 0, 2, 1, 1);
 
-  QPushButton *minusIsotopePushButton = new QPushButton(isotopeFrame);
-  minusIsotopePushButton->setObjectName(
-    QStringLiteral("minusIsotopePushButton"));
+  QPushButton *removeIsotopePushButton = new QPushButton(isotopeFrame);
+  removeIsotopePushButton->setObjectName(
+    QStringLiteral("removeIsotopePushButton"));
 
-  connect(minusIsotopePushButton,
+  connect(removeIsotopePushButton,
           &QPushButton::clicked,
           this,
           &IsoSpecDlg::removeIsotopeFrame);
 
   sizePolicy.setHeightForWidth(
-    minusIsotopePushButton->sizePolicy().hasHeightForWidth());
-  minusIsotopePushButton->setSizePolicy(sizePolicy);
+    removeIsotopePushButton->sizePolicy().hasHeightForWidth());
+  removeIsotopePushButton->setSizePolicy(sizePolicy);
   QIcon icon;
   icon.addFile(
-    QStringLiteral(":/images/minus.svg"), QSize(), QIcon::Normal, QIcon::Off);
-  minusIsotopePushButton->setIcon(icon);
+    QStringLiteral(":/images/svg/remove-isotope.svg"), QSize(), QIcon::Normal, QIcon::Off);
+  removeIsotopePushButton->setIcon(icon);
 
-  newIsotopeFrameGridLayout->addWidget(minusIsotopePushButton, 0, 3, 1, 1);
+  newIsotopeFrameGridLayout->addWidget(removeIsotopePushButton, 0, 3, 1, 1);
 
   elementGroupVBoxLayout->addWidget(newIsotopeFrame);
 
@@ -1255,7 +1255,7 @@ IsoSpecDlg::addElementSkeletonGroupBox()
   minusElementPushButton->setSizePolicy(sizePolicy);
   QIcon icon;
   icon.addFile(
-    QStringLiteral(":/images/minus.svg"), QSize(), QIcon::Normal, QIcon::Off);
+    QStringLiteral(":/images/svg/remove-chemical-element.svg"), QSize(), QIcon::Normal, QIcon::Off);
   minusElementPushButton->setIcon(icon);
 
   symbolLineEditHBoxLayout->addWidget(minusElementPushButton);
@@ -1316,11 +1316,11 @@ IsoSpecDlg::createIsotopeFrame(QGroupBox *elementGroupBox)
 
   isotopeFrameGridLayout->addWidget(isotopeProbDoubleSpinBox, 0, 1, 1, 1);
 
-  QPushButton *plusIsotopePushButton;
-  plusIsotopePushButton = new QPushButton(isotopeFrame);
-  plusIsotopePushButton->setObjectName(QStringLiteral("plusIsotopePushButton"));
+  QPushButton *addIsotopePushButton;
+  addIsotopePushButton = new QPushButton(isotopeFrame);
+  addIsotopePushButton->setObjectName(QStringLiteral("addIsotopePushButton"));
 
-  connect(plusIsotopePushButton,
+  connect(addIsotopePushButton,
           &QPushButton::clicked,
           this,
           &IsoSpecDlg::addIsotopeFrame);
@@ -1329,34 +1329,34 @@ IsoSpecDlg::createIsotopeFrame(QGroupBox *elementGroupBox)
   sizePolicy.setHorizontalStretch(0);
   sizePolicy.setVerticalStretch(0);
   sizePolicy.setHeightForWidth(
-    plusIsotopePushButton->sizePolicy().hasHeightForWidth());
-  plusIsotopePushButton->setSizePolicy(sizePolicy);
+    addIsotopePushButton->sizePolicy().hasHeightForWidth());
+  addIsotopePushButton->setSizePolicy(sizePolicy);
   QIcon icon1;
   icon1.addFile(
-    QStringLiteral(":/images/plus.svg"), QSize(), QIcon::Normal, QIcon::Off);
-  plusIsotopePushButton->setIcon(icon1);
+    QStringLiteral(":/images/svg/add-isotope.svg"), QSize(), QIcon::Normal, QIcon::Off);
+  addIsotopePushButton->setIcon(icon1);
 
-  isotopeFrameGridLayout->addWidget(plusIsotopePushButton, 0, 2, 1, 1);
+  isotopeFrameGridLayout->addWidget(addIsotopePushButton, 0, 2, 1, 1);
 
-  QPushButton *minusIsotopePushButton;
-  minusIsotopePushButton = new QPushButton(isotopeFrame);
-  minusIsotopePushButton->setObjectName(
-    QStringLiteral("minusIsotopePushButton"));
+  QPushButton *removeIsotopePushButton;
+  removeIsotopePushButton = new QPushButton(isotopeFrame);
+  removeIsotopePushButton->setObjectName(
+    QStringLiteral("removeIsotopePushButton"));
 
-  connect(minusIsotopePushButton,
+  connect(removeIsotopePushButton,
           &QPushButton::clicked,
           this,
           &IsoSpecDlg::removeIsotopeFrame);
 
   sizePolicy.setHeightForWidth(
-    minusIsotopePushButton->sizePolicy().hasHeightForWidth());
-  minusIsotopePushButton->setSizePolicy(sizePolicy);
+    removeIsotopePushButton->sizePolicy().hasHeightForWidth());
+  removeIsotopePushButton->setSizePolicy(sizePolicy);
   QIcon icon;
   icon.addFile(
-    QStringLiteral(":/images/minus.svg"), QSize(), QIcon::Normal, QIcon::Off);
-  minusIsotopePushButton->setIcon(icon);
+    QStringLiteral(":/images/svg/remove-isotope.svg"), QSize(), QIcon::Normal, QIcon::Off);
+  removeIsotopePushButton->setIcon(icon);
 
-  isotopeFrameGridLayout->addWidget(minusIsotopePushButton, 0, 3, 1, 1);
+  isotopeFrameGridLayout->addWidget(removeIsotopePushButton, 0, 3, 1, 1);
 
   QVBoxLayout *elementGroupVBoxLayout =
     static_cast<QVBoxLayout *>(elementGroupBox->layout());
