@@ -2,7 +2,7 @@
  *
  * msXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright (C) 2009--2020 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
@@ -265,7 +265,7 @@ MassPeakShaperDlg::fillInThePeakShapers()
 
   QString input_data_string = m_ui.inputDataPointsPlainTextEdit->toPlainText();
   QStringList input_data_string_list =
-    input_data_string.split("\n", QString::SkipEmptyParts);
+    input_data_string.split("\n", Qt::SkipEmptyParts);
 
   QString line;
 
@@ -336,7 +336,7 @@ MassPeakShaperDlg::probeFirstInputDataPoint()
 
   QString input_data_string = m_ui.inputDataPointsPlainTextEdit->toPlainText();
   QStringList input_data_string_list =
-    input_data_string.split("\n", QString::SkipEmptyParts);
+    input_data_string.split("\n", Qt::SkipEmptyParts);
 
   QString line;
 
@@ -374,7 +374,7 @@ MassPeakShaperDlg::sortInputDataPoints()
 
   QString input_data_string = m_ui.inputDataPointsPlainTextEdit->toPlainText();
   QStringList input_data_string_list =
-    input_data_string.split("\n", QString::SkipEmptyParts);
+    input_data_string.split("\n", Qt::SkipEmptyParts);
 
   // The stratey is to convert each line into a DataPoint object that is
   // appended to a vector. Then, the vector is sorted on the basis of the
@@ -808,9 +808,9 @@ MassPeakShaperDlg::run()
     }
 
   m_mzIntegrationParams =
-    MzIntegrationParams(minMz,
+    pappso::MzIntegrationParams(minMz,
                         maxMz,
-                        BinningType::ARBITRARY,
+                        pappso::BinningType::ARBITRARY,
                         -1,
                         pappso::PrecisionFactory::getDaltonInstance(binSize),
                         false,
