@@ -337,6 +337,8 @@ MassSpecTracePlotCompositeWidget::gaussianFitOnVisibleMaxima()
 
   N = local_maxima_trace.size();
 
+  qDebug() << "The number of local maxima: " << N;
+
   double *x     = new double[N];
   double *y     = new double[N];
   double *log_y = new double[N];
@@ -372,7 +374,7 @@ MassSpecTracePlotCompositeWidget::gaussianFitOnVisibleMaxima()
     }
 
   // B is the Normal matrix(augmented) that will store the
-  // equations
+  // equations. So it is an array of arrays.
   double **B = new double *[n + 1];
 
   for(int iter = 0; iter < n + 1; ++iter)
@@ -613,8 +615,14 @@ MassSpecTracePlotCompositeWidget::gaussianFitOnVisibleMaxima()
 
   delete[] a;
 
-  for(int iter = 0; iter < n + 1; ++iter)
-    delete[] B[iter];
+  //qDebug() << "n has value:" << n;
+
+  for(int iter = 0; iter < n; ++iter)
+    {
+      //qDebug() << "Iterating at iter:" << iter;
+      delete[] B[iter];
+      //qDebug() << "deleted B at iter:" << iter;
+    }
 
   // At this point, add the trace ! But only if it has points in it.
 
@@ -626,6 +634,8 @@ MassSpecTracePlotCompositeWidget::gaussianFitOnVisibleMaxima()
       m_ui.statusLineEdit->setText(
         "The Gaussian fit produced an empty trace. Plotting nothing.");
 
+        qDebug() << "The Gaussian fit produced an empty trace. Plotting nothing.";
+
       return;
     }
 
@@ -839,7 +849,7 @@ void
 MassSpecTracePlotCompositeWidget::integrationRequested(
   const pappso::BasePlotContext &context)
 {
-   qDebug().noquote() << "context:" << context.toString();
+  // qDebug().noquote() << "context:" << context.toString();
 
   // The reference implementation is the TicXicChromTracePlotCompositeWidget
   // one. Look there for all the explanative comments.
@@ -869,7 +879,7 @@ MassSpecTracePlotCompositeWidget::integrationRequested(
 
   local_context.m_selectionPolygon.rangeX(range_start, range_end);
 
-  qDebug() << "X axis range:" << range_start << "-" << range_end;
+  // qDebug() << "X axis range:" << range_start << "-" << range_end;
 
   // Having negative retention times does not make any sense.
   if(range_start < 0 && range_end < 0)
@@ -1052,7 +1062,7 @@ MassSpecTracePlotCompositeWidget::integrationRequested(
     }
   else if(m_ui.integrateToDtPushButton->isChecked())
     {
-       qDebug() << "Integrating to drift spectrum.";
+      // qDebug() << "Integrating to drift spectrum.";
 
       processing_step_p->setDestProcessingType("DT");
 
@@ -1110,6 +1120,8 @@ MassSpecTracePlotCompositeWidget::addTrace(
   const QColor &color)
 {
 
+  // qDebug();
+
   // If color is not valid, we need to create one.
 
   QColor local_color(color);
@@ -1145,6 +1157,9 @@ MassSpecTracePlotCompositeWidget::addTrace(
   QList<QCPAbstractPlottable *> dest_plottable_list =
     plottablesToBeUsedAsIntegrationDestination();
 
+  // qDebug() << "The destination plottable size has "
+  //<< dest_plottable_list.size() << "graphs.";
+
   if(m_newTraceHandlingMethod == NewTraceHandlingMethod::OVERLAY)
     {
       // Keep the traces there and simply add a new one.
@@ -1184,6 +1199,8 @@ MassSpecTracePlotCompositeWidget::addTrace(
           if(mz_integration_params_p->getBinningType() ==
              pappso::BinningType::NONE)
             {
+              // qDebug() << "Binning type none.";
+
               pappso::TraceMinusCombiner trace_minus_combiner;
               pappso::TracePlusCombiner trace_plus_combiner;
 
@@ -1222,22 +1239,22 @@ MassSpecTracePlotCompositeWidget::addTrace(
                   QVector<double> key_vector;
                   QVector<double> value_vector;
 
-#pragma GCC warning "Filippo Rusconi: Please check if the bug was fixed in Qt"
+                  // Apparently, as of 20210928, that bug is fixed.
+                  //#pragma GCC warning "Filippo Rusconi: Please check if the
+                  // bug was fixed in Qt"
 
-#if 0
-                    // Now replace the graph's data. Note that the data are
-                    // inherently sorted (true below).
+                  // Now replace the graph's data. Note that the data are
+                  // inherently sorted (true below).
 
-                    // The begin() -- end() ranges constructor did not work as of
-                    // Qt 5.14.2 this day: 20200721
+                  // The begin() -- end() ranges constructor did not work as of
+                  // Qt 5.14.2 this day: 20200721 -- and still does not work
+                  // 20211018.
+
+                  // key_vector   = QVector(map_trace.xValues().begin(),
+                  // map_trace.xValues().end());
+                  // value_vector = QVector(map_trace.yValues().begin(),
+                  // map_trace.yValues().end());
 
-                    key_vector =
-                        QVector(map_trace.firstToVector().begin(),
-                                map_trace.firstToVector().end());
-                    value_vector =
-                        QVector(map_trace.secondToVector().begin(),
-                                map_trace.secondToVector().end());
-#endif
 
                   for(auto &value : map_trace.xValues())
                     key_vector.push_back(value);
@@ -1252,6 +1269,7 @@ MassSpecTracePlotCompositeWidget::addTrace(
           else if(mz_integration_params_p->getBinningType() ==
                   pappso::BinningType::ARBITRARY)
             {
+              // qDebug() << "Binning type arbitrary.";
               pappso::MassSpectrumMinusCombiner mass_spectrum_minus_combiner;
               pappso::MassSpectrumPlusCombiner mass_spectrum_plus_combiner;
 
@@ -1307,13 +1325,13 @@ MassSpecTracePlotCompositeWidget::addTrace(
                   if(m_newTraceHandlingMethod ==
                      NewTraceHandlingMethod::MINUS_COMBINE)
                     {
-                      qDebug() << "NewTraceHandlingMethod::MINUS_COMBINE";
+                      // qDebug() << "NewTraceHandlingMethod::MINUS_COMBINE";
                       mass_spectrum_minus_combiner.setBins(bins);
                     }
                   else if(m_newTraceHandlingMethod ==
                           NewTraceHandlingMethod::PLUS_COMBINE)
                     {
-                      qDebug() << "NewTraceHandlingMethod::PLUS_COMBINE";
+                      // qDebug() << "NewTraceHandlingMethod::PLUS_COMBINE";
                       mass_spectrum_plus_combiner.setBins(bins);
                     }
 
@@ -1322,13 +1340,13 @@ MassSpecTracePlotCompositeWidget::addTrace(
                   if(m_newTraceHandlingMethod ==
                      NewTraceHandlingMethod::MINUS_COMBINE)
                     {
-                      qDebug() << "NewTraceHandlingMethod::MINUS_COMBINE";
+                      // qDebug() << "NewTraceHandlingMethod::MINUS_COMBINE";
                       mass_spectrum_minus_combiner.combine(map_trace, trace);
                     }
                   else if(m_newTraceHandlingMethod ==
                           NewTraceHandlingMethod::PLUS_COMBINE)
                     {
-                      qDebug() << "NewTraceHandlingMethod::PLUS_COMBINE";
+                      // qDebug() << "NewTraceHandlingMethod::PLUS_COMBINE";
                       mass_spectrum_plus_combiner.combine(map_trace, trace);
                     }
 
@@ -1347,22 +1365,24 @@ MassSpecTracePlotCompositeWidget::addTrace(
                   QVector<double> key_vector;
                   QVector<double> value_vector;
 
-#pragma GCC warning "Filippo Rusconi: Please check if the bug was fixed in Qt"
+                  // Apparently, as of 20210928, that bug is fixed.
+                  //#pragma GCC warning "Filippo Rusconi: Please check if the
+                  // bug was fixed in Qt"
 
-#if 0
-                    // Now replace the graph's data. Note that the data are
-                    // inherently sorted (true below).
+                  // Now replace the graph's data. Note that the data are
+                  // inherently sorted (true below).
 
-                    // The begin() -- end() ranges constructor did not work as of
-                    // Qt 5.14.2 this day: 20200721
+                  // The begin() -- end() ranges constructor did not work as of
+                  // Qt 5.14.2 this day: 20200721 -- and still does not work
+                  // 20211018.
 
-                    key_vector =
-                        QVector(map_trace.firstToVector().begin(),
-                                map_trace.firstToVector().end());
-                    value_vector =
-                        QVector(map_trace.secondToVector().begin(),
-                                map_trace.secondToVector().end());
-#endif
+
+                  // key_vector   = QVector(map_trace.xValues().begin(),
+                  // map_trace.xValues().end());
+                  // value_vector = QVector(map_trace.yValues().begin(),
+                  // map_trace.yValues().end());
+
+                  // This is the workaround fix.
 
                   for(auto &value : map_trace.xValues())
                     key_vector.push_back(value);
