@@ -442,6 +442,12 @@ MsRunDataSetTableViewWnd::ticIntensityValue(double intensity_value)
 void
 MsRunDataSetTableViewWnd::keyPressEvent(QKeyEvent *event)
 {
+  // Select all the row in one key shortcut.
+  if(event->key() == Qt::Key_A && event->modifiers() & Qt::ControlModifier)
+    {
+      mp_msRunDataSetTableView->selectAll();
+    }
+
   // If Ctrl return, run the filter
 
   if(event->key() == Qt::Key_Return &&
@@ -579,8 +585,8 @@ MsRunDataSetTableViewWnd::recordQualifiedMassSpectraToIntegrate(
           greatest_dt_value = std::max(greatest_dt_value, dt_value);
         }
 
-      //qDebug() << qSetRealNumberPrecision(10) << "From the table view: "
-               //<< "rt_value: " << rt_value << "dt_value: " << dt_value;
+      // qDebug() << qSetRealNumberPrecision(10) << "From the table view: "
+      //<< "rt_value: " << rt_value << "dt_value: " << dt_value;
 
       // Finally store the pointer to the qualified mass spectrum.
       vector_of_qualified_mass_spectra.push_back(qualified_mass_spectrum_csp);
@@ -653,9 +659,9 @@ MsRunDataSetTableViewWnd::recordQualifiedMassSpectraToIntegrate(
       // Craft a 2D polygon with x=DT and y=RT (alphabetical sorting of the
       // axes).
 
-      //qDebug() << qSetRealNumberPrecision(50)
-               //<< "rt difference:" << end_rt - begin_rt
-               //<< "dt difference:" << end_dt - begin_dt;
+      // qDebug() << qSetRealNumberPrecision(50)
+      //<< "rt difference:" << end_rt - begin_rt
+      //<< "dt difference:" << end_dt - begin_dt;
 
       selection_polygon.set2D(QPointF(begin_dt, end_rt),
                               QPointF(end_dt, end_rt),
@@ -703,8 +709,7 @@ MsRunDataSetTableViewWnd::recordQualifiedMassSpectraToIntegrate(
 
       processing_step.setSrcProcessingType(pappso::Axis::x,
                                            "DATA_TABLE_VIEW_RT");
-      processing_step.setSrcProcessingType(pappso::Axis::y,
-                                           "NOT_SET");
+      processing_step.setSrcProcessingType(pappso::Axis::y, "NOT_SET");
     }
 
   // And now configure the destination type.
