@@ -83,17 +83,20 @@ class MsRunDataSetTableViewWnd : public QMainWindow
 
   virtual ~MsRunDataSetTableViewWnd();
 
-  bool initialize();
   void closeEvent(QCloseEvent *event);
+
   void writeSettings();
   void readSettings();
 
+  bool initialize();
+
   void show();
 
   void executeFilteringPushButtonClicked();
 
   void showMzIntegrationParamsDlg();
-  void mzIntegrationParamsChanged(pappso::MzIntegrationParams mz_integration_params);
+  void
+  mzIntegrationParamsChanged(pappso::MzIntegrationParams mz_integration_params);
 
   public slots:
 
@@ -137,23 +140,13 @@ class MsRunDataSetTableViewWnd : public QMainWindow
 
   bool recordMsFragmentationSpec();
 
-  std::vector<pappso::QualifiedMassSpectrumCstSPtr>
-  recordQualifiedMassSpectraToIntegrate(double &begin_rt,
-                                        double &end_rt,
-                                        double &begin_dt,
-                                        double &end_dt) const;
-  void integrationRangesForRtAndDt(double &begin_rt,
-                                   double &end_rt,
-                                   double &begin_dt,
-                                   double &end_dt) const;
+  void recordQualifiedMassSpectraToIntegrate(
+    std::vector<pappso::QualifiedMassSpectrumCstSPtr>
+      &vector_of_qualified_mass_spectra,
+    ProcessingStep &processing_step,
+    const QString &integration_type) const;
 
-  void integrateToMz();
-  void integrateToDt();
-  void integrateToRt();
-  void integrateToTicIntensity();
-  void integrateToDtMz();
-  void integrateToRtDt();
-  void integrateToRtMz();
+  void integrate(const QString &integration_type);
 };
 
 
