@@ -147,7 +147,7 @@ OpenMsRunDataSetsDlg::createMainMenu()
 
   // Create the menu push button.
   const QIcon main_menu_icon =
-    QIcon(":/images/mobile-phone-like-menu-button.svg");
+    QIcon(":/images/svg/mobile-phone-like-menu-button.svg");
   m_ui.mainMenuPushButton->setIcon(main_menu_icon);
 
   // Finally set the push button to be the menu...
