@@ -152,95 +152,47 @@ class ProgramWindow : public QMainWindow
   void seedInitialTicChromatogramAndMsRunDataSetStatistics(
     MsRunDataSetSPtr &ms_run_data_set_sp);
 
-  void integrateToRt(
+  using QualifiedMassSpectraVector =
+    std::vector<pappso::QualifiedMassSpectrumCstSPtr>;
+
+  // To trace integrations
+  void integrateToDt(
     QCPAbstractPlottable *parent_plottable_p,
-    std::shared_ptr<std::vector<pappso::QualifiedMassSpectrumCstSPtr>>
-      qualified_mass_spectra_sp,
+    std::shared_ptr<QualifiedMassSpectraVector> qualified_mass_spectra_sp,
     const ProcessingFlow &processing_flow);
 
   void integrateToMz(
     QCPAbstractPlottable *parent_plottable_p,
-    std::shared_ptr<std::vector<pappso::QualifiedMassSpectrumCstSPtr>>
-      qualified_mass_spectra_sp,
+    std::shared_ptr<QualifiedMassSpectraVector> qualified_mass_spectra_sp,
     const ProcessingFlow &processing_flow);
 
-  void integrateToDt(
+  void integrateToRt(
     QCPAbstractPlottable *parent_plottable_p,
-    std::shared_ptr<std::vector<pappso::QualifiedMassSpectrumCstSPtr>>
-      qualified_mass_spectra_sp,
+    std::shared_ptr<QualifiedMassSpectraVector> qualified_mass_spectra_sp,
     const ProcessingFlow &processing_flow);
 
+  // To TIC intensity integration
   void integrateToTicIntensity(
     QCPAbstractPlottable *parent_plottable_p,
-    std::shared_ptr<std::vector<pappso::QualifiedMassSpectrumCstSPtr>>
-      qualified_mass_spectra_sp,
+    std::shared_ptr<QualifiedMassSpectraVector> qualified_mass_spectra_sp,
     const ProcessingFlow &processing_flow);
 
+  // To color map integrations
   void integrateToDtMz(
     QCPAbstractPlottable *parent_plottable_p,
-    std::shared_ptr<std::vector<pappso::QualifiedMassSpectrumCstSPtr>>
-      qualified_mass_spectra_sp,
+    std::shared_ptr<QualifiedMassSpectraVector> qualified_mass_spectra_sp,
     const ProcessingFlow &processing_flow);
 
-  void integrateToRtMz(
+  void integrateToDtRt(
     QCPAbstractPlottable *parent_plottable_p,
-    std::shared_ptr<std::vector<pappso::QualifiedMassSpectrumCstSPtr>>
-      qualified_mass_spectra_sp,
+    std::shared_ptr<QualifiedMassSpectraVector> qualified_mass_spectra_sp,
     const ProcessingFlow &processing_flow);
 
-  void integrateToRtDt(
+  void integrateToMzRt(
     QCPAbstractPlottable *parent_plottable_p,
-    std::shared_ptr<std::vector<pappso::QualifiedMassSpectrumCstSPtr>>
-      qualified_mass_spectra_sp,
+    std::shared_ptr<QualifiedMassSpectraVector> qualified_mass_spectra_sp,
     const ProcessingFlow &processing_flow);
 
-  using QualifiedMassSpectraVector =
-    std::vector<pappso::QualifiedMassSpectrumCstSPtr>;
-
-  void integrateFromMsRunDataSetTableViewToRt(
-    MsRunDataSetCstSPtr ms_run_data_set_csp,
-    std::shared_ptr<QualifiedMassSpectraVector>
-      vector_of_qualified_mass_spectra_sp,
-    const ProcessingFlow &processing_flow,
-    const QColor &color);
-
-  void integrateFromMsRunDataSetTableViewToDt(
-    MsRunDataSetCstSPtr ms_run_data_set_csp,
-    std::shared_ptr<QualifiedMassSpectraVector>
-      vector_of_qualified_mass_spectra_sp,
-    const ProcessingFlow &processing_flow,
-    const QColor &color);
-
-  void integrateFromMsRunDataSetTableViewToMz(
-    MsRunDataSetCstSPtr ms_run_data_set_csp,
-    std::shared_ptr<std::vector<pappso::QualifiedMassSpectrumCstSPtr>>
-      qualified_mass_spectra_vector_sp,
-    const ProcessingFlow &processing_flow,
-    const QColor &color);
-
-  void integrateFromMsRunDataSetTableViewToTicIntensity(
-    MsRunDataSetCstSPtr ms_run_data_set_csp,
-    std::shared_ptr<QualifiedMassSpectraVector>
-      vector_of_qualified_mass_spectra_sp,
-    const ProcessingFlow &processing_flow,
-    const QColor &color);
-
-  void integrateFromMsRunDataSetTableViewToDtRtMz(
-    pappso::DataKind data_kind,
-    MsRunDataSetCstSPtr ms_run_data_set_csp,
-    std::shared_ptr<QualifiedMassSpectraVector>
-      vector_of_qualified_mass_spectra_sp,
-    const ProcessingFlow &processing_flow,
-    const QColor &color);
-
-  void integrateFromMsRunDataSetTableViewToRtDt(
-    pappso::DataKind data_kind,
-    MsRunDataSetCstSPtr ms_run_data_set_csp,
-    std::shared_ptr<QualifiedMassSpectraVector>
-      vector_of_qualified_mass_spectra_sp,
-    const ProcessingFlow &processing_flow,
-    const QColor &color);
-
   void xicIntegrationToRt(const ProcessingFlow &processing_flow);
 
   void displayMassSpectralTrace(pappso::Trace trace,
@@ -277,31 +229,21 @@ class ProgramWindow : public QMainWindow
   void finishedSeedingInitialTicChromatogramAndMsRunDataSetStatistics(
     MassDataIntegrator *mass_data_integrator_p);
 
-  void finishedXicIntegrationToRt(MassDataIntegrator *mass_data_integrator_p);
-
-  void finishedIntegratingQualifiedMassSpectrumVectorToRt(
-    QualifiedMassSpectrumVectorMassDataIntegrator *mass_data_integrator_p);
-
-  void finishedIntegratingQualifiedMassSpectrumVectorToDt(
-    QualifiedMassSpectrumVectorMassDataIntegrator *mass_data_integrator_p);
-
-  void finishedIntegratingQualifiedMassSpectrumVectorToMz(
-    QualifiedMassSpectrumVectorMassDataIntegrator *mass_data_integrator_p);
-
   void finishedIntegratingQualifiedMassSpectrumVectorToTicIntensity(
     QualifiedMassSpectrumVectorMassDataIntegrator *mass_data_integrator_p);
 
-  void finishedIntegratingQualifiedMassSpectrumVectorToDtRtMz(
-    QualifiedMassSpectrumVectorMassDataIntegrator *mass_data_integrator_p);
-
-  void finishedIntegratingQualifiedMassSpectrumVectorToRtDt(
-    QualifiedMassSpectrumVectorMassDataIntegrator *mass_data_integrator_p);
+  void finishedXicIntegrationToRt(MassDataIntegrator *mass_data_integrator_p);
 
   void logTextToConsole(QString msg);
   void logColoredTextToConsole(QString text, const QColor &color);
 
   QColor getColorForMsRunDataSet(MsRunDataSetCstSPtr ms_run_data_set_csp) const;
 
+  QFile *getAnalysisFilePtr();
+  AnalysisPreferences *getAnalysisPreferences();
+
+  void recordAnalysisStanza(QString stanza, const QColor &color = Qt::black);
+
   signals:
 
   void readMsRunDataSignal(MsRunReadTask *ms_run_read_task_p,
@@ -328,13 +270,12 @@ class ProgramWindow : public QMainWindow
       *mass_data_integrator_p);
 
   void integrateQualifiedMassSpectrumVectorToRtDtSignal(
-    QualifiedMassSpectrumVectorMassDataIntegratorToRtDt *mass_data_integrator_p,
-    pappso::DataKind data_kind);
+    QualifiedMassSpectrumVectorMassDataIntegratorToRtDt
+      *mass_data_integrator_p);
 
   void integrateQualifiedMassSpectrumVectorToDtRtMzSignal(
     QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz
-      *mass_data_integrator_p,
-    pappso::DataKind data_kind);
+      *mass_data_integrator_p);
 
   void ticIntensityValueSignal(double tic_intensity);
 
@@ -385,14 +326,10 @@ class ProgramWindow : public QMainWindow
   IsoSpecDlg *mp_isoSpecDlg = nullptr;
 
   AnalysisPreferences *mpa_analysisPreferences = nullptr;
-  AnalysisPreferences *getAnalysisPreferences();
 
   AnalysisPreferencesDlg *mp_analysisPreferencesDlg = nullptr;
 
   QFile *mpa_analysisFile = nullptr;
-  QFile *getAnalysisFilePtr();
-
-  void recordAnalysisStanza(QString stanza, const QColor &color = Qt::black);
 
   AboutDlg *mp_aboutDlg = nullptr;
 
@@ -404,7 +341,7 @@ class ProgramWindow : public QMainWindow
   QMenu *mp_fileMenu                = nullptr;
   QMenu *mp_windowsMenu             = nullptr;
   QMenu *mp_utilitiesMenu           = nullptr;
-  QMenu *mp_preferencesMenu           = nullptr;
+  QMenu *mp_preferencesMenu         = nullptr;
   QMenu *mp_helpMenu                = nullptr;
   QAction *mp_openFullMsFileAct     = nullptr;
   QAction *mp_openStreamedMsFileAct = nullptr;
