@@ -2,7 +2,7 @@
  *
  * msXpertSuite - mass spectrometry software suite
  * -----------------------------------------------
- * Copyright(C) 2009,...,2018 Filippo Rusconi
+ * Copyright (C) 2009--2020 Filippo Rusconi
  *
  * http://www.msxpertsuite.org
  *
@@ -403,7 +403,9 @@ TaskMonitorCompositeWidget::shouldUpdate()
 
 {
 
-  //#if 0
+  bool result = true;
+
+//#if 0
   std::chrono::time_point<std::chrono::system_clock> now =
     std::chrono::system_clock::now();
 
@@ -413,13 +415,13 @@ TaskMonitorCompositeWidget::shouldUpdate()
   if(time_lag.count() >= m_timeLag)
     {
       m_lastTimePoint = now;
-      return true;
+      result          = true;
     }
 
-  return false;
-  //#endif
+  //qDebug() << "Returning:" << result;
+//#endif
 
-  // return true;
+  return result;
 }
 
 
@@ -558,6 +560,8 @@ TaskMonitorCompositeWidget::getTaskDescriptionText()
 void
 TaskMonitorCompositeWidget::setStatusText(QString text)
 {
+  //qDebug() << "m_isLocked:" << m_isLocked << "shouldUpdate:" << shouldUpdate();
+
   if(!m_isLocked && shouldUpdate())
     m_ui.statusLineEdit->setText(text);
 }
@@ -674,7 +678,7 @@ TaskMonitorCompositeWidget::craftStringIfFormattedText(
       return text;
     }
 
-  QStringList string_list = text.split("%c", QString::KeepEmptyParts);
+  QStringList string_list = text.split("%c", Qt::KeepEmptyParts);
 
   QString new_text;
 
