@@ -78,7 +78,7 @@ TicXicChromDriftSpecColorMapWnd::~TicXicChromDriftSpecColorMapWnd()
 
 QCPColorMap *
 TicXicChromDriftSpecColorMapWnd::addColorMapPlot(
-  std::shared_ptr<std::map<double, pappso::MapTrace>> &double_map_trace_map_sp,
+  std::shared_ptr<std::map<double, pappso::MapTrace>> &rt_maptrace_map,
   const pappso::ColorMapPlotConfig &color_map_plot_config,
   MsRunDataSetCstSPtr ms_run_data_set_csp,
   const ProcessingFlow &processing_flow,
@@ -105,7 +105,7 @@ TicXicChromDriftSpecColorMapWnd::addColorMapPlot(
     }
 
   return finalNewColorMapPlotConfiguration(composite_widget_p,
-                                           double_map_trace_map_sp,
+                                           rt_maptrace_map,
                                            color_map_plot_config,
                                            ms_run_data_set_csp,
                                            processing_flow,
@@ -115,13 +115,13 @@ TicXicChromDriftSpecColorMapWnd::addColorMapPlot(
 
 
 void
-TicXicChromDriftSpecColorMapWnd::integrateToRtDt(
+TicXicChromDriftSpecColorMapWnd::integrateToDtRt(
   QCPAbstractPlottable *parent_plottable_p,
   std::shared_ptr<std::vector<pappso::QualifiedMassSpectrumCstSPtr>>
     qualified_mass_spectra_sp,
   const ProcessingFlow &processing_flow)
 {
-  // qDebug().noquote() << "Integrating to rt | mz with processing flow:"
+   //qDebug().noquote() << "Integrating to dt | rt with processing flow:"
   //<< processing_flow.toString();
 
   // Get the ms run data set that for the graph we are going to base the
@@ -131,30 +131,40 @@ TicXicChromDriftSpecColorMapWnd::integrateToRtDt(
   if(ms_run_data_set_csp == nullptr)
     qFatal("Cannot be that the pointer is nullptr.");
 
-  // We will try to limit the number of mass spectra to iterate through with the
-  // integration visitor.
+  // FIXME: BEGIN Sanity check that might be removed when the program is stabilized.
+  std::pair<double, double> range_pair;
 
-  double start_rt = std::numeric_limits<double>::infinity();
-  double end_rt   = std::numeric_limits<double>::infinity();
-
-  bool integration_rt = processing_flow.innermostRtRange(start_rt, end_rt);
+  bool integration_rt = processing_flow.innermostRange("ANY_RT", range_pair);
 
   if(!integration_rt)
     qFatal("Programming error.");
 
-  qDebug() << "Innermost RT range:" << start_rt << "-" << end_rt;
+  qDebug() << qSetRealNumberPrecision(10) << "Innermost RT range:"
+           << "for y axis: " << range_pair.first << "-" << range_pair.second;
 
-  // First prepare a vector of QualifiedMassSpectrumCstSPtr
+  bool integration_dt = processing_flow.innermostRange("ANY_DT", range_pair);
+
+  if(integration_dt)
+    qDebug() << qSetRealNumberPrecision(10) << "Innermost DT range:"
+             << "for y axis: " << range_pair.first << "-" << range_pair.second;
+
+  // FIXME: END Sanity check that might be removed when the program is stabilized.
+
+  // First prepare a vector of QualifiedMassSpectrumCstSPtr. If the
+  // qualified_mass_spectra_sp is not empty, the
+  // fillInQualifiedMassSpectraVector() function below does not change the
+  // vector and returns its size.
 
   std::size_t qualified_mass_spectra_count = fillInQualifiedMassSpectraVector(
     ms_run_data_set_csp, qualified_mass_spectra_sp, processing_flow);
 
+  qDebug()
+    << "The number of selected mass spectra on the basis of rough RT/DT values:"
+    << qualified_mass_spectra_count;
+
   if(!qualified_mass_spectra_count)
     return;
 
-  qDebug() << "The number of remaining mass spectra:"
-           << qualified_mass_spectra_count;
-
   // Now start the actual integration work.
 
   // Allocate a mass data integrator to integrate the data.
@@ -163,7 +173,8 @@ TicXicChromDriftSpecColorMapWnd::integrateToRtDt(
     new QualifiedMassSpectrumVectorMassDataIntegratorToRtDt(
       ms_run_data_set_csp, processing_flow, qualified_mass_spectra_sp);
 
-  mass_data_integrator_p->setMaxThreadUseCount(mp_programWindow->getMaxThreadUseCount());
+  mass_data_integrator_p->setMaxThreadUseCount(
+    mp_programWindow->getMaxThreadUseCount());
 
   // Ensure the mass data integrator messages are used.
 
@@ -176,24 +187,20 @@ TicXicChromDriftSpecColorMapWnd::integrateToRtDt(
   MassDataIntegratorTask *mass_data_integrator_task_p =
     new MassDataIntegratorTask();
 
+  qRegisterMetaType<std::size_t>("std::size_t");
   qRegisterMetaType<pappso::DataKind>("pappso::DataKind");
 
   // This signal starts the computation in the MassDataIntegratorTask object.
-  connect(
-    this,
+  connect(this,
 
-    static_cast<void (TicXicChromDriftSpecColorMapWnd::*)(
-      QualifiedMassSpectrumVectorMassDataIntegratorToRtDt *, pappso::DataKind)>(
-      &TicXicChromDriftSpecColorMapWnd::integrateToRtDtSignal),
+          &TicXicChromDriftSpecColorMapWnd::integrateToDtRtSignal,
 
-    mass_data_integrator_task_p,
+          mass_data_integrator_task_p,
 
-    static_cast<void (MassDataIntegratorTask::*)(
-      QualifiedMassSpectrumVectorMassDataIntegratorToRtDt *, pappso::DataKind)>(
-      &MassDataIntegratorTask::integrateToRtDt),
+          &MassDataIntegratorTask::integrateToDtRt,
 
-    // Fundamental for signals that travel across QThread instances...
-    Qt::QueuedConnection);
+          // Fundamental for signals that travel across QThread instances...
+          Qt::QueuedConnection);
 
   // Allocate the thread in which the integrator task will run.
   QThread *thread_p = new QThread;
@@ -227,7 +234,7 @@ TicXicChromDriftSpecColorMapWnd::integrateToRtDt(
             // the thread and ask for it to also be deleted later.
             thread_p->deleteLater(), thread_p->quit();
             thread_p->wait();
-            this->finishedIntegratingToRtDt(mass_data_integrator_p,
+            this->finishedIntegratingToDtRt(mass_data_integrator_p,
                                             parent_plottable_p);
           });
 
@@ -281,11 +288,11 @@ TicXicChromDriftSpecColorMapWnd::integrateToRtDt(
   task_monitor_composite_widget_p->makeMassDataIntegratorConnections(
     mass_data_integrator_p);
 
-  // qDebug() << "going to emit integrateToRtDtMzSignal with mass "
+  // qDebug() << "going to emit integrateToDtRtMzSignal with mass "
   //"data integrator:"
   //<< mass_data_integrator_p;
 
-  emit integrateToRtDtSignal(mass_data_integrator_p, pappso::DataKind::rt);
+  emit integrateToDtRtSignal(mass_data_integrator_p);
 
   // We do not want to make signal/slot calls more than once. This is because
   // one user might well trigger more than one integration from this window to a
@@ -297,20 +304,16 @@ TicXicChromDriftSpecColorMapWnd::integrateToRtDt(
   disconnect(
     this,
 
-    static_cast<void (TicXicChromDriftSpecColorMapWnd::*)(
-      QualifiedMassSpectrumVectorMassDataIntegratorToRtDt *, pappso::DataKind)>(
-      &TicXicChromDriftSpecColorMapWnd::integrateToRtDtSignal),
+    &TicXicChromDriftSpecColorMapWnd::integrateToDtRtSignal,
 
     mass_data_integrator_task_p,
 
-    static_cast<void (MassDataIntegratorTask::*)(
-      QualifiedMassSpectrumVectorMassDataIntegratorToRtDt *, pappso::DataKind)>(
-      &MassDataIntegratorTask::integrateToRtDt));
+    &MassDataIntegratorTask::integrateToDtRt);
 }
 
 
 void
-TicXicChromDriftSpecColorMapWnd::finishedIntegratingToRtDt(
+TicXicChromDriftSpecColorMapWnd::finishedIntegratingToDtRt(
   QualifiedMassSpectrumVectorMassDataIntegrator *mass_data_integrator_p,
   QCPAbstractPlottable *parent_plottable_p)
 {
@@ -337,14 +340,24 @@ TicXicChromDriftSpecColorMapWnd::finishedIntegratingToRtDt(
 
   // Get a pointer to the double maptrace map that will serve as the data to
   // fill-in the color map.
-  std::shared_ptr<std::map<double, pappso::MapTrace>> double_map_trace_map_sp =
-    mass_data_integrator_to_dtrt_p->getDoubleMapTraceMapSPtr();
+  
+  // The logic here is that in IM-MS experiments, for a given retention time
+  // (rt), the might be hundrds of drift time-resolvec mass spectra. So we
+  // create a map that relates rt values with a MapTrace, itself relating dt
+  // values (map key) and TICint values (map value). During iterating in all the
+  // mass spectra, we look at the rt value of the mass spectrum, we then look at
+  // its dt value. We compute a TICint of the mass spectrum that we set in the
+  // MapTrace, associated with the dt value. That MapTrace is then set the
+  // member map that maps the rt value (map key) with the MapTrace (map value).
 
-  // qDebug() << "rt / mz map size:" << double_map_trace_map_sp->size();
+   std::shared_ptr<std::map<double, pappso::MapTrace>> rt_maptrace_map =
+    mass_data_integrator_to_dtrt_p->getRtMapTraceMapSPtr();
 
-  if(!double_map_trace_map_sp->size())
+  // qDebug() << "rt / maptrace map size:" << rt_maptrace_map->size();
+
+  if(!rt_maptrace_map->size())
     {
-      qDebug() << "There is not a single item in the dt / rt map.";
+      qDebug() << "There is not a single item in the dt / maptrace map.";
       return;
     }
 
@@ -353,23 +366,40 @@ TicXicChromDriftSpecColorMapWnd::finishedIntegratingToRtDt(
   // the minimum/maximum mz value.
 
   pappso::ColorMapPlotConfig color_map_plot_config(
+      // x data kind
     pappso::DataKind::rt,
+      // y data kind
     pappso::DataKind::dt,
+
+      // x axis scale
     pappso::AxisScale::orig,
+      // y axis scale
     pappso::AxisScale::orig,
+      // z axis scale
     pappso::AxisScale::orig,
+
     mass_data_integrator_to_dtrt_p->getColorMapKeyCellCount(),
     mass_data_integrator_to_dtrt_p->getColorMapMzCellCount(),
+
+    // min rt value
     mass_data_integrator_to_dtrt_p->getColorMapMinKey(),
+    // max rt value
     mass_data_integrator_to_dtrt_p->getColorMapMaxKey(),
+
+    // smallest m/z value
     mass_data_integrator_to_dtrt_p->getColorMapMinMz(),
-    mass_data_integrator_to_dtrt_p->getColorMapMaxMz());
+    // greatest m/z value
+    mass_data_integrator_to_dtrt_p->getColorMapMaxMz(),
+    /*the min Z value will be determined later*/ 0,
+    /*the max Z value will be determined later*/ 0);
 
   // Also, do not forget to copy the processing flow from the integrator. This
   // processing flow will be set into the plot widget !
 
   ProcessingFlow processing_flow = mass_data_integrator_p->getProcessingFlow();
 
+  //qDebug() << "The processing flow after integration to dt|rt:" << processing_flow.toString();
+
   // Sanity check
   if(ms_run_data_set_csp != processing_flow.getMsRunDataSetCstSPtr())
     qFatal("Cannot be that the ms run data set pointers be different.");
@@ -396,7 +426,7 @@ TicXicChromDriftSpecColorMapWnd::finishedIntegratingToRtDt(
 
   // Note that we do not support multiple color maps stacked one on top of the
   // other.
-  addColorMapPlot(double_map_trace_map_sp,
+  addColorMapPlot(rt_maptrace_map,
                   color_map_plot_config,
                   ms_run_data_set_csp,
                   processing_flow,
