@@ -163,9 +163,9 @@ TicXicChromTracePlotCompositeWidget::plotWidgetKeyPressEvent(
   // char was pressed.
   BaseTracePlotCompositeWidget::plotWidgetKeyPressEvent(context);
 
-  m_plotWidgetPressedKeyCode = context.pressedKeyCode;
+  m_plotWidgetPressedKeyCode = context.m_pressedKeyCode;
 
-  if(context.pressedKeyCode == Qt::Key_Space)
+  if(context.m_pressedKeyCode == Qt::Key_Space)
     {
       QCPAbstractPlottable *plottable_p =
         plottableToBeUsedAsIntegrationSource();
@@ -204,7 +204,7 @@ TicXicChromTracePlotCompositeWidget::plotWidgetKeyPressEvent(
       mp_parentWnd->recordAnalysisStanza(stanza, plottable_p->pen().color());
     }
   // End of
-  // if(context.pressedKeyCode == Qt::Key_Space)
+  // if(context.m_pressedKeyCode == Qt::Key_Space)
 }
 
 
@@ -231,8 +231,8 @@ TicXicChromTracePlotCompositeWidget::moveMouseCursorToNextGraphPoint(
   // in the base class function.
 
   pappso::BasePlotContext local_context = context;
-  local_context.xRegionRangeStart       = m_integrationRange.lower;
-  local_context.xRegionRangeEnd         = m_integrationRange.upper;
+  local_context.m_xRegionRangeStart     = m_integrationRange.lower;
+  local_context.m_xRegionRangeEnd       = m_integrationRange.upper;
 
   if(m_isSinglePointIntegrationMode)
     integrationRequested(local_context);
@@ -243,18 +243,37 @@ void
 TicXicChromTracePlotCompositeWidget::integrationRequested(
   const pappso::BasePlotContext &context)
 {
-  // qDebug().noquote() << "context:" << context.toString();
+   qDebug().noquote() << "context:" << context.toString();
 
-  // Immediately check if the selection range makes sense. For example, having
-  // both the start/end values in the negative values does not mean anything
-  // with respect to TIC | XIC chromatogram data!
+  // Make a local copy of the context because we may need to modify it.
+  pappso::BasePlotContext local_context = context;
 
-  // Remember that the xregion start and end are not sorted.
-  double range_start =
-    std::min<double>(context.xRegionRangeStart, context.xRegionRangeEnd);
-  double range_end =
-    std::max<double>(context.xRegionRangeStart, context.xRegionRangeEnd);
+  // Check if the selection polygon contain meaningful data. We are in trace
+  // plot widget, so the selection polygon might be in the form of a
+  // mono-dimensional polygon, although this is not mandatory. But we rely on
+  // the selection polygon to be of 1 or 2 dimension for some optimizations. So,
+  // if the polygon is 2D, convert it to 1D.
 
+  if(local_context.m_selectionPolygon.is2D())
+    qDebug() << "Odd situation where the selection polygon is two-dimensional "
+                "and the trace plot selection is mono-dimensional. Converting "
+                "it to a 1D selection polygon.";
+
+  local_context.m_selectionPolygon.convertTo1D();
+
+  // Sanity check.
+
+  // The user may select at the left of the x=0, but that does not make sense if
+  // the whole selected region is in the x axis negative space.
+  double range_start;
+  double range_end;
+
+  local_context.m_selectionPolygon.rangeX(range_start, range_end);
+
+  qDebug() << "selection polygon:" << local_context.m_selectionPolygon.toString();
+  //qDebug() << "X axis range:" << range_start << "-" << range_end;
+
+  // Having negative retention times does not make any sense.
   if(range_start < 0 && range_end < 0)
     return;
 
@@ -293,35 +312,28 @@ TicXicChromTracePlotCompositeWidget::integrationRequested(
       return;
     }
 
-  // Create a local copy of the processing flow.
+  // Create a local copy of the processing flow so that we can modify it by
+  // adding the new processing step that describes what we are doing now.
+
+  qDebug();
 
   ProcessingFlow local_processing_flow =
     getProcessingFlowForPlottable(plottable_p);
 
-  // qDebug().noquote() << "Processing flow for graph:"
-  //<< local_processing_flow.toString();
+  qDebug() << local_processing_flow.getMsRunDataSetCstSPtr()->getMsRunId()->getFileName();
+
+  qDebug().noquote() << "Processing flow for graph:"
+                     << local_processing_flow.toString();
 
   MsRunDataSetCstSPtr ms_run_data_set_csp =
     getMsRunDataSetCstSPtrForPlottable(plottable_p);
 
   // qDebug() << ms_run_data_set_csp->getMsRunDataSetStats().toString();
 
-  // Now we need to add a new step in that processing flow so that we document
-  // this new integration that we are performing.
+  // Create a processing step to document the specifics of this new integration.
 
   ProcessingStep *processing_step_p = new ProcessingStep();
 
-  // The step has a multi-map of pairs that each relate a processing type with a
-  // processing spec. So we need to document a new pair type/spec, let's start
-  // with the spec that documents the range of the data we are willing to
-  // integrate, that is, the selected region of the graph.
-
-  // The ProcessingSpec has the range start/end values, an optional ms
-  // fragmentation specification and a date time that is set automatically at
-  // construction time.
-
-  ProcessingSpec *processing_spec_p = new ProcessingSpec();
-
   // If the user had set ms fragmentation specifications, all these are listed
   // in the various ProcessingSpec instances, so we do not loose any. However,
   // the user is entitled to define any new ms fragmentation specifications
@@ -339,7 +351,7 @@ TicXicChromTracePlotCompositeWidget::integrationRequested(
       // flow " "is valid, using it:"
       //<< ms_fragmentation_spec.toString();
 
-      processing_spec_p->setMsFragmentationSpec(ms_fragmentation_spec);
+      processing_step_p->setMsFragmentationSpec(ms_fragmentation_spec);
     }
   else
     {
@@ -348,13 +360,24 @@ TicXicChromTracePlotCompositeWidget::integrationRequested(
       //<< ms_fragmentation_spec.toString();
     }
 
+  // Now make sure we document the selection polygon.
   // Now define the graph range for the integration operation.
 
-  processing_spec_p->setRange(range_start, range_end);
+  qDebug();
 
-  // qDebug() << "Range:" << range_start << "-" << range_end;
+  processing_step_p->setSelectionPolygon(local_context.m_selectionPolygon);
 
-  // Now handle the specific part of the ProcessingSpec, that part that depends
+  qDebug();
+
+  // We must say what is the data source processing type.
+  processing_step_p->setSrcProcessingType(pappso::Axis::x, "RT");
+
+  // It is essential that we document the relation between the axes and the
+  // corresponding data type.
+  processing_step_p->setDataKind(pappso::Axis::x, pappso::DataKind::rt);
+  processing_step_p->setDataKind(pappso::Axis::y, pappso::DataKind::unset);
+
+  // Now handle the specific part of the ProcessingStep, that part that depends
   // on the kind of integration the user is willing to perform. Note below that
   // we gather all the integrations to destinations that involve integration
   // mass spectra. This is because we factorize the requriement to have proper
@@ -364,7 +387,7 @@ TicXicChromTracePlotCompositeWidget::integrationRequested(
     {
       // qDebug() << "Integration to XIC chromatogram.";
 
-      processing_step_p->newSpec(ProcessingType("RT_TO_RT"), processing_spec_p);
+      processing_step_p->setDestProcessingType("RT");
 
       local_processing_flow.push_back(processing_step_p);
 
@@ -373,25 +396,25 @@ TicXicChromTracePlotCompositeWidget::integrationRequested(
     }
   else if(m_ui.integrateToMzPushButton->isChecked() ||
           m_ui.integrateToDtMzPushButton->isChecked() ||
-          m_ui.integrateToRtMzPushButton->isChecked())
+          m_ui.integrateToMzRtPushButton->isChecked())
     {
       if(m_ui.integrateToMzPushButton->isChecked())
         {
-          // qDebug() << "Integrating to mass spectrum.";
-          processing_step_p->newSpec(ProcessingType("RT_TO_MZ"),
-                                     processing_spec_p);
+          qDebug() << "Integrating to mass spectrum.";
+          processing_step_p->setDestProcessingType("MZ");
+
+          qDebug() << "Currently elaborating processing_step_p: "
+                   << processing_step_p->toString();
         }
-      else if(m_ui.integrateToRtMzPushButton->isChecked())
+      else if(m_ui.integrateToMzRtPushButton->isChecked())
         {
-          // qDebug() << "Integrating to rt|m/z color map.";
-          processing_step_p->newSpec(ProcessingType("RT_TO_RT_MZ"),
-                                     processing_spec_p);
+          qDebug() << "Integrating to rt|m/z color map.";
+          processing_step_p->setDestProcessingType("MZ_RT");
         }
       else if(m_ui.integrateToDtMzPushButton->isChecked())
         {
-          // qDebug() << "Integrating to dt|m/z color map.";
-          processing_step_p->newSpec(ProcessingType("RT_TO_DT_MZ"),
-                                     processing_spec_p);
+          qDebug() << "Integrating to dt|m/z color map.";
+          processing_step_p->setDestProcessingType("DT_MZ");
         }
 
       // At this point we need to define how the mass data integrator will
@@ -437,7 +460,7 @@ TicXicChromTracePlotCompositeWidget::integrationRequested(
       // Note that we cannot integrate mass spectra with a high resolution
       // here. We need to reduce the resolution to integer resolution.
       if(m_ui.integrateToDtMzPushButton->isChecked() ||
-         m_ui.integrateToRtMzPushButton->isChecked())
+         m_ui.integrateToMzRtPushButton->isChecked())
         {
           mz_integration_params.setBinningType(pappso::BinningType::NONE);
           mz_integration_params.setDecimalPlaces(0);
@@ -452,8 +475,7 @@ TicXicChromTracePlotCompositeWidget::integrationRequested(
       // last one.
       local_processing_flow.push_back(processing_step_p);
 
-      // qDebug().noquote() << "Pushed back new processing step:"
-      //<< processing_step_p->toString();
+      qDebug();
 
       // The local processing flow contains all the previous steps and the last
       // one that documents this current integration.
@@ -468,48 +490,39 @@ TicXicChromTracePlotCompositeWidget::integrationRequested(
           static_cast<TicXicChromTracePlotWnd *>(mp_parentWnd)
             ->integrateToDtMz(plottable_p, local_processing_flow);
         }
-      else if(m_ui.integrateToRtMzPushButton->isChecked())
+      else if(m_ui.integrateToMzRtPushButton->isChecked())
         {
           static_cast<TicXicChromTracePlotWnd *>(mp_parentWnd)
-            ->integrateToRtMz(plottable_p, local_processing_flow);
+            ->integrateToMzRt(plottable_p, local_processing_flow);
         }
     }
   else if(m_ui.integrateToDtPushButton->isChecked())
     {
-      // qDebug() << "Integrating to drift spectrum.";
+      qDebug() << "Integrating to drift spectrum.";
 
-      processing_step_p->newSpec(ProcessingType("RT_TO_DT"), processing_spec_p);
+      processing_step_p->setDestProcessingType("DT");
 
       local_processing_flow.push_back(processing_step_p);
 
       static_cast<TicXicChromTracePlotWnd *>(mp_parentWnd)
         ->integrateToDt(plottable_p, local_processing_flow);
     }
-  else if(m_ui.integrateToRtDtPushButton->isChecked())
+  else if(m_ui.integrateToDtRtPushButton->isChecked())
     {
-      // qDebug() << "Integrating to TIC|XIC chromatogram / drift spectrum
-      // colormap.";
+      qDebug() << "Integrating to dt/rt colormap.";
 
-      ProcessingSpec *spec_p = new ProcessingSpec();
-
-      processing_step_p->newSpec(ProcessingType("RT_TO_RT_DT"), spec_p);
-
-      // We now need to specify the RT range.
-
-      processing_step_p->newSpec(ProcessingType("RT_TO_ANY"),
-                                 processing_spec_p);
+      processing_step_p->setDestProcessingType("DT_RT");
 
       local_processing_flow.push_back(processing_step_p);
 
       static_cast<TicXicChromTracePlotWnd *>(mp_parentWnd)
-        ->integrateToRtDt(plottable_p, local_processing_flow);
+        ->integrateToDtRt(plottable_p, local_processing_flow);
     }
   else if(m_ui.integrateToIntPushButton->isChecked())
     {
-      // qDebug() << "Integrating to TIC intensity.";
+      qDebug() << "Integrating to TIC intensity.";
 
-      processing_step_p->newSpec(ProcessingType("RT_TO_INT"),
-                                 processing_spec_p);
+      processing_step_p->setDestProcessingType("INT");
 
       local_processing_flow.push_back(processing_step_p);
 
@@ -528,6 +541,8 @@ TicXicChromTracePlotCompositeWidget::integrationRequested(
         QMessageBox::StandardButton::NoButton);
     }
 
+  qDebug();
+
   return;
 }
 
@@ -663,7 +678,7 @@ TicXicChromTracePlotCompositeWidget::craftAnalysisStanza(
           if(curChar == 'X')
             {
               stanza += QString("%1").arg(
-                context.lastCursorHoveredPoint.x(), 0, 'g', 6);
+                context.m_lastCursorHoveredPoint.x(), 0, 'g', 6);
               prevChar = ' ';
               continue;
             }
@@ -674,7 +689,7 @@ TicXicChromTracePlotCompositeWidget::craftAnalysisStanza(
               // of the point at X.
               stanza += QString("%1").arg(
                 static_cast<pappso::BaseTracePlotWidget *>(mp_plotWidget)
-                  ->getYatX(context.lastCursorHoveredPoint.x(),
+                  ->getYatX(context.m_lastCursorHoveredPoint.x(),
                             static_cast<QCPGraph *>(plottable_p)),
                 0,
                 'g',
@@ -685,13 +700,13 @@ TicXicChromTracePlotCompositeWidget::craftAnalysisStanza(
             }
           if(curChar == 'x')
             {
-              stanza += QString("%1").arg(context.xDelta, 0, 'g', 6);
+              stanza += QString("%1").arg(context.m_xDelta, 0, 'g', 6);
               prevChar = ' ';
               continue;
             }
           if(curChar == 'y')
             {
-              stanza += QString("%1").arg(context.yDelta, 0, 'g', 6);
+              stanza += QString("%1").arg(context.m_yDelta, 0, 'g', 6);
 
               prevChar = ' ';
               continue;
@@ -705,14 +720,15 @@ TicXicChromTracePlotCompositeWidget::craftAnalysisStanza(
             }
           if(curChar == 'b')
             {
-              stanza += QString("%1").arg(context.xRegionRangeStart, 0, 'f', 3);
+              stanza +=
+                QString("%1").arg(context.m_xRegionRangeStart, 0, 'f', 3);
 
               prevChar = ' ';
               continue;
             }
           if(curChar == 'e')
             {
-              stanza += QString("%1").arg(context.xRegionRangeEnd, 0, 'f', 3);
+              stanza += QString("%1").arg(context.m_xRegionRangeEnd, 0, 'f', 3);
 
               prevChar = ' ';
               continue;
