@@ -100,10 +100,6 @@ std::pair<MsRunDataSetCstSPtr, QColor>
 TicXicChromTracePlotWnd::finishedIntegratingToInitialTicChromatogram(
   MsRunDataSetTreeMassDataIntegratorToRt *mass_data_integrator_p)
 {
-
-
-  // qDebug() << "the integrator pointer:" << mass_data_integrator_p;
-
   // We get back the integrator that was made to work in another thread. Now get
   // the obtained data and work with them.
 
@@ -140,20 +136,31 @@ TicXicChromTracePlotWnd::finishedIntegratingToInitialTicChromatogram(
         "the mining session from there.\n",
         QMessageBox::Ok);
     }
-
-  // Note that if the mass data file that was loaded contained a single mass
-  // spectrum, then there would be no TIC. So we need to craft a fake one with
-  // two points: (0,TIC) and (1,TIC).
-
-  if(integrated_trace.size() < 2)
+  else if(integrated_trace.size() < 2)
     {
 
-      // In this situation the single data point that is there has x = 0 ; so
-      // just create a new one with x = 1.
+
+      // Note that if the mass data file that was loaded contained a single mass
+      // spectrum, or if the integration yielded a single TIC value for a single
+      // retention time, then there would be no TIC. So we need to craft a fake
+      // one with two points: (current,TIC) and (current+1,TIC).
+
+      // This situation might be encountered when the user integration to RT:
+      //
+      // 1. a single spectrum for a given RT
+      //
+      // 2. multiple spectra for a single RT (that is, multiple specra of
+      // different DT values but of a single RT value).
+
+      // In this situation the single data point that is there has x = <value> ;
+      // so just create a new one with x = <value + 1>.
+
+      pappso::DataPoint unique_data_point = integrated_trace.back();
 
       // qDebug() << "tic trace:" << integrated_trace.toString();
 
-      integrated_trace.push_back(pappso::DataPoint(1, integrated_trace.sumY()));
+      integrated_trace.push_back(
+        pappso::DataPoint(unique_data_point.x + 1, unique_data_point.y));
 
       // qDebug() << "tic trace:" << integrated_trace.toString();
     }
@@ -163,8 +170,6 @@ TicXicChromTracePlotWnd::finishedIntegratingToInitialTicChromatogram(
 
   ProcessingFlow processing_flow = mass_data_integrator_p->getProcessingFlow();
 
-  // qDebug() << "Got processing_flow:" << processing_flow.toString();
-
   // Note that we are in special situation: we are now handling a TIC
   // chromatogram that was computed as a very first integration right after
   // having read data from file. We did set MS level to 1, because, by
@@ -174,21 +179,23 @@ TicXicChromTracePlotWnd::finishedIntegratingToInitialTicChromatogram(
   // thus replace that MS level to 0, that means that all the data (whatever the
   // MS level) will be handled from there.
 
-  ProcessingStep *most_recent_step_p =
-    const_cast<ProcessingStep *>(processing_flow.mostRecentStep());
+  const ProcessingStep *most_recent_step_p = processing_flow.mostRecentStep();
 
-  std::vector<ProcessingSpec *> processing_specs =
-    most_recent_step_p->allProcessingSpecsMatching(ProcessingType("FILE_TO_RT"),
-                                                   true);
-
-  if(processing_specs.size() != 1)
+  if(most_recent_step_p->destMatches("FILE_RT"))
     qFatal(
-      "Cannot be that there are either no or more than one processing step fo "
-      "type "
-      "FILE_TO_RT");
+      "Cannot be that there are either no or more than one processing step for "
+      "type FILE_RT");
 
+  // We need to const_cast the step in order to overwrite is
+  // MsFragmentationSpec.
   MsFragmentationSpec *fragmentation_spec_p =
-    processing_specs.front()->getMsFragmentationSpecPtr();
+    const_cast<ProcessingStep *>(most_recent_step_p)
+      ->getMsFragmentationSpecPtr();
+
+  // While we had set the MS level to 1 because when computing a TIC
+  // chromatogram we only do that with MS1 data, we now want to set it to 0
+  // because all the later integrations need to be capable of involving any
+  // MS level. 0 states that MS level matches.
 
   fragmentation_spec_p->setMsLevel(0);
 
@@ -246,250 +253,6 @@ TicXicChromTracePlotWnd::finishedIntegratingToInitialTicChromatogram(
                                                 plot_color);
 }
 
-#if 0
-// Old version
-void
-TicXicChromTracePlotWnd::integrateToRt(QCPAbstractPlottable *parent_plottable_p,
-                                       const ProcessingFlow &processing_flow)
-
-{
-  qDebug();
-
-  // qDebug().noquote() << "Integrating to rt with processing flow:"
-  //<< processing_flow.toString();
-
-  // Get the ms run data set that for the graph we are going to base the
-  // integration on.
-  MsRunDataSetCstSPtr ms_run_data_set_csp =
-    processing_flow.getMsRunDataSetCstSPtr();
-  if(ms_run_data_set_csp == nullptr)
-    qFatal("Cannot be that the pointer is nullptr.");
-
-  // Pass the integrator the flow we got as param and that describes in its most
-  // recent step the integration that it should perform.
-  MsRunDataSetTreeMassDataIntegratorToRt *mass_data_integrator_p =
-    new MsRunDataSetTreeMassDataIntegratorToRt(ms_run_data_set_csp,
-                                               processing_flow);
-
-  // Ensure the mass data integrator messages are used.
-
-  connect(mass_data_integrator_p,
-          &MassDataIntegrator::logTextToConsoleSignal,
-          mp_programWindow,
-          &ProgramWindow::logTextToConsole);
-
-  // qDebug() << "the integrator pointer:" << mass_data_integrator_p;
-
-  // Allocate a mass data integrator to integrate the data.
-
-  MassDataIntegratorTask *mass_data_integrator_task_p =
-    new MassDataIntegratorTask();
-
-  // This signal starts the computation in the MassDataIntegratorTask object.
-  connect(
-    this,
-    // SIGNAL(integrateToRtSignal(MsRunDataSetTreeMassDataIntegratorToRt
-    // *)),
-    static_cast<void (TicXicChromTracePlotWnd::*)(
-      MsRunDataSetTreeMassDataIntegratorToRt *)>(
-      &TicXicChromTracePlotWnd::integrateToRtSignal),
-    mass_data_integrator_task_p, // SLOT(integrateToRt(MsRunDataSetTreeMassDataIntegratorToRt
-                                 // *)),
-    static_cast<void (MassDataIntegratorTask::*)(
-      MsRunDataSetTreeMassDataIntegratorToRt *)>(
-      &MassDataIntegratorTask::integrateToRt),
-    // Fundamental for signals that travel across QThread instances...
-    Qt::QueuedConnection);
-
-  // Allocate the thread in which the integrator task will run.
-  QThread *thread_p = new QThread;
-
-  // Move the task to the matching thread.
-  mass_data_integrator_task_p->moveToThread(thread_p);
-  thread_p->start();
-
-  // When the read task finishes, it sends a signal that we trap to go on with
-  // the plot widget creation stuff.
-
-  // Since we allocated the QThread dynamically we need to be able to destroy it
-  // later, so make the connection.
-  connect(mass_data_integrator_task_p,
-          static_cast<void (MassDataIntegratorTask::*)(MassDataIntegrator *)>(
-            &MassDataIntegratorTask::finishedIntegratingDataSignal),
-          this,
-          [this,
-           thread_p,
-           mass_data_integrator_p,
-           parent_plottable_p,
-           mass_data_integrator_task_p]() {
-            // Do not forget that we have to delete the MassDataIntegratorTask
-            // allocated instance.
-            mass_data_integrator_task_p->deleteLater();
-            // Once the task has been labelled to be deleted later, we can stop
-            // the thread and ask for it to also be deleted later.
-            thread_p->deleteLater(), thread_p->quit();
-            thread_p->wait();
-            this->finishedIntegratingToRt(mass_data_integrator_p,
-                                          parent_plottable_p);
-          });
-
-
-  // Allocate a new TaskMonitorCompositeWidget that will receive all the
-  // integrator's signals and provide feedback to the user about the ongoing
-  // integration.
-
-  TaskMonitorCompositeWidget *task_monitor_composite_widget_p =
-    mp_programWindow->getTaskMonitorWnd()->addTaskMonitorWidget(Qt::red);
-
-  // Initialize the monitor composite widget's widgets and make all the
-  // connections mass data integrator <--> widget.
-
-  task_monitor_composite_widget_p->setMsRunIdText(
-    ms_run_data_set_csp->getMsRunId()->getSampleName());
-  task_monitor_composite_widget_p->setTaskDescriptionText(
-    "Integrating to XIC chromatogram.");
-  task_monitor_composite_widget_p->setProgressBarMinValue(0);
-
-  // Make the connections
-
-  // When the integrator task instance has finished working, it will send a
-  // signal that we trap to finally destroy (after a time lag of some seconds,
-  // the monitor widget.
-
-  connect(mass_data_integrator_task_p,
-          static_cast<void (MassDataIntegratorTask::*)(MassDataIntegrator *)>(
-            &MassDataIntegratorTask::finishedIntegratingDataSignal),
-          task_monitor_composite_widget_p,
-          &TaskMonitorCompositeWidget::taskFinished,
-          Qt::QueuedConnection);
-
-  // If the user clicks the cancel button, relay the signal to the loader.
-  connect(task_monitor_composite_widget_p,
-          &TaskMonitorCompositeWidget::cancelTaskSignal,
-          mass_data_integrator_p,
-          &MassDataIntegrator::cancelOperation);
-
-  // We need to register the meta type for std::size_t because otherwise it
-  // cannot be shipped though signals.
-
-  qRegisterMetaType<std::size_t>("std::size_t");
-
-  // Now make all the connections that will allow the integrator to provide
-  // dynamic feedback to the user via the task monitor widget.
-  task_monitor_composite_widget_p->makeMassDataIntegratorConnections(
-    mass_data_integrator_p);
-
-  // qDebug() << "going to emit integrateToMzSignal with mass "
-  //"data integrator:"
-  //<< mass_data_integrator_p;
-
-  emit integrateToRtSignal(mass_data_integrator_p);
-
-  // We do not want to make signal/slot calls more than once. This is because
-  // one user might well trigger more than one integration from this window to a
-  // mass spectrum. Thus we do not want that *this window be still connected to
-  // the specific mass_data_integrator_task_p when a new integration is
-  // triggered. We want the signal/slot pairs to be contained to specific
-  // objects. Each TicXicChromTracePlotWnd::integrateToMz() call must be contained
-  // to a this/mass_data_integrator_task_p specific signal/slot pair.
-  disconnect(
-    this,
-    // SIGNAL(integrateToRtSignal(MsRunDataSetTreeMassDataIntegratorToRt *)),
-    static_cast<void (TicXicChromTracePlotWnd::*)(
-      MsRunDataSetTreeMassDataIntegratorToRt *)>(
-      &TicXicChromTracePlotWnd::integrateToRtSignal),
-    mass_data_integrator_task_p,
-    // SLOT(integrateToRt(MsRunDataSetTreeMassDataIntegratorToRt *)));
-    static_cast<void (MassDataIntegratorTask::*)(
-      MsRunDataSetTreeMassDataIntegratorToRt *)>(
-      &MassDataIntegratorTask::integrateToRt));
-}
-
-
-// Old version
-void
-TicXicChromTracePlotWnd::finishedIntegratingToRt(
-  MsRunDataSetTreeMassDataIntegratorToRt *mass_data_integrator_p,
-  QCPAbstractPlottable *parent_plottable_p)
-{
-  // This function is actually a slot that is called when the integration
-  // is terminated in a QThread.
-
-  // qDebug() << "the integrator pointer:" << mass_data_integrator_p;
-
-  // We get back the integrator that was made to work in another thread. Now get
-  // the obtained data and work with them.
-
-  // Store the MsRunDataSetSPtr for later use before deleting the integrattor.
-
-  MsRunDataSetCstSPtr ms_run_data_set_csp =
-    mass_data_integrator_p->getMsRunDataSet();
-
-  pappso::Trace integrated_trace =
-    mass_data_integrator_p->getMapTrace().toTrace();
-  // qDebug() << "mass spectrum trace size:" << integrated_trace.size();
-
-  if(!integrated_trace.size())
-    {
-      qDebug() << "There is not a single point in the integrated trace.";
-
-      return;
-    }
-
-  // Also, do not forget to copy the processing flow from the integrator. This
-  // processing flow will be set into the plot widget !
-
-  ProcessingFlow processing_flow = mass_data_integrator_p->getProcessingFlow();
-
-  // Sanity check
-  if(ms_run_data_set_csp != processing_flow.getMsRunDataSetCstSPtr())
-    qFatal("Cannot be that the ms run data set pointers be different.");
-
-  // Finally, do not forget that we need to delete the integrator now that the
-  // calculation is finished.
-  delete mass_data_integrator_p;
-  mass_data_integrator_p = nullptr;
-
-  // At this point, we need to get a color for the plot. That color needs to be
-  // that of the parent.
-  QColor plot_color = parent_plottable_p->pen().color();
-
-  // We need to establish what widget is the destination of the integration. If
-  // one or more than a widget is/are pinned-down, then that/these should be the
-  // destinations. If no widget is pinned-down, then a new plot widget needs to
-  // be created.
-
-  std::vector<BasePlotCompositeWidget *> pinned_down_widgets =
-    pinnedDownWidgets();
-
-  if(pinned_down_widgets.size())
-    {
-      for(auto &&pinned_down_widget : pinned_down_widgets)
-        static_cast<BaseTracePlotCompositeWidget *>(pinned_down_widget)
-          ->addTrace(integrated_trace,
-                     static_cast<QCPGraph *>(parent_plottable_p),
-                     ms_run_data_set_csp,
-                     ms_run_data_set_csp->getMsRunId()->getSampleName(),
-                     processing_flow,
-                     plot_color);
-    }
-  else
-    addTracePlot(integrated_trace,
-                 ms_run_data_set_csp,
-                 processing_flow,
-                 plot_color,
-                 static_cast<QCPGraph *>(parent_plottable_p));
-
-  // At this point, if the window is really not visible, then show it, otherwise
-  // I was told that the user does not think that they can ask for the window to
-  // show up in the main program window.
-
-  if(!isVisible())
-    {
-      showWindow();
-    }
-}
-#endif
 
 void
 TicXicChromTracePlotWnd::integrateToRt(
@@ -498,9 +261,6 @@ TicXicChromTracePlotWnd::integrateToRt(
     qualified_mass_spectra_sp,
   const ProcessingFlow &processing_flow)
 {
-  //qDebug().noquote() << "Integrating to RT with processing flow:"
-                     //<< processing_flow.toString();
-
   // Get the ms run data set that for the graph we are going to base the
   // integration on.
   MsRunDataSetCstSPtr ms_run_data_set_csp =
@@ -508,28 +268,41 @@ TicXicChromTracePlotWnd::integrateToRt(
   if(ms_run_data_set_csp == nullptr)
     qFatal("Cannot be that the pointer is nullptr.");
 
-  // We will try to limit the number of mass spectra to iterate through with the
-  // integration visitor.
+  // FIXME: BEGIN Sanity check that might be removed when the program is
+  // stabilized.
+  std::pair<double, double> range_pair;
 
-  double start_rt = std::numeric_limits<double>::infinity();
-  double end_rt   = std::numeric_limits<double>::infinity();
-
-  bool integration_rt = processing_flow.innermostRtRange(start_rt, end_rt);
+  bool integration_rt = processing_flow.innermostRange("ANY_RT", range_pair);
 
   if(!integration_rt)
     qFatal("Programming error.");
 
-  // First prepare a vector of QualifiedMassSpectrumCstSPtr
+  qDebug() << qSetRealNumberPrecision(10) << "Innermost RT range:"
+           << "for y axis: " << range_pair.first << "-" << range_pair.second;
+
+  bool integration_dt = processing_flow.innermostRange("ANY_DT", range_pair);
+
+  if(integration_dt)
+    qDebug() << qSetRealNumberPrecision(10) << "Innermost DT range:"
+             << "for y axis: " << range_pair.first << "-" << range_pair.second;
+  // FIXME: END Sanity check that might be removed when the program is
+  // stabilized.
+
+  // First prepare a vector of QualifiedMassSpectrumCstSPtr. If the
+  // qualified_mass_spectra_sp is not empty, the
+  // fillInQualifiedMassSpectraVector() function below does not change the
+  // vector and returns its size.
 
   std::size_t qualified_mass_spectra_count = fillInQualifiedMassSpectraVector(
     ms_run_data_set_csp, qualified_mass_spectra_sp, processing_flow);
 
+  qDebug()
+    << "The number of selected mass spectra on the basis of rough RT/DT values:"
+    << qualified_mass_spectra_count;
+
   if(!qualified_mass_spectra_count)
     return;
 
-  //qDebug() << "The number of remaining mass spectra:"
-           //<< qualified_mass_spectra_count;
-
   // Now start the actual integration work.
 
   // Allocate a mass data integrator to integrate the data.
@@ -538,7 +311,8 @@ TicXicChromTracePlotWnd::integrateToRt(
     new QualifiedMassSpectrumVectorMassDataIntegratorToRt(
       ms_run_data_set_csp, processing_flow, qualified_mass_spectra_sp);
 
-  mass_data_integrator_p->setMaxThreadUseCount(mp_programWindow->getMaxThreadUseCount());
+  mass_data_integrator_p->setMaxThreadUseCount(
+    mp_programWindow->getMaxThreadUseCount());
 
   // Ensure the mass data integrator messages are used.
 
@@ -548,8 +322,6 @@ TicXicChromTracePlotWnd::integrateToRt(
     mp_programWindow,
     &ProgramWindow::logTextToConsole);
 
-  qDebug() << "Receiving processing flow:" << processing_flow.toString();
-
   MassDataIntegratorTask *mass_data_integrator_task_p =
     new MassDataIntegratorTask();
 
@@ -682,8 +454,6 @@ TicXicChromTracePlotWnd::finishedIntegratingToRt(
   QualifiedMassSpectrumVectorMassDataIntegrator *mass_data_integrator_p,
   QCPAbstractPlottable *parent_plottable_p)
 {
-  qDebug();
-
   // This function is actually a slot that is called when the integration to mz
   // is terminated in a QThread.
 
@@ -707,6 +477,34 @@ TicXicChromTracePlotWnd::finishedIntegratingToRt(
       return;
     }
 
+  // Note that if the mass data file that was loaded contained a single mass
+  // spectrum, or if the integration yielded a single TIC value for a single
+  // retention time, then there would be no TIC. So we need to craft a fake one
+  // with two points: (current,TIC) and (current+1,TIC).
+
+  if(integrated_trace.size() < 2)
+    {
+
+      // This situation might be encountered when the user integration to RT:
+      //
+      // 1. a single spectrum for a given RT
+      //
+      // 2. multiple spectra for a single RT (that is, multiple specra of
+      // different DT values but of a single RT value).
+
+      // In this situation the single data point that is there has x = <value> ;
+      // so just create a new one with x = <value + 1>.
+
+      pappso::DataPoint unique_data_point = integrated_trace.back();
+
+      // qDebug() << "tic trace:" << integrated_trace.toString();
+
+      integrated_trace.push_back(
+        pappso::DataPoint(unique_data_point.x + 1, unique_data_point.y));
+
+      // qDebug() << "tic trace:" << integrated_trace.toString();
+    }
+
   // Also, do not forget to copy the processing flow from the integrator. This
   // processing flow will be set into the plot widget !
 
@@ -807,27 +605,27 @@ TicXicChromTracePlotWnd::integrateToDtMz(
 
 
 void
-TicXicChromTracePlotWnd::integrateToRtMz(
+TicXicChromTracePlotWnd::integrateToMzRt(
   QCPAbstractPlottable *parent_plottable_p,
   const ProcessingFlow &processing_flow)
 {
   if(parent_plottable_p == nullptr)
     qFatal("Cannot be that pointer is nullptr.");
 
-  mp_programWindow->integrateToRtMz(
+  mp_programWindow->integrateToMzRt(
     parent_plottable_p, nullptr, processing_flow);
 }
 
 
 void
-TicXicChromTracePlotWnd::integrateToRtDt(
+TicXicChromTracePlotWnd::integrateToDtRt(
   QCPAbstractPlottable *parent_plottable_p,
   const ProcessingFlow &processing_flow)
 {
   if(parent_plottable_p == nullptr)
     qFatal("Cannot be that pointer is nullptr.");
 
-  mp_programWindow->integrateToRtDt(
+  mp_programWindow->integrateToDtRt(
     parent_plottable_p, nullptr, processing_flow);
 }
 
