@@ -90,9 +90,9 @@ class TicXicChromTracePlotWnd : public BaseTracePlotWnd
                      const ProcessingFlow &processing_flow);
   void integrateToDtMz(QCPAbstractPlottable *plottable_p,
                        const ProcessingFlow &processing_flow);
-  void integrateToRtMz(QCPAbstractPlottable *plottable_p,
+  void integrateToMzRt(QCPAbstractPlottable *plottable_p,
                        const ProcessingFlow &processing_flow);
-  void integrateToRtDt(QCPAbstractPlottable *plottable_p,
+  void integrateToDtRt(QCPAbstractPlottable *plottable_p,
                        const ProcessingFlow &processing_flow);
 
   public slots:
