@@ -50,12 +50,7 @@
 #include "MassDataIntegrator.hpp"
 #include "BaseMsRunDataSetTreeNodeVisitor.hpp"
 #include "TicChromTreeNodeCombinerVisitor.hpp"
-#include "IntensityTreeNodeCombinerVisitor.hpp"
-#include "RtDtMzColorMapsTreeNodeCombinerVisitor.hpp"
-#include "TraceTreeNodeCombinerVisitor.hpp"
 #include "MsRunStatisticsTreeNodeVisitor.hpp"
-#include "MassSpectrumTreeNodeCombinerVisitor.hpp"
-#include "DriftSpectrumTreeNodeCombinerVisitor.hpp"
 #include "MultiTreeNodeCombinerVisitor.hpp"
 
 
