@@ -400,7 +400,7 @@ MassSpectrumTreeNodeCombinerVisitor::checkProcessingTypeByRt(
 
   const ProcessingSpec *spec_p = type_spec_pair.second;
 
-  if(spec_p->hasValidRange())
+  if(spec_p->hasValidOneDimensionRange())
     {
       double rt = qualified_mass_spectrum_csp->getRtInMinutes();
 
@@ -431,7 +431,7 @@ MassSpectrumTreeNodeCombinerVisitor::checkProcessingTypeByMz(
 
   const ProcessingSpec *spec_p = type_spec_pair.second;
 
-  if(spec_p->hasValidRange())
+  if(spec_p->hasValidOneDimensionRange())
     {
       // qDebug() << "Needed a m/z range filtering:" << spec_p->toString();
 
@@ -474,7 +474,7 @@ MassSpectrumTreeNodeCombinerVisitor::checkProcessingTypeByDt(
 
   const ProcessingSpec *spec_p = type_spec_pair.second;
 
-  if(spec_p->hasValidRange())
+  if(spec_p->hasValidOneDimensionRange())
     {
       double dt = qualified_mass_spectrum_csp->getDtInMilliSeconds();
 
