@@ -346,7 +346,7 @@ MsRunDataSet::mapMassSpectrum(
 
 //! Return mass spectrum at index \p index.
 pappso::QualifiedMassSpectrumCstSPtr
-MsRunDataSet::massSpectrumAtIndex(std::size_t spectrum_index)
+MsRunDataSet::massSpectrumAtIndex(std::size_t spectrum_index) const
 {
   pappso::MsRunDataSetTreeNode *node =
     msp_msRunDataSetTree->findNode(spectrum_index);
