@@ -360,19 +360,17 @@ MsRunDataSetStats::incrementStatistics(
   const pappso::MassSpectrum &mass_spectrum)
 {
 
-  // qDebug();
-
   std::size_t mass_spectrum_size = mass_spectrum.size();
 
   if(!mass_spectrum_size)
     {
-      // qDebug() << "The mass spectrum is empty, nothing to do, returning.";
+      //qDebug() << "The mass spectrum is empty, nothing to do, returning.";
       return;
     }
-  else
-    {
-      // qDebug() << "The mass spectrum has size:" << mass_spectrum_size;
-    }
+  //else
+    //{
+      //qDebug() << "The mass spectrum has size:" << mass_spectrum_size;
+    //}
 
   // We can actually handle this spectrum, so increase the counter.
   ++m_spectrumCount;
@@ -416,8 +414,19 @@ MsRunDataSetStats::incrementStatistics(
   double last_mz = mass_spectrum.back().x;
   m_lastMzs.push_back(last_mz);
   // The maximum m/z value
+
+  //qDebug() << "Mass spectrum's last m/z:" << last_mz;
+
   if(last_mz > m_maxMz)
-    m_maxMz = last_mz;
+    {
+      //qDebug() << "Will update the precendent m_maxMz:" << m_maxMz << "value";
+      m_maxMz = last_mz;
+    }
+  //else
+    //{
+      //qDebug() << "Not changing current m_maxMz:" << m_maxMz
+               //<< "because last_mz:" << last_mz;
+    //}
 
   // The current mass spectrum has data in it, as they were loaded from file
   // (that is, unmodified in any way). We want to compute some of the various
@@ -494,8 +503,15 @@ MsRunDataSetStats::merge(const MsRunDataSetStats &other)
   if(other.m_minMz < m_minMz)
     m_minMz = other.m_minMz;
 
+  //qDebug() << "The stats to merge have m_maxMz:" << other.m_maxMz
+           //<< "and the current is:" << m_maxMz;
   if(other.m_maxMz > m_maxMz)
-    m_maxMz = other.m_maxMz;
+    {
+      m_maxMz = other.m_maxMz;
+
+      //qDebug() << "Since other.m_maxMz > m_maxMz, updated m_maxMz to:"
+               //<< m_maxMz;
+    }
 
   if(other.m_smallestMzShift < m_smallestMzShift)
     m_smallestMzShift = other.m_smallestMzShift;
@@ -746,9 +762,9 @@ MsRunDataSetStats::toString() const
                       m_spectrumSizeAvg,
                       m_spectrumSizeStdDev,
                       m_minMz,
+                      m_maxMz,
                       m_firstMzAvg,
                       m_firstMzStdDev,
-                      m_maxMz,
                       m_lastMzAvg,
                       m_lastMzStdDev,
                       m_smallestMzShift,
