@@ -34,6 +34,8 @@
 /////////////////////// StdLib includes
 #include <iostream>
 #include <iomanip>
+#include <cmath>
+
 
 /////////////////////// OpenMP include
 #include <omp.h>
@@ -48,17 +50,11 @@
 
 /////////////////////// Local includes
 #include "MsRunDataSetTreeMassDataIntegratorToRt.hpp"
-#include "ProcessingSpec.hpp"
 #include "ProcessingStep.hpp"
 #include "ProcessingType.hpp"
 #include "BaseMsRunDataSetTreeNodeVisitor.hpp"
 #include "TicChromTreeNodeCombinerVisitor.hpp"
-#include "IntensityTreeNodeCombinerVisitor.hpp"
-#include "RtDtMzColorMapsTreeNodeCombinerVisitor.hpp"
-#include "TraceTreeNodeCombinerVisitor.hpp"
 #include "MsRunStatisticsTreeNodeVisitor.hpp"
-#include "MassSpectrumTreeNodeCombinerVisitor.hpp"
-#include "DriftSpectrumTreeNodeCombinerVisitor.hpp"
 #include "MultiTreeNodeCombinerVisitor.hpp"
 
 
@@ -108,7 +104,7 @@ MsRunDataSetTreeMassDataIntegratorToRt::MsRunDataSetTreeMassDataIntegratorToRt(
 
 MsRunDataSetTreeMassDataIntegratorToRt::MsRunDataSetTreeMassDataIntegratorToRt(
   const MsRunDataSetTreeMassDataIntegratorToRt &other)
-  : MassDataIntegrator(other.mcsp_msRunDataSet)
+  : MassDataIntegrator(other)
 {
   // Essential that the m_processingFlow member is configured to have the right
   // pointer to the ms run data set.
@@ -124,7 +120,7 @@ MsRunDataSetTreeMassDataIntegratorToRt::
 }
 
 
-const ProcessingFlow &
+ProcessingFlow
 MsRunDataSetTreeMassDataIntegratorToRt::getProcessingFlow() const
 {
   return m_processingFlow;
@@ -153,13 +149,17 @@ void
 MsRunDataSetTreeMassDataIntegratorToRt::
   seedInitialTicChromatogramAndMsRunDataSetStatistics()
 {
-  // qDebug();
-
   std::chrono::system_clock::time_point chrono_start_time =
     std::chrono::system_clock::now();
 
+  // The number of nodes we want is that of the root nodes, not of all the
+  // nodes, that is, we want the MS1 nodes. Indeed, the division of the nodes
+  // into the various threads can only be performed with the root nodes.
+  std::size_t node_count =
+    mcsp_msRunDataSet->getMsRunDataSetTreeCstSPtr()->getRootNodes().size();
+
   // If there are no data, nothing to do.
-  if(!mcsp_msRunDataSet->getMsRunDataSetTreeCstSPtr()->getRootNodes().size())
+  if(!node_count)
     {
       qDebug() << "The ms run data set is empty, nothing to do.";
 
@@ -168,69 +168,61 @@ MsRunDataSetTreeMassDataIntegratorToRt::
       return;
     }
 
-  // Set aside a spec pointer that we may use more than once for heap
-  // allocations.
-  ProcessingSpec *spec_p = nullptr;
-
-  // Create the processing step to document the processing.
-  ProcessingStep *step_p = new ProcessingStep;
-
-  // qDebug() << "Allocate a processing spec instance.";
-
-  // We need to craft a new processing step, push it back to the processing
-  // flow and then call the proper integration function.
+  // qDebug() << "root nodes count:" << node_count;
 
   // Document that we only care of the MS1 data, because we are computing an
   // initial TIC chromatogram.
   MsFragmentationSpec ms_frag_spec;
   ms_frag_spec.setMsLevel(1);
 
-  // Now the real TIC chrom stuff, that needs to be pushed as the last item
-  // because that item will govern the actual visitor the end product that will
-  // be generated (that is, we want a TIC chromatogram).
+  // We need to craft a new processing step, push it back to the processing
+  // flow and then call the proper integration function.
 
-  // Allocate a new spec.
-  spec_p = new ProcessingSpec(ms_frag_spec);
+  ProcessingStep *step_p = new ProcessingStep(ms_frag_spec);
 
-  step_p->newSpec(ProcessingType("FILE_TO_RT"), spec_p);
+  double range_start = mcsp_msRunDataSet->getMsRunDataSetTreeCstSPtr()
+                         ->getRootNodes()
+                         .front()
+                         ->getQualifiedMassSpectrum()
+                         ->getRtInMinutes();
+  double range_end = mcsp_msRunDataSet->getMsRunDataSetTreeCstSPtr()
+                       ->getRootNodes()
+                       .back()
+                       ->getQualifiedMassSpectrum()
+                       ->getRtInMinutes();
 
-  // For debugging purposes, restrict the rt range to this range. Works
-  // (20190621).
-  // spec_p->setRange(0.2, 0.6);
+  // qDebug() << "Right from the ms run data set, get rt range start:"
+  //<< range_start << "and rt range end:" << range_end;
+
+  pappso::SelectionPolygon selection_polygon;
+  selection_polygon.set1D(range_start, range_end);
+
+  step_p->setSelectionPolygon(selection_polygon);
+
+  // qDebug() << "Created 1D selection polygon:"
+  //<< step_p->getSelectionPolygon().toString();
+
+  // qDebug() << "Is1D:" << step_p->getSelectionPolygon().is1D();
+
+  step_p->setSrcProcessingType(pappso::Axis::x, "FILE_RT");
+  step_p->setDataKind(pappso::Axis::x, pappso::DataKind::rt);
+
+  step_p->setSrcProcessingType(pappso::Axis::y, "NOT_SET");
+  step_p->setDataKind(pappso::Axis::y, pappso::DataKind::unset);
+
+  // Now document the destination processing type:
+  step_p->setDestProcessingType("RT");
+
+  // At this point we have document all the necessary to perform the
+  // integration.
 
   m_processingFlow.push_back(step_p);
 
   // We need to clear the map trace!
   m_mapTrace.clear();
 
-  // We need a processing flow to work, and, in particular, a processing step
-  // from which to find the specifics of the calculation.
-
-  if(!m_processingFlow.size())
-    qFatal("The processing flow cannot be empty. Program aborted.");
-
-  const ProcessingStep *processing_step_p = m_processingFlow.mostRecentStep();
-
-  // Now get a list of the integration types that are stored in the step's map.
-
-  std::vector<ProcessingType> processing_types =
-    processing_step_p->processingTypes();
-
-  if(!processing_types.size())
-    qFatal("The processing step cannot be empty. Program aborted.");
-
-  if(!processing_step_p->matches("ANY_TO_RT"))
-    qFatal("There should be one ProcessingType = ANY_TO_RT. Program aborted.");
-
   // Create as many multi-visitors as there are available threads.
 
-  // The number of nodes we want is that of the root nodes, not of all the
-  // nodes, that is, we want the MS1 nodes. Indeed, the division of the nodes
-  // into the various threads can only be performed with the root nodes.
-  std::size_t node_count =
-    mcsp_msRunDataSet->getMsRunDataSetTreeCstSPtr()->getRootNodes().size();
-
-  // qDebug() << "root nodes count:" << node_count;
   // emit setProgressBarMaxValueSignal(node_count);
 
   // In the pair below, first is the ideal number of threads and second is the
@@ -252,10 +244,6 @@ MsRunDataSetTreeMassDataIntegratorToRt::
     {
       // qDebug() << "thread index:" << iter;
 
-      // Set aside a stats holder instance.
-
-      MsRunDataSetStats ms_run_data_set_stats;
-
       MultiTreeNodeCombinerVisitorSPtr multi_visitor_sp =
         std::make_shared<MultiTreeNodeCombinerVisitor>(mcsp_msRunDataSet,
                                                        m_processingFlow);
@@ -264,12 +252,20 @@ MsRunDataSetTreeMassDataIntegratorToRt::
       // one visitor for initial TIC chrom calculation and one for the initial
       // ms run data set statistics.
 
+      // Start with first visitor that will compute the TIC chromatogram
       TicChromTreeNodeCombinerVisitorSPtr tic_visitor_sp =
         std::make_shared<TicChromTreeNodeCombinerVisitor>(mcsp_msRunDataSet,
                                                           m_processingFlow);
 
       multi_visitor_sp->m_visitors.push_back(tic_visitor_sp);
 
+      // And now the other visitor that will compute some statistics for the ms
+      // run data set.
+
+      // Set aside a stats holder instance.
+
+      MsRunDataSetStats ms_run_data_set_stats;
+
       MsRunStatisticsTreeNodeVisitorSPtr stats_visitor_sp =
         std::make_shared<MsRunStatisticsTreeNodeVisitor>(
           mcsp_msRunDataSet, m_processingFlow, ms_run_data_set_stats);
@@ -339,8 +335,8 @@ MsRunDataSetTreeMassDataIntegratorToRt::
 
   // Now perform a parallel iteration in the various visitors and ask them to
   // work on the matching ms run data set iterators pair.
-  omp_set_num_threads(visitors.size());
-#pragma omp parallel for ordered
+   omp_set_num_threads(visitors.size());
+  #pragma omp parallel for ordered
   for(std::size_t iter = 0; iter < visitors.size(); ++iter)
     {
       auto multi_visitor_sp = visitors.at(iter);
@@ -353,6 +349,13 @@ MsRunDataSetTreeMassDataIntegratorToRt::
         *(multi_visitor_sp.get()),
         iterators.at(iter).first,
         iterators.at(iter).second);
+
+      //qDebug() << "Finished running the visitor at index:" << iter
+               //<< "and the m_maxMz for the data stats is:"
+               //<< dynamic_cast<const MsRunStatisticsTreeNodeVisitor *>(
+                    //multi_visitor_sp->m_visitors.back().get())
+                    //->getMsRunDataSetStats()
+                    //.m_maxMz;
     }
   // End of
   // #pragma omp parallel for
@@ -422,14 +425,32 @@ MsRunDataSetTreeMassDataIntegratorToRt::
 
       emit setProgressBarCurrentValueSignal(iter + 1);
 
+      //qDebug() << "Going to merge a visitor's MsRunDataSetStats and the "
+                  //"m_maxMz for the data stats is:"
+               //<< dynamic_cast<const MsRunStatisticsTreeNodeVisitor *>(
+                    //multi_visitor_sp->m_visitors.back().get())
+                    //->getMsRunDataSetStats()
+                    //.m_maxMz;
+
+      //qDebug() << "We are merging one visitor's ms run data set stats:"
+               //<< stats_visitor_sp.get()->getMsRunDataSetStats().toString();
+
+      //qDebug()
+        //<< "Before the merge, the ms_run_data_set_stats_merge has m_maxMz:"
+        //<< ms_run_data_set_stats_merge.m_maxMz;
+
       ms_run_data_set_stats_merge.merge(
         stats_visitor_sp.get()->getMsRunDataSetStats());
+
+      //qDebug()
+        //<< "After the merge, the ms_run_data_set_stats_merge has m_maxMz:"
+        //<< ms_run_data_set_stats_merge.m_maxMz;
     }
 
   // At this point we finally have the statistical data merged into a single
   // instance. We now need to consolidate.
 
-  // qDebug() << "Final consolidation of the statistics.";
+  //qDebug() << "Final consolidation of the statistics.";
 
   ms_run_data_set_stats_merge.consolidate();
 
@@ -482,16 +503,9 @@ MsRunDataSetTreeMassDataIntegratorToRt::integrateToRt()
   // Get the most recent step that holds all the specifics of this integration.
   const ProcessingStep *processing_step_p = m_processingFlow.mostRecentStep();
 
-  // Now get a list of the integration types that are stored in the step's map.
-
-  std::vector<ProcessingType> processing_types =
-    processing_step_p->processingTypes();
-
-  if(!processing_types.size())
-    qFatal("The processing step cannot be empty. Program aborted.");
-
-  if(!processing_step_p->matches("ANY_TO_RT"))
-    qFatal("There should be one ProcessingType = ANY_TO_RT. Program aborted.");
+  if(!processing_step_p->srcMatches("ANY_RT"))
+    qFatal(
+      "There should be one source ProcessingType = ANY_RT. Program aborted.");
 
   // Try to limit the range of MS run data set tree nodes to be iterated through
   // by looking from what node to what other node we need to go to ensure that
@@ -500,10 +514,9 @@ MsRunDataSetTreeMassDataIntegratorToRt::integrateToRt()
   std::vector<pappso::MsRunDataSetTreeNode *> root_nodes =
     mcsp_msRunDataSet->getMsRunDataSetTreeCstSPtr()->getRootNodes();
 
-  double start_rt = std::numeric_limits<double>::infinity();
-  double end_rt   = std::numeric_limits<double>::infinity();
+  std::pair<double, double> range_pair;
 
-  bool integration_rt = m_processingFlow.innermostRtRange(start_rt, end_rt);
+  bool integration_rt = m_processingFlow.innermostRange("ANY_RT", range_pair);
 
   using Iterator = std::vector<pappso::MsRunDataSetTreeNode *>::const_iterator;
   using Pair     = std::pair<Iterator, Iterator>;
@@ -517,18 +530,18 @@ MsRunDataSetTreeMassDataIntegratorToRt::integrateToRt()
 
   if(integration_rt)
     {
-      pair =
-        mcsp_msRunDataSet->treeNodeIteratorRangeForRtRange(start_rt, end_rt);
+      pair = mcsp_msRunDataSet->treeNodeIteratorRangeForRtRange(
+        range_pair.first, range_pair.second);
 
       begin_iterator = pair.first;
       end_iterator   = pair.second;
 
       node_count = std::distance(begin_iterator, end_iterator);
 
-      qDebug() << "node_count:" << node_count;
+      // qDebug() << "node_count:" << node_count;
     }
-  else
-    qDebug() << "Not integration_rt";
+  // else
+  // qDebug() << "Not integration_rt";
 
   if(begin_iterator == root_nodes.end())
     {
@@ -713,204 +726,6 @@ MsRunDataSetTreeMassDataIntegratorToRt::integrateToRt()
 }
 
 
-#if 0
-void
-MsRunDataSetTreeMassDataIntegratorToRt::calculateInitialMsRunDataSetStatistics()
-{
-  // We want to parallelize the computation.
-
-  std::chrono::system_clock::time_point chrono_start_time =
-    std::chrono::system_clock::now();
-
-  // The number of nodes we want is that of the root nodes, not of all the
-  // nodes, that is, we want the MS1 nodes.
-  std::size_t node_count =
-    mcsp_msRunDataSet->getMsRunDataSetTreeCstSPtr()->getRootNodes().size();
-
-  // qDebug() << "node_count:" << node_count;
-  // emit setProgressBarMaxValueSignal(node_count);
-
-  // In the pair below, first is the ideal number of threads and second is the
-  // number of nodes per thread.
-  std::pair<std::size_t, std::size_t> best_parallel_params =
-    bestParallelIntegrationParams(node_count);
-
-  // qDebug() << "ideal_thread_count:" << best_parallel_params.first
-  //<< "nodes_per_thread:" << best_parallel_params.second;
-
-  using Iterator = std::vector<pappso::MsRunDataSetTreeNode *>::const_iterator;
-
-  std::vector<MsRunStatisticsTreeNodeVisitorSPtr> visitors;
-  std::vector<std::pair<Iterator, Iterator>> iterators = calculateMsRunDataSetTreeNodeIteratorPairs(
-    node_count, best_parallel_params.first, best_parallel_params.second);
-
-  for(std::size_t iter = 0; iter < iterators.size(); ++iter)
-    {
-      //qDebug() << "thread index:" << iter;
-
-      MsRunDataSetStats ms_run_data_set_stats;
-
-      MsRunStatisticsTreeNodeVisitorSPtr visitor_sp =
-        std::make_shared<MsRunStatisticsTreeNodeVisitor>(
-          mcsp_msRunDataSet, m_processingFlow, ms_run_data_set_stats);
-
-      // We want to be able to intercept any cancellation of the operation.
-
-      connect(this,
-              &MsRunDataSetTreeMassDataIntegratorToRt::cancelOperationSignal,
-              [visitor_sp]() { visitor_sp->cancelOperation(); });
-      // visitor_sp.get(),
-      //&TraceTreeNodeCombinerVisitor::cancelOperation,
-      // Qt::QueuedConnection);
-
-      // The visitor gets the number of nodes to process from the data set tree
-      // node. Capture that number and relay it.
-
-      connect(visitor_sp.get(),
-              &MsRunStatisticsTreeNodeVisitor::setProgressBarMaxValueSignal,
-              [this](std::size_t number_of_nodes_to_process) {
-                emit setProgressBarMaxValueSignal(number_of_nodes_to_process);
-              });
-
-      // The visitor emits the signal to tell that the currently iterated node
-      // has number_of_nodes_to_process children to process. Because the visitor
-      // might operate in a thread and that there might be other threads
-      // handling other nodes, we do not consider that the
-      // number_of_nodes_to_process value is the total number of nodes to
-      // process but only the number of nodes that the visitor is handling. We
-      // thus update the max progress bar value by number_of_nodes_to_process.
-      // In a typical setting, the user of the signal is the monitoring object,
-      // be it a non-gui object or the TaskMonitorCompositeWidget.
-      connect(
-        visitor_sp.get(),
-        &MsRunStatisticsTreeNodeVisitor::incrementProgressBarMaxValueSignal,
-        [this](std::size_t number_of_nodes_to_process) {
-          emit incrementProgressBarMaxValueSignal(number_of_nodes_to_process);
-        });
-
-      connect(visitor_sp.get(),
-              &MsRunStatisticsTreeNodeVisitor::setProgressBarCurrentValueSignal,
-              [this](std::size_t number_of_processed_nodes) {
-                emit setProgressBarCurrentValueSignal(
-                  number_of_processed_nodes);
-              });
-
-      connect(visitor_sp.get(),
-              &MsRunStatisticsTreeNodeVisitor::
-                incrementProgressBarCurrentValueAndSetStatusTextSignal,
-              [this](std::size_t increment, QString text) {
-                emit incrementProgressBarCurrentValueAndSetStatusTextSignal(
-                  increment, text);
-              });
-
-      connect(visitor_sp.get(),
-              &MsRunStatisticsTreeNodeVisitor::setStatusTextSignal,
-              [this](QString text) { emit setStatusTextSignal(text); });
-
-      connect(
-        visitor_sp.get(),
-        &MsRunStatisticsTreeNodeVisitor::setStatusTextAndCurrentValueSignal,
-        [this](QString text, std::size_t value) {
-          emit setStatusTextAndCurrentValueSignal(text, value);
-        });
-
-      visitors.push_back(visitor_sp);
-    }
-
-  emit setTaskDescriptionTextSignal(
-    "Computing the statistics of the MS data set");
-
-  omp_set_num_threads(visitors.size());
-#pragma omp parallel for
-  for(std::size_t iter = 0; iter < visitors.size(); ++iter)
-    {
-      auto stat_visitor = visitors.at(iter);
-
-      //qDebug() << "Visitor:" << &stat_visitor << "at index:" << iter
-               //<< "Executing from thread:" << QThread::currentThreadId();
-
-      mcsp_msRunDataSet->msp_msRunDataSetTree->accept(
-        *(stat_visitor.get()),
-        iterators.at(iter).first,
-        iterators.at(iter).second);
-    }
-  // End of
-  // #pragma omp parallel for
-
-  // At this point all the visitors have performed their task and each one has
-  // a MsRunDataSetStats member that was filled with statistical data about
-  // all the spectra visited.
-
-  // qDebug() << "End of the iteration in the visitors";
-
-
-  // If the task was cancelled, the monitor widget was locked. We need to
-  // unlock it.
-  emit unlockTaskMonitorCompositeWidgetSignal();
-  emit setupProgressBarSignal(0, visitors.size() - 1);
-
-  // We now need to merge all these MsRunDataSetStats data into a single one.
-  MsRunDataSetStats ms_run_data_set_stats_merge;
-
-  // We might be here because the user cancelled the operation in the for loop
-  // above (visiting all the visitors). In this case m_isOperationCancelled is
-  // true. We want to set it back to false, so that the following loop is gone
-  // through. The user can ask that the operation be cancelled once more. But we
-  // want that at least the performed work be used to show the trace.
-
-  m_isOperationCancelled = false;
-
-  for(std::size_t iter = 0; iter < visitors.size(); ++iter)
-    {
-      //qDebug() << "Now merging the data statistics for thread at index: "
-               //<< iter;
-
-      if(m_isOperationCancelled)
-        break;
-
-      auto &&visitor_sp = visitors.at(iter);
-
-      //qDebug() << "In the consolidating loop, iter:" << iter
-               //<< "for a new ms run data set statistics merge";
-
-      emit setStatusTextSignal(
-        QString("Consolidating statistics from thread %1").arg(iter + 1));
-
-      ms_run_data_set_stats_merge.merge(
-        visitor_sp.get()->getMsRunDataSetStats());
-    }
-
-
-  // At this point we finally have the statistical data merged into a single
-  // instance. We now need to consolidate.
-
-  //qDebug() << "Final consolidation of the statistics.";
-
-  ms_run_data_set_stats_merge.consolidate();
-
-  // At this point we can set the statistics to the ms run data set. We need to
-  // un-const_cast the shared pointer (this is one of the rarest situations
-  // where we need to do this).
-  std::const_pointer_cast<MsRunDataSet>(mcsp_msRunDataSet)
-    ->setMsRunDataSetStats(ms_run_data_set_stats_merge);
-
-  //qDebug() << mcsp_msRunDataSet->getMsRunDataSetStats().toString();
-
-  std::chrono::system_clock::time_point chrono_end_time;
-
-  QString chrono_string = pappso::Utils::chronoIntervalDebugString(
-    "Calculation of the MS run data set statistics took:",
-    chrono_start_time,
-    chrono_end_time);
-
-  emit logTextToConsoleSignal(chrono_string);
-  //qDebug().noquote() << chrono_string;
-
-  emit finishedCalculatingInitialMsRunDataSetStatisticsSignal(this);
-}
-#endif
-
-
 } // namespace minexpert
 
 } // namespace msxps
