@@ -77,7 +77,7 @@ class MsRunDataSetTreeMassDataIntegratorToRt : public MassDataIntegrator
 
   virtual ~MsRunDataSetTreeMassDataIntegratorToRt();
 
-  const ProcessingFlow &getProcessingFlow() const;
+  ProcessingFlow getProcessingFlow() const;
   void appendProcessingStep(ProcessingStep *processing_step_p);
 
   const MsRunDataSetStats &getMsRunDataSetStats() const;
