@@ -75,11 +75,6 @@ class MsRunStatisticsTreeNodeVisitor : public BaseMsRunDataSetTreeNodeVisitor
 
   virtual ~MsRunStatisticsTreeNodeVisitor();
 
-  virtual void setProcessingFlow(const ProcessingFlow &processing_flow);
-
-  void setLimitMzRangeStart(double value);
-  void setLimitMzRangeEnd(double value);
-
   const MsRunDataSetStats &getMsRunDataSetStats() const;
 
   MsRunStatisticsTreeNodeVisitor &
@@ -88,34 +83,8 @@ class MsRunStatisticsTreeNodeVisitor : public BaseMsRunDataSetTreeNodeVisitor
   // Overrides the base class.
   virtual bool visit(const pappso::MsRunDataSetTreeNode &node) override;
 
-  virtual bool
-  checkProcessingStep(const ProcessingStep &step,
-                      const pappso::MsRunDataSetTreeNode &node) override;
-
-  virtual bool checkProcessingSpec(
-    const std::pair<ProcessingType, ProcessingSpec *> &type_spec_pair,
-    const pappso::MsRunDataSetTreeNode &node) override;
-
-  virtual bool checkProcessingTypeByRt(
-    const std::pair<ProcessingType, ProcessingSpec *> &spec_type_pair,
-    const pappso::MsRunDataSetTreeNode &node) override;
-
-  virtual bool checkProcessingTypeByMz(
-    const std::pair<ProcessingType, ProcessingSpec *> &spec_type_pair,
-    const pappso::MsRunDataSetTreeNode &node) override;
-
-  virtual bool checkProcessingTypeByDt(
-    const std::pair<ProcessingType, ProcessingSpec *> &spec_type_pair,
-    const pappso::MsRunDataSetTreeNode &node) override;
-
   protected:
   MsRunDataSetStats m_msRunDataSetStats;
-
-  // We almost systematically need these m/z range values when making visits:
-  // one criterium might be that the m/z value of data points (the x member of
-  // DataPoint) be contained within the limit m/z range.
-  double m_limitMzRangeStart = std::numeric_limits<double>::max();
-  double m_limitMzRangeEnd   = std::numeric_limits<double>::max();
 };
 
 
