@@ -85,26 +85,6 @@ class MultiTreeNodeCombinerVisitor : public BaseMsRunDataSetTreeNodeVisitor
   // Overrides the base class.
   virtual bool visit(const pappso::MsRunDataSetTreeNode &node) override;
 
-  virtual bool
-  checkProcessingStep(const ProcessingStep &step,
-                      const pappso::MsRunDataSetTreeNode &node) override;
-
-  virtual bool checkProcessingSpec(
-    const std::pair<ProcessingType, ProcessingSpec *> &type_spec_pair,
-    const pappso::MsRunDataSetTreeNode &node) override;
-
-  virtual bool checkProcessingTypeByRt(
-    const std::pair<ProcessingType, ProcessingSpec *> &spec_type_pair,
-    const pappso::MsRunDataSetTreeNode &node) override;
-
-  virtual bool checkProcessingTypeByMz(
-    const std::pair<ProcessingType, ProcessingSpec *> &spec_type_pair,
-    const pappso::MsRunDataSetTreeNode &node) override;
-
-  virtual bool checkProcessingTypeByDt(
-    const std::pair<ProcessingType, ProcessingSpec *> &spec_type_pair,
-    const pappso::MsRunDataSetTreeNode &node) override;
-
   void addVisitor(BaseMsRunDataSetTreeNodeVisitorSPtr visitor_sp);
 
   protected:
