@@ -72,6 +72,8 @@ ProcessingFlow::ProcessingFlow(const ProcessingFlow &other)
 {
   for(auto &&step_p : other)
     push_back(new ProcessingStep(*step_p));
+
+  m_filterMap = other.m_filterMap;
 }
 
 
@@ -94,6 +96,8 @@ ProcessingFlow::operator=(const ProcessingFlow &other)
 
   assign(other.begin(), other.end());
 
+  m_filterMap = other.m_filterMap;
+
   return *this;
 }
 
@@ -158,6 +162,59 @@ ProcessingFlow::steps()
 }
 
 
+const std::map<QString, pappso::FilterNameInterfaceCstSPtr> &
+ProcessingFlow::getFilterMap() const
+{
+  return m_filterMap;
+}
+
+
+pappso::FilterNameInterfaceCstSPtr
+ProcessingFlow::filter(const QString &filter_name) const
+{
+  // qDebug() << "In ProcessingFlow:" << this;
+  // qDebug() << "Searching for filter:" << filter_name;
+
+  std::map<QString, pappso::FilterNameInterfaceCstSPtr>::const_iterator
+    found_iterator = m_filterMap.find(filter_name);
+
+  if(found_iterator == m_filterMap.end())
+    return nullptr;
+
+  return found_iterator->second;
+}
+
+
+void
+ProcessingFlow::addFilter(pappso::FilterNameInterfaceCstSPtr filter_csp)
+{
+  QString filter_name = filter_csp->name();
+
+  using Pair     = std::pair<QString, pappso::FilterNameInterfaceCstSPtr>;
+  using Map      = std::map<QString, pappso::FilterNameInterfaceCstSPtr>;
+  using Iterator = Map::iterator;
+
+  std::pair<Iterator, bool> res =
+    m_filterMap.insert(Pair(filter_name, filter_csp));
+
+  if(!res.second)
+    {
+      // Insertion failed because one other same filter by that name was already
+      // in the map. Set the new value.
+
+      res.first->second = filter_csp;
+    }
+  // else: the insert operation was performed fine.
+}
+
+
+std::size_t
+ProcessingFlow::removeFilter(const QString &filter_name)
+{
+  return m_filterMap.erase(filter_name);
+}
+
+
 bool
 ProcessingFlow::innermostRange(
   ProcessingType src_processing_type_mask,
@@ -166,9 +223,9 @@ ProcessingFlow::innermostRange(
   bool store_only_2d_steps,
   bool store_only_skewed) const
 {
-  qDebug() << "Searching for innermost range for source ProcessingType mask:"
-           << src_processing_type_mask.toBriefDesc() << "in a flow of" << size()
-           << "steps";
+  // qDebug() << "Searching for innermost range for source ProcessingType mask:"
+  //<< src_processing_type_mask.toBriefDesc() << "in a flow of" << size()
+  //<< "steps";
 
   // We are asked to get the innermost range for all steps having their
   // processing type matching the processing_type_mask.
@@ -193,7 +250,7 @@ ProcessingFlow::innermostRange(
 
   for(auto &&step_p : *this)
     {
-      qDebug() << "Iterating in step:" << step_p->toString();
+      // qDebug() << "Iterating in step:" << step_p->toString();
 
       double range_start;
       double range_end;
@@ -211,8 +268,8 @@ ProcessingFlow::innermostRange(
       if(!iter_x_processing_type.bitMatches(src_processing_type_mask) &&
          !iter_y_processing_type.bitMatches(src_processing_type_mask))
         {
-          qDebug() << "The iterated step has none of its source types matching "
-                      "the searched type. Skipping it.";
+          // qDebug() << "The iterated step has none of its source types
+          // matching " "the searched type. Skipping it.";
 
           continue;
         }
@@ -226,7 +283,7 @@ ProcessingFlow::innermostRange(
           // This is the simplest situation, there is only one dimension, the x
           // dimension, that is of interest.
 
-          qDebug() << "That step is 1D";
+          // qDebug() << "That step is 1D";
 
           step_p->m_selectionPolygon.rangeX(range_start, range_end);
 
@@ -235,13 +292,13 @@ ProcessingFlow::innermostRange(
               inner_range_start = range_start;
               inner_range_end   = range_end;
 
-              qDebug() << "First iterated step, with inner range:"
-                       << inner_range_start << "-" << inner_range_end;
+              // qDebug() << "First iterated step, with inner range:"
+              //<< inner_range_start << "-" << inner_range_end;
 
               if(!store_only_2d_steps)
                 {
-                  qDebug() << "Even if 1D, setting step as innermost step as "
-                              "requested.";
+                  // qDebug() << "Even if 1D, setting step as innermost step as
+                  // " "requested.";
                   innermost_step_p = step_p;
                 }
 
@@ -249,7 +306,7 @@ ProcessingFlow::innermostRange(
             }
           else
             {
-              qDebug() << "Not first iterated step.";
+              // qDebug() << "Not first iterated step.";
 
               // We now have to update the innermost range values.
 
@@ -261,7 +318,7 @@ ProcessingFlow::innermostRange(
 
                   was_one_dimension_inner = true;
 
-                  qDebug() << "New inner range start:" << inner_range_start;
+                  // qDebug() << "New inner range start:" << inner_range_start;
                 }
 
               if(range_end < inner_range_end)
@@ -269,13 +326,13 @@ ProcessingFlow::innermostRange(
                   inner_range_end = range_end;
 
                   was_one_dimension_inner = true;
-                  qDebug() << "New inner range end:" << inner_range_end;
+                  // qDebug() << "New inner range end:" << inner_range_end;
                 }
 
               if(was_one_dimension_inner && !store_only_2d_steps)
                 {
-                  qDebug() << "Even if 1D, setting step as innermost step as "
-                              "requested.";
+                  // qDebug() << "Even if 1D, setting step as innermost step as
+                  // " "requested.";
                   innermost_step_p = step_p;
                 }
             }
@@ -284,7 +341,7 @@ ProcessingFlow::innermostRange(
         }
       else
         {
-          qDebug() << "That step is 2D";
+          // qDebug() << "That step is 2D";
 
           // At this point, we know the selection polygon is 2D, which means we
           // have to check what actual source types has the iterated step
@@ -293,27 +350,27 @@ ProcessingFlow::innermostRange(
 
           if(step_p->m_xSrcType.bitMatches(src_processing_type_mask))
             {
-              qDebug() << "Step's m_xSrcType.bitMatches"
-                       << src_processing_type_mask.toBriefDesc();
+              // qDebug() << "Step's m_xSrcType.bitMatches"
+              //<< src_processing_type_mask.toBriefDesc();
 
               // The x axis is the axis we need to check the innermost range of.
 
               step_p->m_selectionPolygon.rangeX(range_start, range_end);
 
-              qDebug() << "Gotten range values:" << range_start << "-"
-                       << range_end;
+              // qDebug() << "Gotten range values:" << range_start << "-"
+              //<< range_end;
             }
           else if(step_p->m_ySrcType.bitMatches(src_processing_type_mask))
             {
-              qDebug() << "Step's m_ySrcType.bitMatches"
-                       << src_processing_type_mask.toBriefDesc();
+              // qDebug() << "Step's m_ySrcType.bitMatches"
+              //<< src_processing_type_mask.toBriefDesc();
 
               // The y axis is the axis we need to check the innermost range of.
 
               step_p->m_selectionPolygon.rangeY(range_start, range_end);
 
-              qDebug() << "Gotten range values:" << range_start << "-"
-                       << range_end;
+              // qDebug() << "Gotten range values:" << range_start << "-"
+              //<< range_end;
             }
           else
             qFatal("Programming error.");
@@ -323,17 +380,17 @@ ProcessingFlow::innermostRange(
               inner_range_start = range_start;
               inner_range_end   = range_end;
 
-              qDebug() << "First iterated step, with inner range:"
-                       << inner_range_start << "-" << inner_range_end;
+              // qDebug() << "First iterated step, with inner range:"
+              //<< inner_range_start << "-" << inner_range_end;
 
-              qDebug() << "Setting step as innermost step.";
+              // qDebug() << "Setting step as innermost step.";
               innermost_step_p = step_p;
 
               is_first_iterated_item = false;
             }
           else
             {
-              qDebug() << "Not first iterated step.";
+              // qDebug() << "Not first iterated step.";
 
               // We now have to update the innermost range values.
 
@@ -345,7 +402,7 @@ ProcessingFlow::innermostRange(
 
                   was_one_dimension_inner = true;
 
-                  qDebug() << "New inner range start:" << inner_range_start;
+                  // qDebug() << "New inner range start:" << inner_range_start;
                 }
 
               if(range_end < inner_range_end)
@@ -354,12 +411,12 @@ ProcessingFlow::innermostRange(
 
                   was_one_dimension_inner = true;
 
-                  qDebug() << "New inner range end:" << inner_range_end;
+                  // qDebug() << "New inner range end:" << inner_range_end;
                 }
 
               if(was_one_dimension_inner)
                 {
-                  qDebug() << "Setting step as innermost step.";
+                  // qDebug() << "Setting step as innermost step.";
                   innermost_step_p = step_p;
                 }
             }
@@ -369,7 +426,7 @@ ProcessingFlow::innermostRange(
         }
     }
 
-  qDebug() << "Finished iterating in the vector of steps.";
+  // qDebug() << "Finished iterating in the vector of steps.";
 
   // At this point we have iterated in all the steps. To know if at least one
   // step was found matching the src_processing_type_mask, we can test the
@@ -389,16 +446,16 @@ ProcessingFlow::innermostRange(
 
   if(out_processing_steps_p != nullptr)
     {
-      qDebug() << "The caller wants the innermost step to be stored.";
+      // qDebug() << "The caller wants the innermost step to be stored.";
 
       if(innermost_step_p != nullptr)
         {
-          qDebug() << "One innermost step was indeed found.";
+          // qDebug() << "One innermost step was indeed found.";
 
           if(!store_only_skewed)
             {
-              qDebug()
-                << "The caller wants *all* 2D steps, not only the skewed ones.";
+              // qDebug()
+              //<< "The caller wants *all* 2D steps, not only the skewed ones.";
               should_store_the_step_in_the_end = true;
             }
           else
@@ -406,21 +463,21 @@ ProcessingFlow::innermostRange(
               if(!innermost_step_p->getSelectionPolygon().isRectangle())
                 {
                   should_store_the_step_in_the_end = true;
-                  qDebug() << "The caller wants *only* the 2D skewed steps and "
-                              "the step *is* skewed";
-                }
-              else
-                {
-                  qDebug() << "The caller wants *only* the 2D skewed steps and "
-                              "the step is *not* skewed";
+                  // qDebug() << "The caller wants *only* the 2D skewed steps
+                  // and " "the step *is* skewed";
                 }
+              // else
+              //{
+              // qDebug() << "The caller wants *only* the 2D skewed steps and "
+              //"the step is *not* skewed";
+              //}
             }
         }
-      else
-        qDebug() << "Not a single step was found.";
+      // else
+      // qDebug() << "Not a single step was found.";
     }
-  else
-    qDebug() << "The caller does *not* want the innermost step to be stored.";
+  // else
+  // qDebug() << "The caller does *not* want the innermost step to be stored.";
 
   // At this point, if we have to store the step, check that another
   // call by another processing type mask has not set it already
@@ -428,7 +485,7 @@ ProcessingFlow::innermostRange(
 
   if(should_store_the_step_in_the_end)
     {
-      qDebug() << "Really, the step should be stored.";
+      // qDebug() << "Really, the step should be stored.";
 
       // We only push back the range if it has not been found
       // already. Note that this function might called multiple
@@ -442,8 +499,8 @@ ProcessingFlow::innermostRange(
             {
               was_step_found = true;
 
-              qDebug() << "Did not store the step because it had been "
-                          "stored already.";
+              // qDebug() << "Did not store the step because it had been "
+              //"stored already.";
 
               break;
             }
@@ -451,8 +508,8 @@ ProcessingFlow::innermostRange(
 
       if(!was_step_found)
         {
-          qDebug() << "The step is actually skewed, so storing it:"
-                   << innermost_step_p->toString();
+          // qDebug() << "The step is actually skewed, so storing it:"
+          //<< innermost_step_p->toString();
 
           out_processing_steps_p->push_back(*innermost_step_p);
         }
@@ -482,27 +539,27 @@ ProcessingFlow::innermostRange(
 
 
 #if 0
-bool
-ProcessingFlow::innermostRtRange(
-  std::pair<ProcessingType, ProcessingStep *> &pair) const
-{
-  return innermostRange(ProcessingType("RT_TO_ANY"), pair);
-}
+    bool
+      ProcessingFlow::innermostRtRange(
+          std::pair<ProcessingType, ProcessingStep *> &pair) const
+      {
+        return innermostRange(ProcessingType("RT_TO_ANY"), pair);
+      }
 
 
-bool
-ProcessingFlow::innermostMzRange(double &range_start, double &range_end) const
-{
-  return innermostRange(ProcessingType("MZ_TO_ANY"), range_start, range_end);
-}
+    bool
+      ProcessingFlow::innermostMzRange(double &range_start, double &range_end) const
+      {
+        return innermostRange(ProcessingType("MZ_TO_ANY"), range_start, range_end);
+      }
 
 
-bool
-ProcessingFlow::innermostMzRange(
-  std::pair<ProcessingType, ProcessingStep *> &pair) const
-{
-  return innermostRange(ProcessingType("MZ_TO_ANY"), pair);
-}
+    bool
+      ProcessingFlow::innermostMzRange(
+          std::pair<ProcessingType, ProcessingStep *> &pair) const
+      {
+        return innermostRange(ProcessingType("MZ_TO_ANY"), pair);
+      }
 
 #endif
 
@@ -837,6 +894,22 @@ ProcessingFlow::toString(int offset, const QString &spacer) const
         }
     }
 
+  // Now handle the filter map. We'll only report the names of filters.
+
+  if(m_filterMap.size())
+    {
+      text += "\nFilters:\n";
+
+      for(std::map<QString, pappso::FilterNameInterfaceCstSPtr>::const_iterator
+            iterator = m_filterMap.begin();
+          iterator != m_filterMap.end();
+          ++iterator)
+        {
+          text += iterator->first;
+          text += "\n";
+        }
+    }
+
   return text;
 }
 
