@@ -33,6 +33,8 @@
 
 /////////////////////// StdLib includes
 #include <vector>
+#include <cmath>
+
 
 /////////////////////// OPENMP include
 #include <omp.h>
@@ -48,6 +50,7 @@
 
 /////////////////////// Local includes
 #include "QualifiedMassSpectrumVectorMassDataIntegratorToDt.hpp"
+#include "ProcessingStep.hpp"
 
 
 int qualifiedMassSpectrumVectorMassDataIntegratorToDtMetaTypeId =
@@ -102,256 +105,10 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDt::
 }
 
 
-// Old version
-#if 0
 void
-QualifiedMassSpectrumVectorMassDataIntegratorToDt::integrateToDt()
+QualifiedMassSpectrumVectorMassDataIntegratorToDt::integrate()
 {
-  //qDebug();
-
-  std::chrono::system_clock::time_point chrono_start_time =
-    std::chrono::system_clock::now();
-
-  // If there are no data, nothing to do.
-  std::size_t mass_spectra_count = m_vectorOfQualifiedMassSpectraSPtr->size();
-
-  if(!mass_spectra_count)
-    {
-      qDebug() << "The qualified mass spectrum vector is empty, nothing to do.";
-
-      emit cancelOperationSignal();
-    }
-
-  // Nothing special to do about processing flow info because that must have
-  // been set before calling this function.
-
-  // We need to clear the map trace!
-  m_mapTrace.clear();
-
-  // Determine the best number of threads to use and how many spectra to provide
-  // each of the threads.
-
-  // In the pair below, first is the ideal number of threads and second is the
-  // number of mass spectra per thread.
-  std::pair<std::size_t, std::size_t> best_parallel_params =
-    bestParallelIntegrationParams(mass_spectra_count);
-
-  // Handy alias.
-  using Iterator =
-    std::vector<pappso::QualifiedMassSpectrumCstSPtr>::const_iterator;
-
-  // Prepare a set of iterators that will distribute all the mass spectra
-  // uniformly.
-  std::vector<std::pair<Iterator, Iterator>> iterators =
-    calculateQualifiedMassSpectrumVectorIteratorPairs(
-      best_parallel_params.first, best_parallel_params.second);
-
-  // Set aside a vector of std::map<double, double> to store retention time
-  // values and their TIC intensity
-
-  // Handy aliases.
-  using Pair    = std::pair<double, double>;
-  using Map     = std::map<double, double>;
-  using MapSPtr = std::shared_ptr<Map>;
-
-  std::vector<MapSPtr> vector_of_dt_spec_map;
-  for(std::size_t iter = 0; iter < iterators.size(); ++iter)
-    vector_of_dt_spec_map.push_back(
-      std::make_shared<Map>());
-
-  // Now that we have all the parallel material, we can start the parallel work.
-  emit setProgressBarMaxValueSignal(mass_spectra_count);
-
-  omp_set_num_threads(iterators.size());
-#pragma omp parallel for ordered
-  for(std::size_t iter = 0; iter < iterators.size(); ++iter)
-    {
-      qDebug() << "thread index:" << iter;
-
-      // Get the iterator range that we need to deal in this thread.
-      Iterator current_iterator = iterators.at(iter).first;
-      Iterator end_iterator     = iterators.at(iter).second;
-
-      // Get the map for this thread.
-
-      MapSPtr current_dt_spec_map_sp = vector_of_dt_spec_map.at(iter);
-
-      std::size_t qualified_mass_spectrum_count =
-        std::distance(current_iterator, end_iterator);
-
-      qDebug() << "For thread index:" << iter
-               << "the number of mass spectra to process:"
-               << qualified_mass_spectrum_count;
-
-      while(current_iterator != end_iterator)
-        {
-          // We want to be able to intercept any cancellation of the
-          // operation.
-
-          if(m_isOperationCancelled)
-            break;
-
-          // Make the computation.
-
-          // For each mass spectrum all we need is compute its TIC value, that
-          // is, the sum of all its intensities (the y value of the DataPoint
-          // instances it contains).
-
-          pappso::QualifiedMassSpectrumCstSPtr qualified_mass_spectrum_csp =
-            *current_iterator;
-
-          // Test if the mass spectrum is nullptr.
-
-          pappso::MassSpectrumCstSPtr mass_spectrum_csp =
-            qualified_mass_spectrum_csp->getMassSpectrumCstSPtr();
-
-          if(qualified_mass_spectrum_csp == nullptr)
-            qFatal("Cannot be nullptr");
-          if(qualified_mass_spectrum_csp.get() == nullptr)
-            qFatal("Cannot be nullptr");
-
-          if(qualified_mass_spectrum_csp->getMassSpectrumSPtr() == nullptr)
-            {
-              //qDebug() << "Need to access the mass data right from the file.";
-
-              std::size_t mass_spectrum_index =
-                mcsp_msRunDataSet->getMsRunDataSetTreeCstSPtr()
-                  ->massSpectrumIndex(qualified_mass_spectrum_csp);
-
-              pappso::MsRunReaderCstSPtr ms_run_reader_csp =
-                mcsp_msRunDataSet->getMsRunReaderCstSPtr();
-
-              qualified_mass_spectrum_csp =
-                std::make_shared<pappso::QualifiedMassSpectrum>(
-                  ms_run_reader_csp->qualifiedMassSpectrum(mass_spectrum_index,
-                                                           true));
-            }
-
-          if(qualified_mass_spectrum_csp == nullptr)
-            {
-              qFatal("Failed to read the mass spectral data from the file.");
-            }
-
-          // Great, at this point we actually have the mass spectrum!
-
-          double mass_spectrum_sum_y = 0;
-          mass_spectrum_sum_y = qualified_mass_spectrum_csp->getMassSpectrumSPtr()->sumY();
-
-          if(!mass_spectrum_sum_y)
-            {
-              continue;
-            }
-
-          double dt = qualified_mass_spectrum_csp->getDtInMilliSeconds();
-
-          using Iterator = Map::iterator;
-
-          // Try to insert the pair into the map. Check if that was done or not.
-          std::pair<Iterator, bool> res =
-            current_dt_spec_map_sp->insert(Pair(dt, mass_spectrum_sum_y));
-
-          if(!res.second)
-            {
-              // One other same rt value was seen already (like in ion mobility
-              // mass spectrometry, for example). Only increment the y value.
-
-              res.first->second += mass_spectrum_sum_y;
-            }
-
-          // qDebug() << "Processed one more spectrum.";
-
-          emit incrementProgressBarCurrentValueAndSetStatusTextSignal(
-            1, "Processing spectra");
-
-          // Now we can go to the next mass spectrum.
-          ++current_iterator;
-        }
-        // End of
-        // while(current_iterator != end_iterator)
-    }
-  // End of
-  // #pragma omp parallel for ordered
-  // for(std::size_t iter = 0; iter < iterators.size(); ++iter)
-
-  // In the loop above, for each thread, a std::map<double, double> map was
-  // filled in with the rt, TIC pairs.
-
-  // If the task was cancelled, the monitor widget was locked. We need to
-  // unlock it.
-  emit unlockTaskMonitorCompositeWidgetSignal();
-  emit setupProgressBarSignal(0, vector_of_dt_spec_map.size() - 1);
-
-  // We might be here because the user cancelled the operation in the for loop
-  // above (visiting all the visitors). In this case m_isOperationCancelled is
-  // true. We want to set it back to false, so that the following loop is gone
-  // through. The user can ask that the operation be cancelled once more. But
-  // we want that at least the performed work be used to show the trace.
-
-  m_isOperationCancelled = false;
-  int iter               = 0;
-
-  for(auto &&map_sp : vector_of_dt_spec_map)
-    {
-      ++iter;
-
-      if(m_isOperationCancelled)
-        break;
-
-      //qDebug() << "In the consolidating loop, iter:" << iter
-               //<< "with a map of this size : " << map_sp->size();
-
-      for(auto &&pair : *map_sp)
-        {
-
-          emit setStatusTextSignal(
-            QString("Consolidating drift spectra from thread %1")
-              .arg(iter));
-
-          emit setProgressBarCurrentValueSignal(iter);
-
-          double dt  = pair.first;
-          double tic = pair.second;
-
-          using Iterator = Map::iterator;
-
-          std::pair<Iterator, bool> res = m_mapTrace.insert(Pair(dt, tic));
-
-          if(!res.second)
-            {
-              // One other same rt value was seen already (like in ion mobility
-              // mass spectrometry, for example). Only increment the y value.
-
-              res.first->second += tic;
-            }
-        }
-    }
-
-	// Write the data to file so that we can check. It works !!!
-  // pappso::Utils::appendToFile(m_mapTrace.toString(), "/tmp/prova.txt");
-
-  std::chrono::system_clock::time_point chrono_end_time =
-    std::chrono::system_clock::now();
-
-  QString chrono_string = pappso::Utils::chronoIntervalDebugString(
-    "Integration to drift spectrum took:",
-    chrono_start_time,
-    chrono_end_time);
-
-  emit logTextToConsoleSignal(chrono_string);
-
-  //qDebug().noquote() << chrono_string;
-
-  return;
-}
-
-// Old version
-#endif
-
-
-void
-QualifiedMassSpectrumVectorMassDataIntegratorToDt::integrateToDt()
-{
-  // qDebug();
+  qDebug();
 
   std::chrono::system_clock::time_point chrono_start_time =
     std::chrono::system_clock::now();
@@ -367,8 +124,7 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDt::integrateToDt()
       emit cancelOperationSignal();
     }
 
-  // Nothing special to do about processing flow info because that must have
-  // been set before calling this function.
+  qDebug() << "The number of mass spectra to handle:" << mass_spectra_count;
 
   // We need to clear the map trace!
   m_mapTrace.clear();
@@ -381,21 +137,36 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDt::integrateToDt()
   if(!m_processingFlow.size())
     qFatal("The processing flow cannot be empty. Program aborted.");
 
-  // Get the most recent step that holds all the specifics of this integration.
-  const ProcessingStep *most_recent_processing_step_p =
-    m_processingFlow.mostRecentStep();
+  // When constructing this integrator, the processing flow passed as parameter
+  // was used to gather innermost ranges data. When the steps that match
+  // the innermost criterion, these are stored in m_innermostSteps2D. When the
+  // steps are non-2D steps, then only the m_xxRange values are updated.
 
-  // There must be at least one ProcessingSpec (and thus a type) in the
-  // processing step.
+  // If the m_innermostSteps2D vector of ProcessingSteps is non-empty, then,
+  // that means that we may have some MZ-related 2D processing steps to account
+  // at combination time, because that is precisely the only moment that we can
+  // do that work. To be able to provide selection polygon an mass
+  // spectrum-specific dt|rt data, we have the SelectionPolygonSpec class. There
+  // can be as many such class instances as required to document the integration
+  // boundaries rt/mz and dt/mz for example. This is why we need to fill in the
+  // vector of SelectionPolygonSpec and then set that to the combiner.
 
-  if(!most_recent_processing_step_p->processingTypes().size())
-    qFatal("The processing step cannot be empty. Program aborted.");
+  // Note, however, that the hassle described above only is acceptable when the
+  // selection polygon is not square. If the selection polygon is square, then
+  // the ranges suffice, and they have been filled-in properly upon construction
+  // of this integrator.
 
-  // We are integrating to m/z so we need a spec description that matches
-  // "ANY_TO_MZ".
+  qDebug() << "Now filling-in the selection_polygon_specs for any 2D step that "
+              "has a source type involving the MZ data kind.";
 
-  if(!most_recent_processing_step_p->matches("ANY_TO_DT"))
-    qFatal("There should be one ProcessingType = ANY_TO_DT. Program aborted.");
+  std::vector<pappso::SelectionPolygonSpec> selection_polygon_specs;
+
+  std::size_t selection_polygon_spec_count =
+    fillInSelectionPolygonSpecs(selection_polygon_specs);
+
+  qDebug() << selection_polygon_spec_count
+           << "selection polygon specs were retained for the configuration of "
+              "the filter.";
 
   // At this point, allocate a visitor that is specific for the calculation of
   // the mass spectrum. Since we are not binning, we can use a visitor that
@@ -411,8 +182,7 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDt::integrateToDt()
   // In the pair below, first is the ideal number of threads and second is the
   // number of mass spectra per thread.
   std::pair<std::size_t, std::size_t> best_parallel_params =
-    bestParallelIntegrationParams(
-      mcsp_qualifiedMassSpectraToIntegrateVector->size());
+    bestParallelIntegrationParams(mass_spectra_count);
 
   // Handy alias.
   using MassSpecVector = std::vector<pappso::QualifiedMassSpectrumCstSPtr>;
@@ -437,14 +207,11 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDt::integrateToDt()
   for(std::size_t iter = 0; iter < iterators.size(); ++iter)
     vector_of_dt_spec_map.push_back(std::make_shared<DtTicMap>());
 
-
-  // Now that we have all the parallel material, we can start the parallel work.
-  emit setProgressBarMaxValueSignal(
-    mcsp_qualifiedMassSpectraToIntegrateVector->size());
-
   // Now that we have all the parallel material, we can start the parallel work.
   emit setProgressBarMaxValueSignal(mass_spectra_count);
 
+  qDebug() << "Starting the iteration in the visitors.";
+
   omp_set_num_threads(iterators.size());
 #pragma omp parallel for ordered
   for(std::size_t iter = 0; iter < iterators.size(); ++iter)
@@ -482,6 +249,11 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDt::integrateToDt()
           qualified_mass_spectrum_csp =
             checkQualifiedMassSpectrum(qualified_mass_spectrum_csp);
 
+          if(qualified_mass_spectrum_csp == nullptr)
+            qFatal(
+              "Programming error. Not possible that shared pointer to "
+              "qualified mass spectrum be nullptr.");
+
           // Great, at this point we actually have the mass spectrum!
 
           // We now need to ensure that the processing flow allows for this mass
@@ -491,74 +263,42 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDt::integrateToDt()
           // that matches the greatest MS level found in the member processing
           // flow instance.
 
-          if(!checkMsLevel(qualified_mass_spectrum_csp) ||
-             !checkRtRange(qualified_mass_spectrum_csp) ||
-             !checkDtRange(qualified_mass_spectrum_csp))
+          if(!checkMsLevel(qualified_mass_spectrum_csp))
             {
-              // if(!checkMsLevel(qualified_mass_spectrum_csp))
-              // qDebug() << "MS level did not check.";
-
-              // if(!checkRtRange(qualified_mass_spectrum_csp))
-              // qDebug() << "RT range did not check.";
-
-              // if(!checkDtRange(qualified_mass_spectrum_csp))
-              // qDebug() << "DT range did not check.";
-
-              // qDebug() << "The checks returned false";
-
+              qDebug() << "The MsLevel check returned false";
+              ++current_iterator;
+              continue;
+            }
+          if(!checkMsFragmentation(qualified_mass_spectrum_csp))
+            {
+              qDebug() << "The MsFragmentation check returned false";
               ++current_iterator;
               continue;
             }
 
-          // At this point we need to go deep in the processing flow's steps to
-          // check if the node matches the whole set of the steps' specs.
+          // Make easy check about RT and DT.
 
-          // qDebug() << "The processing flow has" << m_processingFlow.size() <<
-          // "steps.";
-
-          // Boolean value needed because we need to break two loops.
-          bool should_increment_iterator_and_continue = false;
-
-          for(auto &&step : m_processingFlow)
+          if(!checkRtRange(qualified_mass_spectrum_csp))
             {
-              if(!checkProcessingStep(*step, qualified_mass_spectrum_csp))
-                {
-                  should_increment_iterator_and_continue = true;
-                  break;
-                }
+              qDebug() << "checkRtRange failed check.";
+
+              ++current_iterator;
+              continue;
             }
-
-          if(should_increment_iterator_and_continue)
+          if(!checkDtRange(qualified_mass_spectrum_csp))
             {
+              qDebug() << "checkDtRange failed check.";
+
               ++current_iterator;
               continue;
             }
 
           double mass_spectrum_sum_y = 0;
 
-          // These values are set in the in the base class constuctor.
-
-          if(m_mzRange.first != std::numeric_limits<double>::max() &&
-             m_mzRange.second != std::numeric_limits<double>::min())
-            {
-              mass_spectrum_sum_y =
-                qualified_mass_spectrum_csp->getMassSpectrumSPtr()->sumY(
-                  m_mzRange.first, m_mzRange.second);
-            }
-          else
-            {
-              mass_spectrum_sum_y =
-                qualified_mass_spectrum_csp->getMassSpectrumSPtr()->sumY();
-            }
-
-          if(!mass_spectrum_sum_y)
-            {
-              ++current_iterator;
-              continue;
-            }
-
-          // Now handle the mobility data, either drift time or 1/K0 depending
-          // on the file type.
+          // Handle the mobility data, either drift time or 1/K0 depending
+          // on the file type. Do that now because we'll need the dt value when
+          // performing the sumY calculation if a filtering with selection in
+          // polygon is required.
 
           double dt = -1;
 
@@ -595,6 +335,65 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDt::integrateToDt()
               dt = qualified_mass_spectrum_csp->getDtInMilliSeconds();
             }
 
+          // At this point we need to account for any limitations in the m/z
+          // dimension of the data. These limitations might be as easy as having
+          // a single m/z range limit or as complex as having one or more 2D
+          // steps involving the m/z dimension in the m_innermostSteps2D, if the
+          // selection polygons are not rectangle.
+
+          if(!selection_polygon_specs.size())
+            {
+              if(!std::isnan(m_mzRange.first) && !std::isnan(m_mzRange.second))
+                {
+                  mass_spectrum_sum_y =
+                    qualified_mass_spectrum_csp->getMassSpectrumSPtr()->sumY(
+                      m_mzRange.first, m_mzRange.second);
+                }
+              else
+                {
+                  mass_spectrum_sum_y =
+                    qualified_mass_spectrum_csp->getMassSpectrumSPtr()->sumY();
+                }
+            }
+          else
+            {
+              // It may be possible that the 2D steps limiting the m/z range
+              // have been followed by 1D steps further limiting the m/Z range.
+              // The point is, the 1D steps are not stored in
+              // selection_polygon_specs. However, by necessity, if there is at
+              // least one selection_polygon_spec, then the m_mzRange holds
+              // values. Futher, if there had been a 1D step limiting even more
+              // the m/z range, then that step would have been accounted for in
+              // m_mzRange. So we can account that limit right away.
+
+              if(std::isnan(m_mzRange.first) || std::isnan(m_mzRange.second))
+                qFatal("Programming error.");
+
+              pappso::Trace filtered_trace(
+                *qualified_mass_spectrum_csp->getMassSpectrumSPtr());
+
+              pappso::FilterResampleKeepXRange the_keep_filter(
+                m_mzRange.first, m_mzRange.second);
+
+              filtered_trace = the_keep_filter.filter(filtered_trace);
+
+              pappso::FilterResampleKeepPointInPolygon the_polygon_filter(
+                selection_polygon_specs);
+
+              filtered_trace = the_polygon_filter.filter(
+                filtered_trace,
+                dt,
+                qualified_mass_spectrum_csp->getRtInMinutes());
+
+              mass_spectrum_sum_y = filtered_trace.sumY();
+            }
+
+          if(!mass_spectrum_sum_y)
+            {
+              ++current_iterator;
+              continue;
+            }
+
           // Try to insert the pair into the map. Check if that was done or
           // not.
           std::pair<DtTicMapIterator, bool> res =
@@ -616,14 +415,12 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDt::integrateToDt()
           // Now we can go to the next mass spectrum.
           ++current_iterator;
         }
-        // End of
-        // while(current_iterator != end_iterator)
+      // End of
+      // while(current_iterator != end_iterator)
 
-#if 0
-			qDebug() << "At the end of the current iterator pair, current map:";
-			for(auto &&pair : *current_dt_spec_map_sp)
-				qDebug().noquote() << "(" << pair.first << "," << pair.second << ")";
-#endif
+      // qDebug() << "At the end of the current iterator pair, current map:";
+      // for(auto &&pair : *current_dt_spec_map_sp)
+      // qDebug().noquote() << "(" << pair.first << "," << pair.second << ")";
     }
   // End of
   // #pragma omp parallel for ordered
@@ -662,8 +459,7 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDt::integrateToDt()
         {
 
           emit setStatusTextSignal(
-            QString("Consolidating drift spectra from thread %1")
-              .arg(iter));
+            QString("Consolidating drift spectra from thread %1").arg(iter));
 
           emit setProgressBarCurrentValueSignal(iter);
 
