@@ -34,6 +34,8 @@
 /////////////////////// StdLib includes
 #include <iostream>
 #include <iomanip>
+#include <cmath>
+
 
 /////////////////////// OpenMP include
 #include <omp.h>
@@ -48,7 +50,6 @@
 
 /////////////////////// Local includes
 #include "QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz.hpp"
-#include "ProcessingSpec.hpp"
 #include "ProcessingStep.hpp"
 #include "ProcessingType.hpp"
 
@@ -170,19 +171,13 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::
 
 
 void
-QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::integrate(
-  pappso::DataKind data_kind)
+QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::integrate()
 {
-  // qDebug() << "Data kind:" << static_cast<int>(data_kind);
+  qDebug();
 
   std::chrono::system_clock::time_point chrono_start_time =
     std::chrono::system_clock::now();
 
-  // In this integration, we are going to craft the data to create a colormap
-  // that will plot mz data with either rt or dt data (depending on data_kind).
-
-  m_lastIntegrationDataKind = data_kind;
-
   // If there are no data, nothing to do.
   std::size_t mass_spectra_count =
     mcsp_qualifiedMassSpectraToIntegrateVector->size();
@@ -194,12 +189,37 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::integrate(
       emit cancelOperationSignal();
     }
 
-  // Nothing special to do about processing flow info because that must have
-  // been set before calling this function.
+  qDebug() << "The number of mass spectra to handle:" << mass_spectra_count;
 
   // We need to clear the map trace!
   m_mapTrace.clear();
 
+  // In this integration, we are going to craft the data to create a colormap
+  // that will plot mz data against either rt or dt data. The way we know if we
+  // have to plot mz data against one or the other is by looking into the
+  // processing flow's processing type.
+
+  if(!m_processingFlow.size())
+    qFatal("The processing flow cannot be empty. Program aborted.");
+
+  const ProcessingStep *last_processing_step =
+    m_processingFlow.mostRecentStep();
+
+  ProcessingType last_processing_type =
+    last_processing_step->getDestProcessingType();
+
+  // We need to know what kind of integration we deal with here, rt_mz or dt_mz
+  // because we have to store it so that after the computation is performed, the
+  // consumer of this integrator knows what integration was performed. It will
+  // have no other means to know what kind of integration we have performed.
+
+  if(last_processing_type.bitMatches("DT_MZ"))
+    m_lastIntegrationDataKind = pappso::DataKind::dt;
+  else if(last_processing_type.bitMatches("MZ_RT"))
+    m_lastIntegrationDataKind = pappso::DataKind::rt;
+  else
+    qFatal("Programming error. The processing is not allowed here.");
+
   // Choose the right message to emit and make sanity checks.
   if(m_lastIntegrationDataKind == pappso::DataKind::rt)
     {
@@ -214,22 +234,44 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::integrate(
   else
     qFatal("Programming error.");
 
+  // When constructing this integrator, the processing flow passed as parameter
+  // was used to gather innermost ranges data. When the steps that match
+  // the innermost criterion, these are stored in m_innermostSteps2D. When the
+  // steps are non-2D steps, then only the m_xxRange values are updated.
 
-  // We need a processing flow to work, and, in particular, a processing step
-  // from which to find the specifics of the calculation.
+  // If the m_innermostSteps2D vector of ProcessingSteps is non-empty, then,
+  // that means that we may have some MZ-related 2D processing steps to account
+  // at combination time, because that is precisely the only moment that we can
+  // do that work. To be able to provide selection polygon an mass
+  // spectrum-specific dt|rt data, we have the SelectionPolygonSpec class. There
+  // can be as many such class instances as required to document the integration
+  // boundaries rt/mz and dt/mz for example. This is why we need to fill in the
+  // vector of SelectionPolygonSpec and then set that to the combiner.
 
-  if(!m_processingFlow.size())
-    qFatal("The processing flow cannot be empty. Program aborted.");
+  // Note, however, that the hassle described above only is acceptable when the
+  // selection polygon is not square. If the selection polygon is square, then
+  // the ranges suffice, and they have been filled-in properly upon construction
+  // of this integrator.
 
-  // Get the most recent step that holds all the specifics of this integration.
-  const ProcessingStep *most_recent_processing_step_p =
-    m_processingFlow.mostRecentStep();
+  qDebug() << "Now filling-in the selectionpolygonspecs.";
 
-  // There must be at least one ProcessingSpec (and thus type) in the
-  // processing step since we need to know what color map is being computed
-  // (either rt/mz or dt/mz).
-  if(!most_recent_processing_step_p->processingTypes().size())
-    qFatal("The processing step must have at least one type. Program aborted.");
+  std::vector<pappso::SelectionPolygonSpec> selection_polygon_specs;
+
+  std::size_t selection_polygon_spec_count =
+    fillInSelectionPolygonSpecs(selection_polygon_specs);
+
+  qDebug() << selection_polygon_spec_count
+           << "selection polygon specs were retained for the configuration of "
+              "the filter.";
+
+  // Prepare the template for all the combiners we'll use in the different
+  // threads.
+
+  // Set the decimal places to 0 below, because we have a large
+  // binning for the color map with mz data, otherwize it would
+  // be huge.
+
+  pappso::TracePlusCombiner plus_combiner(0);
 
   // Using a shared pointer make it easy to handle the copying of the map to
   // users of that map without bothering.
@@ -284,6 +326,8 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::integrate(
   // Now that we have all the parallel material, we can start the parallel work.
   emit setProgressBarMaxValueSignal(mass_spectra_count);
 
+  qDebug() << "Starting the iteration in the visitors.";
+
   omp_set_num_threads(iterators.size());
 #pragma omp parallel for ordered
   for(std::size_t iter = 0; iter < iterators.size(); ++iter)
@@ -299,23 +343,6 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::integrate(
       MapSPtr current_double_maptrace_map_sp =
         vector_of_double_maptrace_maps.at(iter);
 
-      // Set the decimal places to 0 below, because we have a large
-      // binning for the color map with mz data, otherwize it would
-      // be huge.
-      pappso::TracePlusCombiner combiner(0);
-
-      // We need to inform the combiner if we have to limit the mz range
-      // for the combinations.
-
-      // These values are set in the in the base class constuctor.
-
-      if(m_mzRange.first != std::numeric_limits<double>::max() &&
-         m_mzRange.second != std::numeric_limits<double>::min())
-        {
-          combiner.setFilterResampleKeepXRange(pappso::FilterResampleKeepXRange(
-            m_mzRange.first, m_mzRange.second));
-        }
-
       while(current_iterator != end_iterator)
         {
           // We want to be able to intercept any cancellation of the
@@ -339,6 +366,11 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::integrate(
           qualified_mass_spectrum_csp =
             checkQualifiedMassSpectrum(qualified_mass_spectrum_csp);
 
+          if(qualified_mass_spectrum_csp == nullptr)
+            qFatal(
+              "Programming error. Not possible that shared pointer to "
+              "qualified mass spectrum be nullptr.");
+
           // Great, at this point we actually have the mass spectrum!
 
           // We now need to ensure that the processing flow allows for this mass
@@ -348,45 +380,32 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::integrate(
           // that matches the greatest MS level found in the member processing
           // flow instance.
 
-          if(!checkMsLevel(qualified_mass_spectrum_csp) ||
-             !checkRtRange(qualified_mass_spectrum_csp) ||
-             !checkDtRange(qualified_mass_spectrum_csp))
+          if(!checkMsLevel(qualified_mass_spectrum_csp))
             {
-              // if(!checkMsLevel(qualified_mass_spectrum_csp))
-              // qDebug() << "MS level did not check.";
-
-              // if(!checkRtRange(qualified_mass_spectrum_csp))
-              // qDebug() << "RT range did not check.";
-
-              // if(!checkDtRange(qualified_mass_spectrum_csp))
-              // qDebug() << "DT range did not check.";
-
-              // qDebug() << "The checks returned false";
-
+              qDebug() << "The MsLevel check returned false";
+              ++current_iterator;
+              continue;
+            }
+          if(!checkMsFragmentation(qualified_mass_spectrum_csp))
+            {
+              qDebug() << "The MsFragmentation check returned false";
               ++current_iterator;
               continue;
             }
 
-          // At this point we need to go deep in the processing flow's steps to
-          // check if the node matches the whole set of the steps' specs.
+          // Make easy check about RT and DT.
 
-          // qDebug() << "The processing flow has" << m_processingFlow.size() <<
-          // "steps.";
-
-          // Boolean value needed because we need to break two loops.
-          bool should_increment_iterator_and_continue = false;
-
-          for(auto &&step : m_processingFlow)
+          if(!checkRtRange(qualified_mass_spectrum_csp))
             {
-              if(!checkProcessingStep(*step, qualified_mass_spectrum_csp))
-                {
-                  should_increment_iterator_and_continue = true;
-                  break;
-                }
+              qDebug() << "checkRtRange failed check.";
+
+              ++current_iterator;
+              continue;
             }
-
-          if(should_increment_iterator_and_continue)
+          if(!checkDtRange(qualified_mass_spectrum_csp))
             {
+              qDebug() << "checkDtRange failed check.";
+
               ++current_iterator;
               continue;
             }
@@ -394,15 +413,18 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::integrate(
           // Great, at this point we actually have the mass spectrum and we need
           // to get the dt|rt value for it.
 
+          double rt_value = qualified_mass_spectrum_csp->getRtInMinutes();
+          double dt_value = qualified_mass_spectrum_csp->getDtInMilliSeconds();
+
           double first_key; // that is, either rt or dt.
 
-          if(data_kind == pappso::DataKind::rt)
+          if(m_lastIntegrationDataKind == pappso::DataKind::rt)
             {
-              first_key = qualified_mass_spectrum_csp->getRtInMinutes();
+              first_key = rt_value;
             }
-          else if(data_kind == pappso::DataKind::dt)
+          else if(m_lastIntegrationDataKind == pappso::DataKind::dt)
             {
-              first_key = qualified_mass_spectrum_csp->getDtInMilliSeconds();
+              first_key = dt_value;
             }
           else
             qFatal("Programming error.");
@@ -415,6 +437,42 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::integrate(
           double_maptrace_map_iterator =
             current_double_maptrace_map_sp->find(first_key);
 
+          // Before combining, we need to check if some filtering is
+          // required as for m/z data.
+
+          pappso::Trace filtered_trace(
+            *qualified_mass_spectrum_csp->getMassSpectrumSPtr());
+
+          // It may be possible that the 2D steps limiting the m/z range
+          // have been followed by 1D steps further limiting the m/z range.
+          // The point is, the 1D steps are not stored in
+          // selection_polygon_specs. However, by necessity, if there is at
+          // least one selection_polygon_spec, then the m_mzRange holds
+          // values. Futher, if there had been a 1D step limiting even more
+          // the m/z range, then that step would have been accounted for in
+          // m_mzRange. So we can account that limit right away.
+
+          if(!std::isnan(m_mzRange.first) && !std::isnan(m_mzRange.second))
+            {
+              pappso::FilterResampleKeepXRange the_keep_filter(
+                m_mzRange.first, m_mzRange.second);
+
+              filtered_trace = the_keep_filter.filter(filtered_trace);
+            }
+
+          if(selection_polygon_specs.size())
+            {
+              pappso::FilterResampleKeepPointInPolygon the_polygon_filter(
+                selection_polygon_specs);
+
+              filtered_trace = the_polygon_filter.filter(
+                filtered_trace,
+                qualified_mass_spectrum_csp->getDtInMilliSeconds(),
+                qualified_mass_spectrum_csp->getRtInMinutes());
+            }
+
+          // At this point we can make the combination with the filtered trace.
+
           // Tells if the key was already found in the map.
           if(double_maptrace_map_iterator !=
              current_double_maptrace_map_sp->end())
@@ -430,9 +488,8 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::integrate(
               // qDebug() << "Before combination, map trace map size: "
               //<< double_map_trace_map_iterator->second.size();
 
-              combiner.combine(
-                double_maptrace_map_iterator->second,
-                *(qualified_mass_spectrum_csp->getMassSpectrumCstSPtr()));
+              plus_combiner.combine(double_maptrace_map_iterator->second,
+                                    filtered_trace);
             }
           else
             {
@@ -450,9 +507,7 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::integrate(
 
               pappso::MapTrace map_trace;
 
-              combiner.combine(
-                map_trace,
-                *(qualified_mass_spectrum_csp->getMassSpectrumCstSPtr()));
+              plus_combiner.combine(map_trace, filtered_trace);
 
               // Finally, store in the map, the map_trace along with its
               // first_key.
@@ -495,6 +550,9 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::integrate(
   m_isOperationCancelled = false;
   int iter               = 0;
 
+  // This time we do not need to configure the combiner with m/z dimension
+  // related limitations because these have been enforced during the first
+  // combination round above.
   pappso::TracePlusCombiner combiner(0);
 
   for(auto &&map_sp : vector_of_double_maptrace_maps)
@@ -540,11 +598,15 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::integrate(
 
               // Combine the new mass spectrum right into the found MapTrace.
 
-              // qDebug() << "Before combination, map trace map size: "
-              //<< double_map_trace_map_iterator->second.size();
+              //std::size_t before_size = current_maptrace.size();
 
               combiner.combine(double_maptrace_map_iterator->second,
                                current_maptrace);
+
+              //std::size_t after_size = current_maptrace.size();
+
+              //qDebug() << "Before combination, map trace map size: "
+                       //<< before_size << "after combination:" << after_size;
             }
           else
             {
@@ -562,8 +624,15 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::integrate(
 
               pappso::MapTrace map_trace;
 
+              //std::size_t before_size = map_trace.size();
+
               combiner.combine(map_trace, current_maptrace);
 
+              //std::size_t after_size = map_trace.size();
+
+              //qDebug() << "Before combination, map trace map size: "
+                       //<< before_size << "after combination:" << after_size;
+
               // Finally, store in the map, the map_trace along with its
               // first_key.
 
@@ -612,12 +681,12 @@ QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz::integrate(
 
   QString chrono_string;
 
-  if(data_kind == pappso::DataKind::rt)
+  if(m_lastIntegrationDataKind == pappso::DataKind::rt)
     chrono_string = pappso::Utils::chronoIntervalDebugString(
       "Integration to RT / MZ colormap took:",
       chrono_start_time,
       chrono_end_time);
-  else if(data_kind == pappso::DataKind::dt)
+  else if(m_lastIntegrationDataKind == pappso::DataKind::dt)
     chrono_string = pappso::Utils::chronoIntervalDebugString(
       "Integration to DT / MZ colormap took:",
       chrono_start_time,
