@@ -99,7 +99,7 @@ class QualifiedMassSpectrumVectorMassDataIntegratorToDtRtMz
 
   public slots:
 
-  void integrate(pappso::DataKind data_kind);
+  void integrate();
 
 
   signals:
