@@ -415,14 +415,14 @@ QualifiedMassSpectrumVectorMassDataIntegratorToMz::integrateToMzNoBinning()
 
           // qDebug() << "Going to combine a new qualified mass spectrum.";
 
-          std::size_t before_size = current_map_trace_sp->size();
+          //std::size_t before_size = current_map_trace_sp->size();
 
           trace_plus_combiner.combine(*current_map_trace_sp, filtered_trace);
 
-          std::size_t after_size = current_map_trace_sp->size();
+          //std::size_t after_size = current_map_trace_sp->size();
 
-          qDebug() << "Before combination, map trace map size: " << before_size
-                   << "after combination:" << after_size;
+          //qDebug() << "Before combination, map trace map size: " << before_size
+                   //<< "after combination:" << after_size;
 
           // qDebug() << "Processed one more spectrum.";
 
