@@ -87,7 +87,7 @@ class QualifiedMassSpectrumVectorMassDataIntegratorToMz
   public slots:
 
 
-  void integrateToMz();
+  void integrate();
   signals:
 
   protected:
