@@ -85,7 +85,7 @@ class QualifiedMassSpectrumVectorMassDataIntegratorToRt
 
   public slots:
 
-  void integrateToRt();
+  void integrate();
 
   signals:
 
