@@ -34,6 +34,8 @@
 /////////////////////// StdLib includes
 #include <iostream>
 #include <iomanip>
+#include <cmath>
+
 
 /////////////////////// OpenMP include
 #include <omp.h>
@@ -48,7 +50,6 @@
 
 /////////////////////// Local includes
 #include "QualifiedMassSpectrumVectorMassDataIntegratorToRtDt.hpp"
-#include "ProcessingSpec.hpp"
 #include "ProcessingStep.hpp"
 #include "ProcessingType.hpp"
 
@@ -154,10 +155,10 @@ QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::getColorMapMaxMz() const
 
 using Map = std::map<double, pappso::MapTrace>;
 const std::shared_ptr<Map> &
-QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::getDoubleMapTraceMapSPtr()
+QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::getRtMapTraceMapSPtr()
   const
 {
-  return m_doubleMapTraceMapSPtr;
+  return m_rtMapTraceMapSPtr;
 }
 
 pappso::DataKind
@@ -169,20 +170,13 @@ QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::
 
 
 void
-QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::integrate(
-  pappso::DataKind data_kind)
+QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::integrate()
 {
-  // qDebug();
+  qDebug();
 
   std::chrono::system_clock::time_point chrono_start_time =
     std::chrono::system_clock::now();
 
-  // In this integration, we are going to craft the data to create a colormap
-  // that will plot rt data with either dt data or vice versa (depending on
-  // data_kind).
-
-  m_lastIntegrationDataKind = data_kind;
-
   // If there are no data, nothing to do.
   std::size_t mass_spectra_count =
     mcsp_qualifiedMassSpectraToIntegrateVector->size();
@@ -196,46 +190,70 @@ QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::integrate(
 
   qDebug() << "The number of mass spectra to handle:" << mass_spectra_count;
 
-  // Nothing special to do about processing flow info because that must have
-  // been set before calling this function.
-
   // We need to clear the map trace!
   m_mapTrace.clear();
 
   emit setTaskDescriptionTextSignal(
     "Integrating to a TIC|XIC chromatogram / drift spectrum color map");
 
-  // We need a processing flow to work, and, in particular, a processing step
-  // from which to find the specifics of the calculation.
+  // In this integration, we are going to craft the data to create a colormap
+  // that will plot dt data against rt data or vice versa.
 
   if(!m_processingFlow.size())
     qFatal("The processing flow cannot be empty. Program aborted.");
 
-  // Get the most recent step that holds all the specifics of this integration.
-  const ProcessingStep *most_recent_processing_step_p =
-    m_processingFlow.mostRecentStep();
+  // When constructing this integrator, the processing flow passed as parameter
+  // was used to gather innermost ranges data. When the steps that match
+  // the innermost criterion, these are stored in m_innermostSteps2D. When the
+  // steps are non-2D steps, then only the m_xxRange values are updated.
 
-  // There must be at least one ProcessingSpec (and thus a type) in the
-  // processing step.
-  if(!most_recent_processing_step_p->processingTypes().size())
-    qFatal("The processing step cannot be empty. Program aborted.");
+  // If the m_innermostSteps2D vector of ProcessingSteps is non-empty, then,
+  // that means that we may have some MZ-related 2D processing steps to account
+  // at combination time, because that is precisely the only moment that we can
+  // do that work. To be able to provide selection polygon an mass
+  // spectrum-specific dt|rt data, we have the SelectionPolygonSpec class. There
+  // can be as many such class instances as required to document the integration
+  // boundaries rt/mz and dt/mz for example. This is why we need to fill in the
+  // vector of SelectionPolygonSpec and then set that to the combiner.
 
-  if(!most_recent_processing_step_p->matches("ANY_TO_RT"))
-    qFatal("There should be one ProcessingType = ANY_TO_RT. Program aborted.");
+  // Note, however, that the hassle described above only is acceptable when the
+  // selection polygon is not square. If the selection polygon is square, then
+  // the ranges suffice, and they have been filled-in properly upon construction
+  // of this integrator.
+
+  qDebug() << "Now filling-in the selection_polygon_specs.";
+
+  std::vector<pappso::SelectionPolygonSpec> selection_polygon_specs;
+
+  std::size_t selection_polygon_spec_count =
+    fillInSelectionPolygonSpecs(selection_polygon_specs);
+
+  qDebug() << selection_polygon_spec_count
+           << "selection polygon specs were retained for the configuration of "
+              "the filter.";
 
   // Using a shared pointer make it easy to handle the copying of the map to
   // users of that map without bothering.
 
-  if(m_doubleMapTraceMapSPtr == nullptr)
-    m_doubleMapTraceMapSPtr =
+  if(m_rtMapTraceMapSPtr == nullptr)
+    m_rtMapTraceMapSPtr =
       std::make_shared<std::map<double, pappso::MapTrace>>();
   else
-    m_doubleMapTraceMapSPtr->clear();
+    m_rtMapTraceMapSPtr->clear();
 
   // In this function, rt is the x_axis and dt is the y_axis, for each (rt,dt)
-  // pair, the TIC intensity is the color. So we have m_doubleMapTraceMapSPtr =
+  // pair, the TIC intensity is the color. So we have m_rtMapTraceMapSPtr =
   // <rt, <dt, int>>.
 
+  // The logic here is that in IM-MS experiments, for a given retention time
+  // (rt), the might be hundrds of drift time-resolvec mass spectra. So we
+  // create a map that relates rt values with a MapTrace, itself relating dt
+  // values (map key) and TICint values (map value). During iterating in all the
+  // mass spectra, we look at the rt value of the mass spectrum, we then look at
+  // its dt value. We compute a TICint of the mass spectrum that we set in the
+  // MapTrace, associated with the dt value. That MapTrace is then set the
+  // member map that maps the rt value (map key) with the MapTrace (map value).
+
   // We want to parallelize the computation. Se we will allocate a vector of
   // iterators (as many as possible for the available processor threads), each
   // iterator having a subset of the initial data to integrate.
@@ -266,14 +284,14 @@ QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::integrate(
   using Map     = std::map<double, pappso::MapTrace>;
   using MapSPtr = std::shared_ptr<Map>;
 
-  std::vector<MapSPtr> vector_of_double_maptrace_maps;
+  std::vector<MapSPtr> vector_of_rt_maptrace_maps;
   for(std::size_t iter = 0; iter < iterators.size(); ++iter)
-    vector_of_double_maptrace_maps.push_back(std::make_shared<Map>());
+    vector_of_rt_maptrace_maps.push_back(std::make_shared<Map>());
 
   // Now that we have all the parallel material, we can start the parallel work.
   emit setProgressBarMaxValueSignal(mass_spectra_count);
 
-  qDebug() << "number of iterators:" << iterators.size();
+  qDebug() << "Starting the iteration in the visitors.";
 
   omp_set_num_threads(iterators.size());
 #pragma omp parallel for ordered
@@ -287,8 +305,7 @@ QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::integrate(
 
       // Get the map for this thread.
 
-      MapSPtr current_double_maptrace_map_sp =
-        vector_of_double_maptrace_maps.at(iter);
+      MapSPtr iterated_rt_maptrace_map_sp = vector_of_rt_maptrace_maps.at(iter);
 
       while(current_iterator != end_iterator)
         {
@@ -316,6 +333,11 @@ QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::integrate(
           qualified_mass_spectrum_csp =
             checkQualifiedMassSpectrum(qualified_mass_spectrum_csp);
 
+          if(qualified_mass_spectrum_csp == nullptr)
+            qFatal(
+              "Programming error. Not possible that shared pointer to "
+              "qualified mass spectrum be nullptr.");
+
           // Great, at this point we actually have the mass spectrum!
 
           // We now need to ensure that the processing flow allows for this mass
@@ -325,129 +347,127 @@ QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::integrate(
           // that matches the greatest MS level found in the member processing
           // flow instance.
 
-          if(!checkMsLevel(qualified_mass_spectrum_csp) ||
-             !checkRtRange(qualified_mass_spectrum_csp) ||
-             !checkDtRange(qualified_mass_spectrum_csp))
+          if(!checkMsLevel(qualified_mass_spectrum_csp))
             {
-              // if(!checkMsLevel(qualified_mass_spectrum_csp))
-              // qDebug() << "MS level did not check.";
+              qDebug() << "The MsLevel check returned false";
+              ++current_iterator;
+              continue;
+            }
+          if(!checkMsFragmentation(qualified_mass_spectrum_csp))
+            {
+              qDebug() << "The MsFragmentation check returned false";
+              ++current_iterator;
+              continue;
+            }
 
-              // if(!checkRtRange(qualified_mass_spectrum_csp))
-              // qDebug() << "RT range did not check.";
+          // Make easy check about RT and DT.
 
-              // if(!checkDtRange(qualified_mass_spectrum_csp))
-              // qDebug() << "DT range did not check.";
+          if(!checkRtRange(qualified_mass_spectrum_csp))
+            {
+              qDebug() << "checkRtRange failed check.";
 
-              qDebug() << "The checks returned false";
+              ++current_iterator;
+              continue;
+            }
+          if(!checkDtRange(qualified_mass_spectrum_csp))
+            {
+              qDebug() << "checkDtRange failed check.";
 
               ++current_iterator;
               continue;
             }
 
-          // At this point we need to go deep in the processing flow's steps to
-          // check if the node matches the whole set of the steps' specs.
-
-          // qDebug() << "The processing flow has" << m_processingFlow.size() <<
-          // "steps.";
-
-          // Boolean value needed because we need to break two loops.
-          bool should_increment_iterator_and_continue = false;
-
-          for(auto &&step : m_processingFlow)
-            {
-              if(!checkProcessingStep(*step, qualified_mass_spectrum_csp))
-                {
-                  should_increment_iterator_and_continue = true;
-                  break;
-                }
-            }
-
-          if(should_increment_iterator_and_continue)
-            {
-              ++current_iterator;
-              continue;
-            }
-
-          // Great, at this point we actually know that we have to account for
-          // the mass spectrum and we need to get the dt|rt value for it.
-
-          double first_key;  // that is, either rt or dt.
-          double second_key; // that is, either rt or dt.
-
-          if(m_lastIntegrationDataKind == pappso::DataKind::rt)
-            {
-              first_key  = qualified_mass_spectrum_csp->getRtInMinutes();
-              second_key = qualified_mass_spectrum_csp->getDtInMilliSeconds();
-            }
-          else if(m_lastIntegrationDataKind == pappso::DataKind::dt)
-            {
-              first_key  = qualified_mass_spectrum_csp->getDtInMilliSeconds();
-              second_key = qualified_mass_spectrum_csp->getRtInMinutes();
-            }
-          else
-            qFatal("Programming error.");
-
-          // We want to know what is the TIC intensity value of the spectrum.
-          // Note that we need to check if we are limited in the mz range that
-          // needs to be accounted for in the TIC sum calculation.
+          double dt_key = qualified_mass_spectrum_csp->getDtInMilliSeconds();
+          double rt_key = qualified_mass_spectrum_csp->getRtInMinutes();
 
           double mass_spectrum_sum_y = 0;
 
-          // These values are set in the in the base class constuctor.
+          // We want to know what is the TIC intensity value of the spectrum.
 
-          if(m_mzRange.first != std::numeric_limits<double>::max() &&
-             m_mzRange.second != std::numeric_limits<double>::min())
+          // At this point we need to account for any limitations in the m/z
+          // dimension of the data. These limitations might be as easy as having
+          // a single m/z range limit or as complex as having one or more 2D
+          // steps involving the m/z dimension in the m_innermostSteps2D, if the
+          // selection polygons are not rectangle.
+
+          if(!selection_polygon_specs.size())
             {
-              // qDebug() << "applying mz range filtering:" << m_mzRange.first
-              //<< "-" << m_mzRange.second;
-
-              mass_spectrum_sum_y =
-                qualified_mass_spectrum_csp->getMassSpectrumSPtr()->sumY(
-                  m_mzRange.first, m_mzRange.second);
+              if(!std::isnan(m_mzRange.first) && !std::isnan(m_mzRange.second))
+                {
+                  mass_spectrum_sum_y =
+                    qualified_mass_spectrum_csp->getMassSpectrumSPtr()->sumY(
+                      m_mzRange.first, m_mzRange.second);
+                }
+              else
+                {
+                  mass_spectrum_sum_y =
+                    qualified_mass_spectrum_csp->getMassSpectrumSPtr()->sumY();
+                }
             }
           else
             {
-              // qDebug() << "NOT applying mz range filtering.";
+              // It may be possible that the 2D steps limiting the m/z range
+              // have been followed by 1D steps further limiting the m/Z range.
+              // The point is, the 1D steps are not stored in
+              // selection_polygon_specs. However, by necessity, if there is at
+              // least one selection_polygon_spec, then the m_mzRange holds
+              // values. Futher, if there had been a 1D step limiting even more
+              // the m/z range, then that step would have been accounted for in
+              // m_mzRange. So we can account that limit right away.
 
-              mass_spectrum_sum_y =
-                qualified_mass_spectrum_csp->getMassSpectrumSPtr()->sumY();
+              if(std::isnan(m_mzRange.first) || std::isnan(m_mzRange.second))
+                qFatal("Programming error.");
+
+              pappso::Trace filtered_trace(
+                *qualified_mass_spectrum_csp->getMassSpectrumSPtr());
+
+              pappso::FilterResampleKeepXRange the_keep_filter(
+                m_mzRange.first, m_mzRange.second);
+
+              filtered_trace = the_keep_filter.filter(filtered_trace);
+
+              pappso::FilterResampleKeepPointInPolygon the_polygon_filter(
+                selection_polygon_specs);
+
+              filtered_trace =
+                the_polygon_filter.filter(filtered_trace, dt_key, rt_key);
+
+              mass_spectrum_sum_y = filtered_trace.sumY();
             }
 
           if(!mass_spectrum_sum_y)
             {
-              // qDebug() << "The sum_y is 0";
-
               ++current_iterator;
               continue;
             }
 
-          // Now look into the <double, MapTrace> map if we already encountered
-          // the first_key value. Store the found place in an iterator.
+          // Now look into the <double, MapTrace> map that relates rt values
+          // with MapTraces, if we already encountered the rt_key value. Store
+          // the found place in an iterator.
 
           Map::iterator double_maptrace_map_iterator;
 
           double_maptrace_map_iterator =
-            current_double_maptrace_map_sp->find(first_key);
+            iterated_rt_maptrace_map_sp->find(rt_key);
 
           // Tells if the key was already found in the map.
-          if(double_maptrace_map_iterator !=
-             current_double_maptrace_map_sp->end())
+          if(double_maptrace_map_iterator != iterated_rt_maptrace_map_sp->end())
             {
 
               // The map already contained a pair that had the key value ==
-              // first_key. The ->second member of the iterator is the MapTrace
-              // that maps <second_key, TICint>. So we now need to search for
-              // a map item that has key == second_key.
+              // rt_key (->first membero of the pair). The ->second member of
+              // the pair is the MapTrace that maps <dt_key, TICint>. So we
+              // now need to search for a map item that has key == dt_key.
 
               using MapIterator = std::map<double, double>::iterator;
 
               std::pair<MapIterator, bool> res =
                 double_maptrace_map_iterator->second.insert(
-                  std::pair<double, double>(second_key, mass_spectrum_sum_y));
+                  std::pair<double, double>(dt_key, mass_spectrum_sum_y));
 
               if(!res.second)
                 {
-                  // One other same second_key value was seen already. Only
+                  // One other same dt_key value was seen already. Only
                   // increment the y value.
 
                   res.first->second += mass_spectrum_sum_y;
@@ -456,23 +476,23 @@ QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::integrate(
           else
             {
 
-              // This is the first time that we encounter first_key. So we need
-              // to create a new MapTrace for that first_key. That MapTrace
+              // This is the first time that we encounter dt_key. So we need
+              // to create a new MapTrace for that dt_key. That MapTrace
               // (std::map<double,double>) will store the TIC values for all the
-              // second_key elements.
+              // dt_key elements.
 
               // Instantiate a new MapTrace in which we'll store all the TIC
-              // intensities for all the second_key map items.
+              // intensities for all the dt_key map items.
 
               pappso::MapTrace map_trace;
 
               map_trace.insert(
-                std::pair<double, double>(second_key, mass_spectrum_sum_y));
+                std::pair<double, double>(dt_key, mass_spectrum_sum_y));
 
               // Finally, store in the map<double, MapTrace> map, the map_trace
-              // along with its first_key.
+              // along with its dt_key.
 
-              (*current_double_maptrace_map_sp)[first_key] = map_trace;
+              (*iterated_rt_maptrace_map_sp)[rt_key] = map_trace;
             }
 
           // At this point we have processed the qualified mass spectrum.
@@ -496,7 +516,7 @@ QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::integrate(
   // If the task was cancelled, the monitor widget was locked. We need to
   // unlock it.
   emit unlockTaskMonitorCompositeWidgetSignal();
-  emit setupProgressBarSignal(0, vector_of_double_maptrace_maps.size() - 1);
+  emit setupProgressBarSignal(0, vector_of_rt_maptrace_maps.size() - 1);
 
   // We might be here because the user cancelled the operation in the for loop
   // above (visiting all the visitors). In this case m_isOperationCancelled is
@@ -509,7 +529,7 @@ QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::integrate(
 
   pappso::TracePlusCombiner combiner(0);
 
-  for(auto &&map_sp : vector_of_double_maptrace_maps)
+  for(auto &&map_sp : vector_of_rt_maptrace_maps)
     {
       // qDebug();
 
@@ -520,7 +540,7 @@ QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::integrate(
 
       for(auto &&pair : *map_sp)
         {
-          // first is the first_key
+          // first is the dt_key
           // second is the pappso::MapTrace
 
           emit setStatusTextSignal(
@@ -528,35 +548,34 @@ QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::integrate(
 
           emit setProgressBarCurrentValueSignal(iter);
 
-          double first_key = pair.first;
+          double dt_key = pair.first;
 
           pappso::MapTrace current_maptrace = pair.second;
 
-          // Now search in the member if a first_key was found already.
+          // Now search in the member if a dt_key was found already.
 
           Map::iterator double_maptrace_map_iterator;
 
-          double_maptrace_map_iterator =
-            m_doubleMapTraceMapSPtr->find(first_key);
+          double_maptrace_map_iterator = m_rtMapTraceMapSPtr->find(dt_key);
 
           // Tells if the key was already found in the map.
-          if(double_maptrace_map_iterator != m_doubleMapTraceMapSPtr->end())
+          if(double_maptrace_map_iterator != m_rtMapTraceMapSPtr->end())
             {
               // qDebug() << "Adding new key value:" << rt_or_dt_key;
 
               // The map already contained a pair that had the key value ==
-              // first_key. We want to sum, map item by map item all the map
+              // dt_key. We want to sum, map item by map item all the map
               // items in the current_maptrace MapTrace to
               // the combined one that will hold the final color map data:
               // double_maptrace_map_iterator->second.
 
               // For example, thread 1 yielded a <double, MapTrace> map like
-              // this: <rt, map<dt, tic>>. rt is first_key and dt is second_key.
+              // this: <rt, map<dt, tic>>. rt is dt_key and dt is rt_key.
 
               // We search in the member <double, MapTrace> map if an item has a
-              // value matching rt (that is, first_key). If found then we need
+              // value matching rt (that is, dt_key). If found then we need
               // to combine the iterated MapTrace into the member MapTrace for
-              // key first_key. It's like if MapTrace were a mass spectrum, but
+              // key dt_key. It's like if MapTrace were a mass spectrum, but
               // in fact m/z is replaced with dt.
 
               // qDebug() << "Before combination, map trace map size: "
@@ -569,16 +588,16 @@ QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::integrate(
             {
 
               // No map<double, MapTrace> item was found with double equals to
-              // first_key. So we need to create a MapTrace ex nihilo.
+              // dt_key. So we need to create a MapTrace ex nihilo.
 
               pappso::MapTrace map_trace;
 
               combiner.combine(map_trace, current_maptrace);
 
               // Finally, store in the map, the map_trace along with its
-              // first_key.
+              // dt_key.
 
-              (*m_doubleMapTraceMapSPtr)[first_key] = map_trace;
+              (*m_rtMapTraceMapSPtr)[dt_key] = map_trace;
             }
         }
     }
@@ -586,19 +605,19 @@ QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::integrate(
   // We now need to go through the double map trace map to inspect its
   // characteristics.
 
-  if(!m_doubleMapTraceMapSPtr->size())
+  if(!m_rtMapTraceMapSPtr->size())
     {
       qDebug() << "There are no data in the double maptrace map.";
 
       return;
     }
 
-  m_colorMapMinKey = m_doubleMapTraceMapSPtr->begin()->first;
-  m_colorMapMaxKey = m_doubleMapTraceMapSPtr->rbegin()->first;
+  m_colorMapMinKey = m_rtMapTraceMapSPtr->begin()->first;
+  m_colorMapMaxKey = m_rtMapTraceMapSPtr->rbegin()->first;
 
-  m_colorMapKeyCellCount = m_doubleMapTraceMapSPtr->size();
+  m_colorMapKeyCellCount = m_rtMapTraceMapSPtr->size();
 
-  for(auto &&pair : *m_doubleMapTraceMapSPtr)
+  for(auto &&pair : *m_rtMapTraceMapSPtr)
     {
       pappso::MapTrace map_trace = pair.second;
 
@@ -620,18 +639,10 @@ QualifiedMassSpectrumVectorMassDataIntegratorToRtDt::integrate(
 
   QString chrono_string;
 
-  if(m_lastIntegrationDataKind == pappso::DataKind::rt)
-    chrono_string = pappso::Utils::chronoIntervalDebugString(
-      "Integration to RT / DT colormap took:",
-      chrono_start_time,
-      chrono_end_time);
-  else if(m_lastIntegrationDataKind == pappso::DataKind::dt)
-    chrono_string = pappso::Utils::chronoIntervalDebugString(
-      "Integration to DT / RT colormap took:",
-      chrono_start_time,
-      chrono_end_time);
-  else
-    qFatal("Programming error.");
+  chrono_string = pappso::Utils::chronoIntervalDebugString(
+    "Integration to RT / DT colormap took:",
+    chrono_start_time,
+    chrono_end_time);
 
   emit logTextToConsoleSignal(chrono_string);
 
