@@ -94,27 +94,38 @@ class QualifiedMassSpectrumVectorMassDataIntegratorToRtDt
   double getColorMapMaxMz() const;
 
   using Map = std::map<double, pappso::MapTrace>;
-  const std::shared_ptr<Map> &getDoubleMapTraceMapSPtr() const;
+  const std::shared_ptr<Map> &getRtMapTraceMapSPtr() const;
 
 
   public slots:
 
-  void integrate(pappso::DataKind data_kind);
+  void integrate();
 
 
   signals:
 
   protected:
+  // m_colorMapKeyCellCount: the number of rt values in the m_rtMapTraceMapSPtr
+  // map.
   std::size_t m_colorMapKeyCellCount = 0;
+
+  // m_colorMapMzCellCount: the maximum number of m/z data points
   std::size_t m_colorMapMzCellCount  = 0;
 
+  // The m_colorMapMinKey is the smallest rt value
   double m_colorMapMinKey = std::numeric_limits<double>::max();
+  // The m_colorMapMinKey is the greatest rt value
   double m_colorMapMaxKey = std::numeric_limits<double>::min();
 
+  // m_colorMapMinMz: the smallest m/z value of all the data
   double m_colorMapMinMz = std::numeric_limits<double>::max();
+
+  // m_colorMapMinMz: the greatest m/z value of all the data
   double m_colorMapMaxMz = std::numeric_limits<double>::min();
 
-  std::shared_ptr<Map> m_doubleMapTraceMapSPtr = nullptr;
+  // The map relating rt value with MapTrace (that in turn relate dt values
+  // with TICint).
+  std::shared_ptr<Map> m_rtMapTraceMapSPtr = nullptr;
 
   pappso::DataKind m_lastIntegrationDataKind;
 };
