@@ -34,6 +34,8 @@
 /////////////////////// StdLib includes
 #include <iostream>
 #include <iomanip>
+#include <cmath>
+
 
 /////////////////////// OpenMP include
 #include <omp.h>
@@ -48,7 +50,6 @@
 
 /////////////////////// Local includes
 #include "QualifiedMassSpectrumVectorMassDataIntegratorToTicInt.hpp"
-#include "ProcessingSpec.hpp"
 #include "ProcessingStep.hpp"
 #include "ProcessingType.hpp"
 
@@ -118,7 +119,7 @@ QualifiedMassSpectrumVectorMassDataIntegratorToTicInt::getTicIntensity() const
 
 
 void
-QualifiedMassSpectrumVectorMassDataIntegratorToTicInt::integrateToTicIntensity()
+QualifiedMassSpectrumVectorMassDataIntegratorToTicInt::integrate()
 {
   qDebug();
 
@@ -153,24 +154,39 @@ QualifiedMassSpectrumVectorMassDataIntegratorToTicInt::integrateToTicIntensity()
   const ProcessingStep *most_recent_processing_step_p =
     m_processingFlow.mostRecentStep();
 
-  // There must be at least one ProcessingSpec (and thus a type) in the
-  // processing step.
+  if(!most_recent_processing_step_p->getDestProcessingType().bitMatches("INT"))
+    qFatal("There should be one ProcessingType = INT. Program aborted.");
 
-  if(!most_recent_processing_step_p->processingTypes().size())
-    qFatal("The processing step cannot be empty. Program aborted.");
+  // When constructing this integrator, the processing flow passed as parameter
+  // was used to gather innermost ranges data. When the steps that match
+  // the innermost criterion, these are stored in m_innermostSteps2D. When the
+  // steps are non-2D steps, then only the m_xxRange values are updated.
 
-  // We are integrating to m/z so we need a spec description that matches
-  // "ANY_TO_MZ".
+  // If the m_innermostSteps2D vector of ProcessingSteps is non-empty, then,
+  // that means that we may have some MZ-related 2D processing steps to account
+  // at combination time, because that is precisely the only moment that we can
+  // do that work. To be able to provide selection polygon an mass
+  // spectrum-specific dt|rt data, we have the SelectionPolygonSpec class. There
+  // can be as many such class instances as required to document the integration
+  // boundaries rt/mz and dt/mz for example. This is why we need to fill in the
+  // vector of SelectionPolygonSpec and then set that to the combiner.
 
-  if(!most_recent_processing_step_p->matches("ANY_TO_INT"))
-    qFatal("There should be one ProcessingType = ANY_TO_INT. Program aborted.");
+  // Note, however, that the hassle described above only is acceptable when the
+  // selection polygon is not square. If the selection polygon is square, then
+  // the ranges suffice, and they have been filled-in properly upon construction
+  // of this integrator.
 
-  std::pair<double, double> innermost_mz_range =
-    most_recent_processing_step_p->innermostRangeProcessingSpecsMatching(
-      ProcessingType("ANY_TO_INT"));
+  qDebug() << "Now filling-in the selection_polygon_specs for any 2D step that "
+              "has a source type involving the MZ data kind.";
 
-  qDebug() << "Innermost ANY_TO_INT range:" << innermost_mz_range.first << "--"
-           << innermost_mz_range.second;
+  std::vector<pappso::SelectionPolygonSpec> selection_polygon_specs;
+
+  std::size_t selection_polygon_spec_count =
+    fillInSelectionPolygonSpecs(selection_polygon_specs);
+
+  qDebug() << selection_polygon_spec_count
+           << "selection polygon specs were retained for the configuration of "
+              "the filter.";
 
   // Determine the best number of threads to use and how many spectra to provide
   // each of the threads.
@@ -201,6 +217,8 @@ QualifiedMassSpectrumVectorMassDataIntegratorToTicInt::integrateToTicIntensity()
   // Now that we have all the parallel material, we can start the parallel work.
   emit setProgressBarMaxValueSignal(mass_spectra_count);
 
+  qDebug() << "Starting the iteration in the visitors.";
+
   omp_set_num_threads(iterators.size());
 #pragma omp parallel for ordered
   for(std::size_t iter = 0; iter < iterators.size(); ++iter)
@@ -246,6 +264,11 @@ QualifiedMassSpectrumVectorMassDataIntegratorToTicInt::integrateToTicIntensity()
           qualified_mass_spectrum_csp =
             checkQualifiedMassSpectrum(qualified_mass_spectrum_csp);
 
+          if(qualified_mass_spectrum_csp == nullptr)
+            qFatal(
+              "Programming error. Not possible that shared pointer to "
+              "qualified mass spectrum be nullptr.");
+
           // Great, at this point we actually have the mass spectrum!
 
           // We now need to ensure that the processing flow allows for this mass
@@ -255,56 +278,87 @@ QualifiedMassSpectrumVectorMassDataIntegratorToTicInt::integrateToTicIntensity()
           // that matches the greatest MS level found in the member processing
           // flow instance.
 
-          if(!checkMsLevel(qualified_mass_spectrum_csp) ||
-             !checkRtRange(qualified_mass_spectrum_csp) ||
-             !checkDtRange(qualified_mass_spectrum_csp))
+          if(!checkMsLevel(qualified_mass_spectrum_csp))
             {
-              // if(!checkMsLevel(qualified_mass_spectrum_csp))
-              // qDebug() << "MS level did not check.";
+              qDebug() << "The MsLevel check returned false";
+              ++current_iterator;
+              continue;
+            }
+          if(!checkMsFragmentation(qualified_mass_spectrum_csp))
+            {
+              qDebug() << "The MsFragmentation check returned false";
+              ++current_iterator;
+              continue;
+            }
 
-              // if(!checkRtRange(qualified_mass_spectrum_csp))
-              // qDebug() << "RT range did not check.";
+          // Make easy check about RT and DT.
 
-              // if(!checkDtRange(qualified_mass_spectrum_csp))
-              // qDebug() << "DT range did not check.";
+          if(!checkRtRange(qualified_mass_spectrum_csp))
+            {
+              qDebug() << "checkRtRange failed check.";
 
-              // qDebug() << "The checks returned false";
+              ++current_iterator;
+              continue;
+            }
+          if(!checkDtRange(qualified_mass_spectrum_csp))
+            {
+              qDebug() << "checkDtRange failed check.";
 
               ++current_iterator;
               continue;
             }
 
-          // At this point we need to go deep in the processing flow's steps to
-          // check if the node matches the whole set of the steps' specs.
+          // At this point we need to account for any limitations in the m/z
+          // dimension of the data. These limitations might be as easy as having
+          // a single m/z range limit or as complex as having one or more 2D
+          // steps involving the m/z dimension in the m_innermostSteps2D, if the
+          // selection polygons are not rectangle.
 
-          // qDebug() << "The processing flow has" << m_processingFlow.size() <<
-          // "steps.";
-
-          // for(auto &&step : m_processingFlow)
-          //{
-          // if(!checkProcessingStep(*step, qualified_mass_spectrum_csp))
-          //{
-          //++current_iterator;
-          // continue;
-          //}
-          //}
-
-          // Note that we need to check if we are limited in the mz range that
-          // needs to be accounted for in the TIC sum calculation.
-
-          // These values are set in the in the base class constuctor.
-
-          if(m_mzRange.first != std::numeric_limits<double>::max() &&
-             m_mzRange.second != std::numeric_limits<double>::min())
+          if(!selection_polygon_specs.size())
             {
-              *current_tic_intensity_sp +=
-                qualified_mass_spectrum_csp->getMassSpectrumSPtr()->sumY(
-                  m_mzRange.first, m_mzRange.second);
+              if(!std::isnan(m_mzRange.first) && !std::isnan(m_mzRange.second))
+                {
+                  *current_tic_intensity_sp +=
+                    qualified_mass_spectrum_csp->getMassSpectrumSPtr()->sumY(
+                      m_mzRange.first, m_mzRange.second);
+                }
+              else
+                {
+                  *current_tic_intensity_sp +=
+                    qualified_mass_spectrum_csp->getMassSpectrumSPtr()->sumY();
+                }
             }
           else
             {
-              *current_tic_intensity_sp +=
-                qualified_mass_spectrum_csp->getMassSpectrumSPtr()->sumY();
+              // It may be possible that the 2D steps limiting the m/z range
+              // have been followed by 1D steps further limiting the m/Z range.
+              // The point is, the 1D steps are not stored in
+              // selection_polygon_specs. However, by necessity, if there is at
+              // least one selection_polygon_spec, then the m_mzRange holds
+              // values. Futher, if there had been a 1D step limiting even more
+              // the m/z range, then that step would have been accounted for in
+              // m_mzRange. So we can account that limit right away.
+
+              if(std::isnan(m_mzRange.first) || std::isnan(m_mzRange.second))
+                qFatal("Programming error.");
+
+              pappso::Trace filtered_trace(
+                *qualified_mass_spectrum_csp->getMassSpectrumSPtr());
+
+              pappso::FilterResampleKeepXRange the_keep_filter(
+                m_mzRange.first, m_mzRange.second);
+
+              filtered_trace = the_keep_filter.filter(filtered_trace);
+
+              pappso::FilterResampleKeepPointInPolygon the_polygon_filter(
+                selection_polygon_specs);
+
+              filtered_trace = the_polygon_filter.filter(
+                filtered_trace,
+                qualified_mass_spectrum_csp->getDtInMilliSeconds(),
+                qualified_mass_spectrum_csp->getRtInMinutes());
+
+              *current_tic_intensity_sp += filtered_trace.sumY();
             }
 
           emit incrementProgressBarCurrentValueAndSetStatusTextSignal(
