@@ -539,7 +539,7 @@ RtDtColorMapTreeNodeCombinerVisitor::checkProcessingTypeByMz(
 
   const ProcessingSpec *spec_p = type_spec_pair.second;
 
-  if(spec_p->hasValidRange())
+  if(spec_p->hasValidOneDimensionRange())
     {
       qDebug() << "Needed a m/z range filtering:" << spec_p->toString();
 
