@@ -9,7 +9,7 @@
 #include <pappsomspp/processing/combiners/massdatacombinerinterface.h>
 #include <pappsomspp/processing/combiners/massspectrumpluscombiner.h>
 
-#include "../../src/nongui/MzIntegrationParams.hpp"
+#include <pappsomspp/processing/combiners/mzintegrationparams.h>
 
 
 using namespace pappso;
